@@ -3,7 +3,7 @@
  * Copyright (C) 2005       Thomas Denk
  * Copyright (C) 2008       Paul Wise
  * Copyright (C) 2007-2008  Giel van Schijndel
- * Copyright (C) 2007-2008  Warzone Resurrection Project
+ * Copyright (C) 2007-2009  Warzone Resurrection Project
  *
  * This program is distributed under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or (at your option) any later version.
@@ -13,9 +13,9 @@
  * Id: autorevision.cpp 3543 2007-01-27 00:25:43Z daniel2000
  * HeadURL: http://svn.berlios.de/svnroot/repos/codeblocks/trunk/src/build_tools/autorevision/autorevision.cpp
  *
- * $Revision: 6709 $
- * $Id: autorevision.cpp 6709 2009-02-22 23:49:34Z muggenhor $
- * $HeadURL: svn+ssh://per@svn.gna.org/svn/warzone/tags/2.1.3/build_tools/autorevision/autorevision.cpp $
+ * $Revision: 6708 $
+ * $Id: autorevision.cpp 6708 2009-02-22 23:48:02Z muggenhor $
+ * $HeadURL: https://warzone2100.svn.sourceforge.net/svnroot/warzone2100/tags/2.2.2/build_tools/autorevision/autorevision.cpp $
  */
 
 #include <iostream>
@@ -24,6 +24,7 @@
 #include <sstream>
 
 #include <cstring>
+#include <cstdlib>
 
 using namespace std;
 
@@ -302,7 +303,7 @@ int main(int argc, char** argv)
     return 0;
 }
 
-bool removeAfterNewLine(string& str)
+static bool removeAfterNewLine(string& str)
 {
     // Find a newline and erase everything that comes after it
     string::size_type lbreak_pos = str.find_first_of("\r\n");
