@@ -1,7 +1,7 @@
 /*
 	This file is part of Warzone 2100.
 	Copyright (C) 1999-2004  Eidos Interactive
-	Copyright (C) 2005-2007  Warzone Resurrection Project
+	Copyright (C) 2005-2009  Warzone Resurrection Project
 
 	Warzone 2100 is free software; you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -22,7 +22,7 @@
 #include "lib/framework/frame.h"
 #include <stdlib.h>
 #include <string.h>
-#include "lib/framework/strnlen1.h"
+#include "lib/framework/string_ext.h"
 #include "lib/ivis_common/ivisdef.h"
 #include "lib/ivis_common/piestate.h"
 #include "lib/ivis_common/rendmode.h"
@@ -39,15 +39,9 @@
 # include <GL/glc.h>
 #endif
 
-#ifdef WZ_OS_MAC
-static const char font_family[] = "Lucida Grande";
-static const char font_face_regular[] = "Regular";
-static const char font_face_bold[] = "Bold";
-#else
-static const char font_family[] = "DejaVu Sans Mono";
-static const char font_face_regular[] = "Book";
-static const char font_face_bold[] = "Bold";
-#endif
+static char font_family[128];
+static char font_face_regular[128];
+static char font_face_bold[128];
 
 static float font_size = 12.f;
 // Contains the font color in the following order: red, green, blue, alpha
@@ -63,6 +57,27 @@ static GLint _glcFont_Bold = 0;
  */
 /***************************************************************************/
 
+void iV_font(const char *fontName, const char *fontFace, const char *fontFaceBold)
+{
+	if (_glcContext)
+	{
+		debug(LOG_ERROR, "Cannot change font in running game, yet.");
+		return;
+	}
+	if (fontName)
+	{
+		sstrcpy(font_family, fontName);
+	}
+	if (fontFace)
+	{
+		sstrcpy(font_face_regular, fontFace);
+	}
+	if (fontFaceBold)
+	{
+		sstrcpy(font_face_bold, fontFaceBold);
+	}
+}
+
 static inline void iV_printFontList(void)
 {
 	unsigned int i;
@@ -71,7 +86,7 @@ static inline void iV_printFontList(void)
 
 	if (font_count == 0)
 	{
-		debug(LOG_ERROR, "iV_printFontList: The required font (%s) isn't loaded", font_family);
+		debug(LOG_ERROR, "The required font (%s) isn't loaded", font_family);
 
 		// Fall back to unselected fonts since the requested font apparently
 		// isn't available.
@@ -103,11 +118,11 @@ static void iV_initializeGLC(void)
 	_glcContext = glcGenContext();
 	if (!_glcContext)
 	{
-		debug(LOG_ERROR, "glcGenContext() failed");
+		debug(LOG_ERROR, "Failed to initialize");
 	}
 	else
 	{
-		debug(LOG_NEVER, "glcGenContext() succesful: _glcContext = %d", _glcContext);
+		debug(LOG_NEVER, "Succesfully initialized. _glcContext = %d", _glcContext);
 	}
 
 	glcContext(_glcContext);
@@ -121,20 +136,20 @@ static void iV_initializeGLC(void)
 
 	if (!glcNewFontFromFamily(_glcFont_Regular, font_family))
 	{
-		debug(LOG_ERROR, "iV_initializeGLC: Failed to select font family %s as regular font", font_family);
+		debug(LOG_ERROR, "Failed to select font family %s as regular font", font_family);
 	}
 	else
 	{
-		debug(LOG_NEVER, "iV_initializeGLC: Successfully selected font family %s as regular font", font_family);
+		debug(LOG_NEVER, "Successfully selected font family %s as regular font", font_family);
 	}
 
 	if (!glcFontFace(_glcFont_Regular, font_face_regular))
 	{
-		debug(LOG_WARNING, "iV_initializeGLC: Failed to select the \"%s\" font face of font family %s", font_face_regular, font_family);
+		debug(LOG_WARNING, "Failed to select the \"%s\" font face of font family %s", font_face_regular, font_family);
 	}
 	else
 	{
-		debug(LOG_NEVER, "iV_initializeGLC: Successfully selected the \"%s\" font face of font family %s", font_face_regular, font_family);
+		debug(LOG_NEVER, "Successfully selected the \"%s\" font face of font family %s", font_face_regular, font_family);
 	}
 
 	if (!glcNewFontFromFamily(_glcFont_Bold, font_family))
@@ -143,19 +158,19 @@ static void iV_initializeGLC(void)
 	}
 	else
 	{
-		debug(LOG_NEVER, "iV_initializeGLC: Successfully selected font family %s for the bold font", font_family);
+		debug(LOG_NEVER, "Successfully selected font family %s for the bold font", font_family);
 	}
 
 	if (!glcFontFace(_glcFont_Bold, font_face_bold))
 	{
-		debug(LOG_WARNING, "iV_initializeGLC: Failed to select the \"%s\" font face of font family %s", font_face_bold, font_family);
+		debug(LOG_WARNING, "Failed to select the \"%s\" font face of font family %s", font_face_bold, font_family);
 	}
 	else
 	{
-		debug(LOG_NEVER, "iV_initializeGLC: Successfully selected the \"%s\" font face of font family %s", font_face_bold, font_family);
+		debug(LOG_NEVER, "Successfully selected the \"%s\" font face of font family %s", font_face_bold, font_family);
 	}
 
-	debug(LOG_NEVER, "finished initializing GLC");
+	debug(LOG_NEVER, "Finished initializing GLC");
 }
 
 void iV_TextInit()
@@ -262,7 +277,7 @@ unsigned int iV_GetTextWidth(const char* string)
 	glcMeasureString(GL_FALSE, string);
 	if (!glcGetStringMetric(GLC_BOUNDS, boundingbox))
 	{
-		debug(LOG_ERROR, "iV_GetTextWidth: couldn't retrieve a bounding box for the string");
+		debug(LOG_ERROR, "Couldn't retrieve a bounding box for the string \"%s\"", string);
 		return 0;
 	}
 
@@ -279,7 +294,8 @@ unsigned int iV_GetCountedTextWidth(const char* string, size_t string_length)
 	glcMeasureCountedString(GL_FALSE, string_length, string);
 	if (!glcGetStringMetric(GLC_BOUNDS, boundingbox))
 	{
-		debug(LOG_ERROR, "iV_GetCountedTextWidth: couldn't retrieve a bounding box for the string");
+		debug(LOG_ERROR, "Couldn't retrieve a bounding box for the string \"%s\" of length %u", 
+		      string, (unsigned int)string_length);
 		return 0;
 	}
 
@@ -296,7 +312,7 @@ unsigned int iV_GetTextHeight(const char* string)
 	glcMeasureString(GL_FALSE, string);
 	if (!glcGetStringMetric(GLC_BOUNDS, boundingbox))
 	{
-		debug(LOG_ERROR, "iV_GetTextHeight: couldn't retrieve a bounding box for the string");
+		debug(LOG_ERROR, "Couldn't retrieve a bounding box for the string \"%s\"", string);
 		return 0;
 	}
 
@@ -312,7 +328,7 @@ unsigned int iV_GetCharWidth(uint32_t charCode)
 
 	if (!glcGetCharMetric(charCode, GLC_BOUNDS, boundingbox))
 	{
-		debug(LOG_ERROR, "iV_GetCharWidth: couldn't retrieve a bounding box for the character");
+		debug(LOG_ERROR, "Couldn't retrieve a bounding box for the character code %u", charCode);
 		return 0;
 	}
 
@@ -328,7 +344,7 @@ int iV_GetTextLineSize()
 
 	if (!glcGetMaxCharMetric(GLC_BOUNDS, boundingbox))
 	{
-		debug(LOG_ERROR, "iV_GetTextLineSize: couldn't retrieve a bounding box for the character");
+		debug(LOG_ERROR, "Couldn't retrieve a bounding box for the character");
 		return 0;
 	}
 
@@ -343,7 +359,7 @@ static float iV_GetMaxCharBaseY(void)
 
 	if (!glcGetMaxCharMetric(GLC_BASELINE, base_line))
 	{
-		debug(LOG_ERROR, "iV_GetMaxCharBaseY: couldn't retrieve the baseline for the character");
+		debug(LOG_ERROR, "Couldn't retrieve the baseline for the character");
 		return 0;
 	}
 
@@ -359,7 +375,7 @@ int iV_GetTextAboveBase(void)
 
 	if (!glcGetMaxCharMetric(GLC_BOUNDS, boundingbox))
 	{
-		debug(LOG_ERROR, "iV_GetTextAboveBase: couldn't retrieve a bounding box for the character");
+		debug(LOG_ERROR, "Couldn't retrieve a bounding box for the character");
 		return 0;
 	}
 
@@ -378,7 +394,7 @@ int iV_GetTextBelowBase(void)
 
 	if (!glcGetMaxCharMetric(GLC_BOUNDS, boundingbox))
 	{
-		debug(LOG_ERROR, "iV_GetTextBelowBase: couldn't retrieve a bounding box for the character");
+		debug(LOG_ERROR, "Couldn't retrieve a bounding box for the character");
 		return 0;
 	}
 
@@ -458,7 +474,8 @@ int iV_DrawFormattedText(const char* String, UDWORD x, UDWORD y, UDWORD Width, U
 			i = 0;
 			for (; *curChar != 0
 			    && *curChar != ASCII_SPACE
-			    && *curChar != ASCII_NEWLINE;
+			    && *curChar != ASCII_NEWLINE
+			    && *curChar != '\n';
 			     ++i, ++curChar)
 			{
 				if (*curChar == ASCII_COLOURMODE) // If it's a colour mode toggle char then just add it to the word.
@@ -497,7 +514,8 @@ int iV_DrawFormattedText(const char* String, UDWORD x, UDWORD y, UDWORD Width, U
 				}
 			}
 			// Check for new line character.
-			else if (*curChar == ASCII_NEWLINE)
+			else if (*curChar == ASCII_NEWLINE
+			      || *curChar == '\n')
 			{
 				NewLine = true;
 				++curChar;
