@@ -1,6 +1,6 @@
 /*
 	This file is part of Warzone 2100.
-	Copyright (C) 2005-2007  Warzone Resurrection Project
+	Copyright (C) 2008-2009  Warzone Resurrection Project
 
 	Warzone 2100 is free software; you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -17,14 +17,16 @@
 	Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 */
 
-#ifndef _sequence_h
-#define _sequence_h
+#ifndef __INCLUDED_LIB_SEQUENCE_SEQUENCE_H__
+#define __INCLUDED_LIB_SEQUENCE_SEQUENCE_H__
 
 #include "lib/framework/types.h"
 
-extern BOOL seq_Play(char *filename);
-extern BOOL seq_Playing(void);
-extern BOOL seq_Update(void);
+extern bool seq_Play(const char* filename);
+extern bool seq_Playing(void);
+extern bool seq_Update(void);
 extern void seq_Shutdown(void);
+extern int seq_GetFrameNumber(void);
+extern void seq_SetDisplaySize(int sizeX, int sizeY, int posX, int posY);
 
-#endif
+#endif // __INCLUDED_LIB_SEQUENCE_SEQUENCE_H__
