@@ -1,7 +1,7 @@
 /*
 	This file is part of Warzone 2100.
 	Copyright (C) 1999-2004  Eidos Interactive
-	Copyright (C) 2005-2007  Warzone Resurrection Project
+	Copyright (C) 2005-2009  Warzone Resurrection Project
 
 	Warzone 2100 is free software; you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -252,7 +252,7 @@ static AUDIO_ID_MAP asAudioID[] =
 {ID_CLAN_ENTER,                                "pcv480.ogg"},
 {ID_CLAN_EXIT,                                 "pcv481.ogg"},
 {ID_GIFT,                                      "pcv482.ogg"},
-{ID_POWER_TRANSMIT,                            "pcv483.ogg"},
+{ID_POWER_TRANSMIT,                            "power-transferred.ogg"},
 {ID_SENSOR_DOWNLOAD,                           "pcv484.ogg"},
 {ID_TECHNOLOGY_TRANSFER,                       "pcv485.ogg"},
 {ID_UNITS_TRANSFER,                            "pcv486.ogg"},
