@@ -1,7 +1,7 @@
 /*
 	This file is part of Warzone 2100.
 	Copyright (C) 1999-2004  Eidos Interactive
-	Copyright (C) 2005-2007  Warzone Resurrection Project
+	Copyright (C) 2005-2009  Warzone Resurrection Project
 
 	Warzone 2100 is free software; you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -26,19 +26,15 @@ extern "C"
 {
 #endif
 
-enum
+typedef enum
 {
-	playlist_custom,
-	playlist_ingame,
-	playlist_frontend,
-
-	// Must be the last
-	playlist_last,
-};
+	SONG_FRONTEND,
+	SONG_INGAME,
+} SONG_CONTEXT;
 
 BOOL cdAudio_Open(const char* user_musicdir);
 void cdAudio_Close(void);
-BOOL cdAudio_PlayTrack( SDWORD iTrack );
+BOOL cdAudio_PlayTrack(SONG_CONTEXT context);
 void cdAudio_Stop(void);
 void cdAudio_Pause(void);
 void cdAudio_Resume(void);
