@@ -1,7 +1,7 @@
 /*
 	This file is part of Warzone 2100.
 	Copyright (C) 1999-2004  Eidos Interactive
-	Copyright (C) 2005-2007  Warzone Resurrection Project
+	Copyright (C) 2005-2009  Warzone Resurrection Project
 
 	Warzone 2100 is free software; you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -23,8 +23,9 @@
 
 // this has to be first
 #include "lib/framework/frame.h"
-#include "lib/framework/math-help.h"
+#include "lib/framework/math_ext.h"
 #include "lib/framework/frameresource.h"
+#include "lib/exceptionhandler/dumpinfo.h"
 
 #ifndef WZ_NOSOUND
 # ifdef WZ_OS_MAC
@@ -47,12 +48,13 @@
 #include "openal_error.h"
 #include "mixer.h"
 
-#define ATTENUATION_FACTOR	0.0003f
 
 #ifndef WZ_NOSOUND
 ALuint current_queue_sample = -1;
 #endif
 
+static BOOL openal_initialized = false;
+
 struct __audio_stream
 {
 #ifndef WZ_NOSOUND
@@ -86,11 +88,10 @@ static AUDIO_STREAM* active_streams = NULL;
 static ALfloat		sfx_volume = 1.0;
 static ALfloat		sfx3d_volume = 1.0;
 
-static ALCdevice* device = 0;
-static ALCcontext* context = 0;
+static ALCdevice* device = NULL;
+static ALCcontext* context = NULL;
 #endif
 
-BOOL openal_initialized = false;
 
 /** Removes the given sample from the "active_samples" linked list
  *  \param previous either NULL (if \c to_remove is the first item in the
@@ -121,6 +122,7 @@ static void sound_RemoveSample(SAMPLE_LIST* previous, SAMPLE_LIST* to_remove)
 static void PrintOpenALVersion(code_part part)
 {
 	const ALchar* pDeviceNames = NULL;
+	char buf[256];
 
 	debug(part, "OpenAL Vendor: %s", alGetString(AL_VENDOR));
 	debug(part, "OpenAL Version: %s", alGetString(AL_VERSION));
@@ -139,6 +141,13 @@ static void PrintOpenALVersion(code_part part)
 		pDeviceNames += strlen(pDeviceNames) + 1;
 	}
 
+	// Copy this info to be used by the crash handler for the dump file
+	ssprintf(buf, "OpenAL Vendor: %s", alGetString(AL_VENDOR));
+	addDumpInfo(buf);
+	ssprintf(buf, "OpenAL Version: %s", alGetString(AL_VERSION));
+	addDumpInfo(buf);
+	ssprintf(buf, "OpenAL Renderer: %s", alGetString(AL_RENDERER));
+	addDumpInfo(buf);
 }
 #endif
 
@@ -150,8 +159,13 @@ BOOL sound_InitLibrary( void )
 {
 #ifndef WZ_NOSOUND
 	int err;
-	ALfloat listenerVel[3] = { 0.0, 0.0, 0.0 };
-	ALfloat listenerOri[6] = { 0.0, 0.0, 1.0, 0.0, 1.0, 0.0 };
+	const ALfloat listenerVel[3] = { 0.0, 0.0, 0.0 };
+	const ALfloat listenerOri[6] = { 0.0, 0.0, 1.0, 0.0, 1.0, 0.0 };
+	ALCint attrSize;
+	ALCint *temp;
+	ALCint *attributes;
+	ALCint *data;
+	char buf[256];
 
 #ifdef WZ_OS_WIN
 	/* HACK: Select the "software" OpenAL device on Windows because it
@@ -168,7 +182,7 @@ BOOL sound_InitLibrary( void )
 #else
 	device = alcOpenDevice(0);
 #endif
-	if(device == 0)
+	if(!device)
 	{
 		PrintOpenALVersion(LOG_ERROR);
 		debug(LOG_ERROR, "Couldn't open audio device.");
@@ -176,6 +190,12 @@ BOOL sound_InitLibrary( void )
 	}
 
 	context = alcCreateContext(device, NULL);		//NULL was contextAttributes
+	if (!context)
+	{
+		PrintOpenALVersion(LOG_ERROR);
+		debug(LOG_ERROR, "Couldn't open audio context.");
+		return false;
+	}
 	alcMakeContextCurrent(context);
 
 	err = sound_GetDeviceError(device);
@@ -185,6 +205,48 @@ BOOL sound_InitLibrary( void )
 		debug(LOG_ERROR, "Couldn't initialize audio context: %s", alcGetString(device, err));
 		return false;
 	}
+	// getting more info about openAL
+	alcGetIntegerv(device, ALC_ATTRIBUTES_SIZE, sizeof(attrSize),&attrSize);
+	attributes = (ALCint *)malloc(attrSize * sizeof(ALCint));
+	alcGetIntegerv(device, ALC_ALL_ATTRIBUTES, attrSize, attributes);
+	data = attributes;
+	temp = attributes + attrSize ;
+	while (data < temp)
+	{
+		switch (*data)
+		{
+			case ALC_FREQUENCY:
+				data += 1;
+				ssprintf(buf, "ALC_FREQUENCY = %d", *data);
+				addDumpInfo(buf);
+				break;
+			case ALC_REFRESH:
+				data += 1;
+				ssprintf(buf, "ALC_REFRESH = %d", *data);
+				addDumpInfo(buf);
+				break;
+			case ALC_SYNC:
+				data += 1;
+				ssprintf(buf, "ALC_SYNC = %d", *data);
+				addDumpInfo(buf);
+				break;
+			case ALC_MONO_SOURCES:
+				data += 1;
+				ssprintf(buf, "ALC_MONO_SOURCES = %d", *data);
+				addDumpInfo(buf);
+				break;
+			case ALC_STEREO_SOURCES:
+				data += 1;
+				ssprintf(buf, "ALC_STEREO_SOURCES = %d", *data);
+				addDumpInfo(buf);
+				break;
+			default:
+				break;
+		}
+		data += 1;
+	}
+	free(attributes);
+
 #endif
 
 	openal_initialized = true;
@@ -206,32 +268,40 @@ BOOL sound_InitLibrary( void )
 	return true;
 }
 
-//*
-// =======================================================================================================================
-// =======================================================================================================================
-//
+#if !defined(WZ_NOSOUND)
+static void sound_UpdateStreams(void);
+#endif
+
 void sound_ShutdownLibrary( void )
 {
+#if !defined(WZ_NOSOUND)
+	AUDIO_STREAM* stream;
+#endif
 	SAMPLE_LIST * aSample = active_samples, * tmpSample = NULL;
 
-	debug(LOG_SOUND, "sound_ShutdownLibrary: starting shutdown");
-#ifndef WZ_NOSOUND
-	if(context != 0) {
-#ifdef WIN32
-		/* Ifdef'ed out the two lines below on Linux since this caused some versions
-		 * of OpenAL to hang on exit. - Per */
-		debug(LOG_SOUND, "sound_ShutdownLibrary: make default context NULL");
-		alcMakeContextCurrent(NULL);		//this should work now -Q
-#endif
-		debug(LOG_SOUND, "sound_ShutdownLibrary: destroy previous context");
-		alcDestroyContext(context); // this gives a long delay on some impl.
-		context = 0;
+	if ( !openal_initialized )
+	{
+		return;
 	}
-	debug(LOG_SOUND, "sound_ShutdownLibrary: close device");
-	if(device != 0) {
-		alcCloseDevice(device);
-		device = 0;
+	debug(LOG_SOUND, "starting shutdown");
+
+#if !defined(WZ_NOSOUND)
+	// Stop all streams, sound_UpdateStreams() will deallocate all stopped streams
+	for (stream = active_streams; stream != NULL; stream = stream->next)
+	{
+		sound_StopStream(stream);
 	}
+	sound_UpdateStreams();
+
+	/* On Linux since this caused some versions of OpenAL to hang on exit. - Per */
+	debug(LOG_SOUND, "make default context NULL");
+	alcMakeContextCurrent(NULL);
+
+	debug(LOG_SOUND, "destroy previous context");
+	alcDestroyContext(context); // this gives a long delay on some impl.
+
+	debug(LOG_SOUND, "close device");
+	alcCloseDevice(device);
 #endif
 
 	while( aSample )
@@ -258,6 +328,9 @@ static void sound_DestroyIteratedSample(SAMPLE_LIST** previous, SAMPLE_LIST** sa
 	}
 #endif
 
+	// Do the cleanup of this sample
+	sound_FinishedCallback((*sample)->curr);
+
 	// Remove the sample from the list
 	sound_RemoveSample(*previous, *sample);
 	// Free it
@@ -283,14 +356,18 @@ unsigned int sound_GetActiveSamplesCount()
 	return num;
 }
 
-static void sound_UpdateStreams(void);
-
 void sound_Update()
 {
 #ifndef WZ_NOSOUND
 	SAMPLE_LIST* node = active_samples;
 	SAMPLE_LIST* previous = NULL;
 	ALCenum err;
+	ALfloat gain;
+
+	if ( !openal_initialized )
+	{
+		return;
+	}
 
 	// Update all streaming audio
 	sound_UpdateStreams();
@@ -299,6 +376,18 @@ void sound_Update()
 	{
 		ALenum state, err;
 
+		// query what the gain is for this sample
+		alGetSourcef(node->curr->iSample, AL_GAIN, &gain);
+		err = sound_GetError();
+
+		// if gain is 0, then we can't hear it, so we kill it.
+		if (gain == 0.0f)
+		{
+			sound_DestroyIteratedSample(&previous, &node);
+			continue;
+		}
+
+		//ASSERT(alIsSource(node->curr->iSample), "Not a valid source!");
 		alGetSourcei(node->curr->iSample, AL_SOURCE_STATE, &state);
 
 		// Check whether an error occurred while retrieving the state.
@@ -329,8 +418,8 @@ void sound_Update()
 				node = node->next;
 				break;
 
-			case AL_STOPPED:
-				sound_FinishedCallback(node->curr);
+			// NOTE: if it isn't playing | paused, then it is most likely either done
+			// or a error.  In either case, we want to kill the sample in question.
 
 			default:
 				sound_DestroyIteratedSample(&previous, &node);
@@ -361,6 +450,10 @@ BOOL sound_QueueSamplePlaying( void )
 #ifndef WZ_NOSOUND
 	ALenum	state;
 
+	if ( !openal_initialized )
+	{
+		return false;
+	}
 	if ( current_queue_sample == (ALuint)AL_INVALID )
 	{
 		return false;
@@ -381,8 +474,25 @@ BOOL sound_QueueSamplePlaying( void )
 
 	if (current_queue_sample != (ALuint)AL_INVALID)
 	{
-		alDeleteSources(1, &current_queue_sample);
-		sound_GetError();
+		SAMPLE_LIST* node = active_samples;
+		SAMPLE_LIST* previous = NULL;
+
+		// We need to remove it from the queue of actively played samples
+		while (node != NULL)
+		{
+			if (node->curr->iSample == current_queue_sample)
+			{
+				sound_DestroyIteratedSample(&previous, &node);
+				current_queue_sample = AL_INVALID;
+				return false;
+			}
+			previous = node;
+			if (node)
+			{
+				node = node->next;
+			}
+		}
+		debug(LOG_ERROR, "Sample %u not deleted because it wasn't in the active queue!", current_queue_sample);
 		current_queue_sample = AL_INVALID;
 	}
 #endif
@@ -399,10 +509,14 @@ static inline TRACK* sound_DecodeOggVorbisTrack(TRACK *psTrack, PHYSFS_file* PHY
 #ifndef WZ_NOSOUND
 	ALenum		format;
 	ALuint		buffer;
+	struct OggVorbisDecoderState *decoder;
+	soundDataBuffer	*soundBuffer;
 
-	struct OggVorbisDecoderState* decoder = sound_CreateOggVorbisDecoder(PHYSFS_fileHandle, true);
-	soundDataBuffer* soundBuffer;
-
+	if ( !openal_initialized )
+	{
+		return NULL;
+	}
+	decoder = sound_CreateOggVorbisDecoder(PHYSFS_fileHandle, true);
 	soundBuffer = sound_DecodeOggVorbis(decoder, 0);
 	sound_DestroyOggVorbisDecoder(decoder);
 
@@ -456,6 +570,7 @@ TRACK* sound_LoadTrackFromFile(const char *fileName)
 
 	// Use PhysicsFS to open the file
 	fileHandle = PHYSFS_openRead(fileName);
+	debug(LOG_WZ, "Reading...[directory: %s] %s", PHYSFS_getRealDir(fileName), fileName);
 	if (fileHandle == NULL)
 	{
 		debug(LOG_ERROR, "sound_LoadTrackFromFile: PHYSFS_openRead(\"%s\") failed with error: %s\n", fileName, PHYSFS_getLastError());
@@ -572,6 +687,7 @@ BOOL sound_Play2DSample( TRACK *psTrack, AUDIO_SAMPLE *psSample, BOOL bQueued )
 #ifndef WZ_NOSOUND
 	ALfloat zero[3] = { 0.0, 0.0, 0.0 };
 	ALfloat volume;
+	ALint error;
 
 	if (sfx_volume == 0.0)
 	{
@@ -580,8 +696,28 @@ BOOL sound_Play2DSample( TRACK *psTrack, AUDIO_SAMPLE *psSample, BOOL bQueued )
 	volume = ((float)psTrack->iVol / 100.0f);       // each object can have OWN volume!
 	psSample->fVol = volume;                        // save computed volume
 	volume *= sfx_volume;                           // and now take into account the Users sound Prefs.
+
+	// We can't hear it, so don't bother creating it.
+	if (volume == 0.0f)
+	{
+		return false;
+	}
+
+	// Clear error codes
+	alGetError();
+
 	alGenSources( 1, &(psSample->iSample) );
-	sound_GetError();
+
+	error = sound_GetError();
+	if (error != AL_NO_ERROR)
+	{
+		/* FIXME: We run out of OpenAL sources very quickly, so we
+		 * should handle the case where we've ran out of them.
+		 * Currently we don't do this, causing some unpleasant side
+		 * effects, e.g. crashing...
+		 */
+	}
+
 	alSourcef( psSample->iSample, AL_PITCH, 1.0f );
 	alSourcef( psSample->iSample, AL_GAIN,volume );
 	alSourcefv( psSample->iSample, AL_POSITION, zero );
@@ -589,9 +725,20 @@ BOOL sound_Play2DSample( TRACK *psTrack, AUDIO_SAMPLE *psSample, BOOL bQueued )
 	alSourcei( psSample->iSample, AL_BUFFER, psTrack->iBufferName );
 	alSourcei( psSample->iSample, AL_SOURCE_RELATIVE, AL_TRUE );
 	alSourcei( psSample->iSample, AL_LOOPING, (sound_SetupChannel(psSample)) ? AL_TRUE : AL_FALSE );
-	sound_GetError();
+
+	// NOTE: this is only useful for debugging.
+#ifdef DEBUG
+	psSample->is3d = false;
+	psSample->isLooping = sound_TrackLooped(psSample->iTrack)? AL_TRUE : AL_FALSE;
+	memcpy(psSample->filename,psTrack->fileName, strlen(psTrack->fileName));
+	psSample->filename[strlen(psTrack->fileName)]='\0';
+#endif
+	// Clear error codes
+	alGetError();
+
 	alSourcePlay( psSample->iSample );
 	sound_GetError();
+
 	if ( bQueued )
 	{
 		current_queue_sample = psSample->iSample;
@@ -614,6 +761,7 @@ BOOL sound_Play3DSample( TRACK *psTrack, AUDIO_SAMPLE *psSample )
 #ifndef WZ_NOSOUND
 	ALfloat zero[3] = { 0.0, 0.0, 0.0 };
 	ALfloat volume;
+	ALint error;
 
 	if (sfx3d_volume == 0.0)
 	{
@@ -622,17 +770,47 @@ BOOL sound_Play3DSample( TRACK *psTrack, AUDIO_SAMPLE *psSample )
 
 	volume = ((float)psTrack->iVol / 100.f);        // max range is 0-100
 	psSample->fVol = volume;                        // store results for later
+
+	// If we can't hear it, then don't bother playing it.
+	if (volume == 0.0f)
+	{
+		return false;
+	}
+	// Clear error codes
+	alGetError();
+
 	alGenSources( 1, &(psSample->iSample) );
-	sound_GetError();
+
+	error = sound_GetError();
+	if (error != AL_NO_ERROR)
+	{
+		/* FIXME: We run out of OpenAL sources very quickly, so we
+		 * should handle the case where we've ran out of them.
+		 * Currently we don't do this, causing some unpleasant side
+		 * effects, e.g. crashing...
+		 */
+	}
+
 	// HACK: this is a workaround for a bug in the 64bit implementation of OpenAL on GNU/Linux
 	// The AL_PITCH value really should be 1.0.
-	alSourcef( psSample->iSample, AL_PITCH, 1.001 );
+	alSourcef(psSample->iSample, AL_PITCH, 1.001f);
 
 	sound_SetObjectPosition( psSample );
 	alSourcefv( psSample->iSample, AL_VELOCITY, zero );
 	alSourcei( psSample->iSample, AL_BUFFER, psTrack->iBufferName );
 	alSourcei( psSample->iSample, AL_LOOPING, (sound_SetupChannel(psSample)) ? AL_TRUE : AL_FALSE );
-	sound_GetError();
+
+	// NOTE: this is only useful for debugging.
+#ifdef DEBUG
+	psSample->is3d = true;
+	psSample->isLooping = sound_TrackLooped(psSample->iTrack)? AL_TRUE : AL_FALSE;
+	memcpy(psSample->filename,psTrack->fileName, strlen(psTrack->fileName));
+	psSample->filename[strlen(psTrack->fileName)]='\0';
+#endif
+
+	// Clear error codes
+	alGetError();
+
 	alSourcePlay( psSample->iSample );
 	sound_GetError();
 #endif
@@ -676,10 +854,10 @@ AUDIO_STREAM* sound_PlayStreamWithBuf(PHYSFS_file* fileHandle, float volume, voi
 #if !defined(WZ_NOSOUND)
 	AUDIO_STREAM* stream;
 	ALuint*       buffers = alloca(sizeof(ALuint) * buffer_count);
-
+	ALint error;
 	unsigned int i;
 
-	if (!openal_initialized)
+	if ( !openal_initialized )
 	{
 		debug(LOG_WARNING, "OpenAL isn't initialized, not creating an audio stream");
 		return NULL;
@@ -693,6 +871,21 @@ AUDIO_STREAM* sound_PlayStreamWithBuf(PHYSFS_file* fileHandle, float volume, voi
 		return NULL;
 	}
 
+	// Clear error codes
+	alGetError();
+
+	// Retrieve an OpenAL sound source
+	alGenSources(1, &(stream->source));
+
+	error = sound_GetError();
+	if (error != AL_NO_ERROR)
+	{
+		// Failed to create OpenAL sound source, so bail out...
+		debug(LOG_SOUND, "alGenSources failed, most likely out of sound sources");
+		free(stream);
+		return NULL;
+	}
+
 	stream->fileHandle = fileHandle;
 
 	stream->decoder = sound_CreateOggVorbisDecoder(stream->fileHandle, false);
@@ -706,15 +899,11 @@ AUDIO_STREAM* sound_PlayStreamWithBuf(PHYSFS_file* fileHandle, float volume, voi
 	stream->volume = volume;
 	stream->bufferSize = streamBufferSize;
 
-	// Retrieve an OpenAL sound source
-	alGenSources(1, &(stream->source));
-	sound_GetError();
-
 	alSourcef(stream->source, AL_GAIN, stream->volume);
 
 	// HACK: this is a workaround for a bug in the 64bit implementation of OpenAL on GNU/Linux
 	// The AL_PITCH value really should be 1.0.
-	alSourcef(stream->source, AL_PITCH, 1.001);
+	alSourcef(stream->source, AL_PITCH, 1.001f);
 
 	// Create some OpenAL buffers to store the decoded data in
 	alGenBuffers(buffer_count, buffers);
@@ -758,6 +947,8 @@ AUDIO_STREAM* sound_PlayStreamWithBuf(PHYSFS_file* fileHandle, float volume, voi
 	// Bail out if we didn't fill any buffers
 	if (i == 0)
 	{
+		debug(LOG_ERROR, "Failed to fill buffers with decoded audio data!");
+
 		// Destroy the decoder
 		sound_DestroyOggVorbisDecoder(stream->decoder);
 
@@ -968,14 +1159,30 @@ static void sound_DestroyStream(AUDIO_STREAM* stream)
 {
 	ALint buffer_count;
 	ALuint* buffers;
+	ALint error;
 
 	// Stop the OpenAL source from playing
 	alSourceStop(stream->source);
-	sound_GetError();
+	error = sound_GetError();
+
+	if (error != AL_NO_ERROR)
+	{
+		// FIXME: We should really handle these errors.
+	}
 
 	// Retrieve the amount of buffers which were processed
 	alGetSourcei(stream->source, AL_BUFFERS_PROCESSED, &buffer_count);
-	sound_GetError();
+	error = sound_GetError();
+	if (error != AL_NO_ERROR)
+	{
+		/* FIXME: We're leaking memory and resources here when bailing
+		 * out. But not doing so could cause stack overflows as a
+		 * result of the below alloca() call (due to buffer_count not
+		 * being properly initialised.
+		 */
+		debug(LOG_SOUND, "alGetSourcei(AL_BUFFERS_PROCESSED) failed; bailing out...");
+		return;
+	}
 
 	// Detach all buffers and retrieve their ID numbers
 	buffers = alloca(buffer_count * sizeof(ALuint));
@@ -1067,40 +1274,36 @@ void sound_StopSample(AUDIO_SAMPLE* psSample)
 #endif
 }
 
-//*
-// =======================================================================================================================
-// =======================================================================================================================
-//
-void sound_SetPlayerPos( SDWORD iX, SDWORD iY, SDWORD iZ )
+void sound_SetPlayerPos(Vector3f pos)
 {
 #ifndef WZ_NOSOUND
-	alListener3f( AL_POSITION, iX, iY, iZ );
+	alListener3f(AL_POSITION, pos.x, pos.y, pos.z);
 	sound_GetError();
 #endif
 }
 
-//
-// =======================================================================================================================
-// =======================================================================================================================
-//
-/** sets player's sound orientation
- * \param iX pitch in degree (current function implementation ignores this)
- * \param iY roll in degree (current function implementation ignores this)
- * \param iZ yaw in degree
+/**
+ * Sets the player's orientation to use for sound
+ * \param forward forward pointing vector
+ * \param up      upward pointing vector
  */
-void sound_SetPlayerOrientation(WZ_DECL_UNUSED float x, WZ_DECL_UNUSED float y, float z)
+void sound_SetPlayerOrientation(Vector3f forward, Vector3f up)
 {
 #ifndef WZ_NOSOUND
-	const float yaw = deg2radf(z);
-	const float ori[6] = {-sinf( yaw ), cosf( yaw ), 0.0f, 0.0f, 0.0f, 1.0f};
+	const ALfloat ori[6] =
+	{
+		forward.x, forward.y, forward.z,
+		up.x,      up.y,      up.z,
+	};
 
-	alListenerfv( AL_ORIENTATION, ori );
+	alListenerfv(AL_ORIENTATION, ori);
 	sound_GetError();
 #endif
 }
 
 //*
 // =======================================================================================================================
+// Compute the sample's volume relative to AL_POSITION.
 // =======================================================================================================================
 //
 void sound_SetObjectPosition(AUDIO_SAMPLE *psSample)
@@ -1114,6 +1317,12 @@ void sound_SetObjectPosition(AUDIO_SAMPLE *psSample)
 	float	distance, gain;
 	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 
+	// only set it when we have a valid sample
+	if (!psSample)
+	{
+		return;
+	}
+
 	// compute distance
 	alGetListener3f( AL_POSITION, &listenerX, &listenerY, &listenerZ );
 	sound_GetError();
@@ -1123,21 +1332,24 @@ void sound_SetObjectPosition(AUDIO_SAMPLE *psSample)
 	distance = sqrtf(dX * dX + dY * dY + dZ * dZ); // Pythagorean theorem
 
 	// compute gain
-	gain = (1.0 - (distance * ATTENUATION_FACTOR)) * psSample->fVol * sfx3d_volume;
-	if (gain > 1.0)
+	gain = (1.0f - (distance * ATTENUATION_FACTOR)) * psSample->fVol * sfx3d_volume;
+	// max volume
+	if (gain > 1.0f)
 	{
-		gain = 1.0;
+		gain = 1.0f;
 	}
-	if ( gain < 0.0 )
+	if (gain < 0.0f)
 	{
-		gain = 0.0;
+		// this sample can't be heard right now
+		gain = 0.0f;
 	}
 	alSourcef( psSample->iSample, AL_GAIN, gain );
 
 	// the alSource3i variant would be better, if it wouldn't provide linker errors however
-	alSource3f( psSample->iSample, AL_POSITION, (float)psSample->x,(float)psSample->x,(float)psSample->x );
+	alSource3f( psSample->iSample, AL_POSITION, (float)psSample->x,(float)psSample->y,(float)psSample->z );
 	sound_GetError();
 #endif
+	return;
 }
 
 //*
@@ -1272,3 +1484,15 @@ void sound_SetEffectsVolume(float volume)
 	}
 #endif
 }
+
+void soundTest()
+{
+	int i;
+
+	for (i = 0; i < 25; i++)
+	{
+		assert(sound_InitLibrary());
+		sound_ShutdownLibrary();
+	}
+	fprintf(stdout, "\tSound self-test: PASSED\n");
+}
