@@ -1,7 +1,7 @@
 /*
 	This file is part of Warzone 2100.
 	Copyright (C) 1999-2004  Eidos Interactive
-	Copyright (C) 2005-2007  Warzone Resurrection Project
+	Copyright (C) 2005-2009  Warzone Resurrection Project
 
 	Warzone 2100 is free software; you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -39,10 +39,11 @@
 #include "lib/framework/frameresource.h"
 #include "lib/framework/input.h"
 #include "lib/framework/tagfile.h"
-#include "lib/framework/math-help.h"
+#include "lib/framework/math_ext.h"
 
 #include "lib/ivis_common/ivisdef.h" //ivis matrix code
 #include "lib/ivis_common/piedef.h" //ivis matrix code
+#include "lib/framework/fixedpoint.h"
 #include "lib/ivis_common/piepalette.h"
 #include "lib/ivis_common/piestate.h"
 #include "lib/ivis_opengl/piematrix.h"
@@ -96,7 +97,6 @@
 #define SET_CYCLIC(x)			((x->control) = (UBYTE)(x->control | EFFECT_CYCLIC))
 #define SET_SCALED(x)			((x->control) = (UBYTE)(x->control | EFFECT_SCALED))
 #define SET_LIT(x)				((x->control) = (UBYTE)(x->control | EFFECT_LIT))
-#define SET_LITABS(x)			((x.control) = (UBYTE)(x.control | EFFECT_LIT))
 
 #define MINIMUM_IMPACT_VELOCITY		(16)
 #define	NORMAL_SMOKE_LIFESPAN		(6000 + rand()%3000)
@@ -182,7 +182,6 @@ static	UDWORD	missCount;
 static	UDWORD	skipped,skippedEffects,letThrough;
 static	UDWORD	auxVar; // dirty filthy hack - don't look for what this does.... //FIXME
 static	UDWORD	auxVarSec; // dirty filthy hack - don't look for what this does.... //FIXME
-static	UDWORD	aeCalls;
 static	UDWORD	specifiedSize;
 static  UDWORD	ellSpec;
 
@@ -213,6 +212,7 @@ static void	renderWaypointEffect	( EFFECT *psEffect );
 static void	renderBloodEffect		( EFFECT *psEffect );
 static void	renderDestructionEffect	( EFFECT *psEffect );
 static void renderFirework			( EFFECT *psEffect );
+
 static void positionEffect(EFFECT *psEffect);
 /* There is no render destruction effect! */
 
@@ -228,11 +228,10 @@ static void effectSetupDestruction  ( EFFECT *psEffect );
 static void	effectSetupFire			( EFFECT *psEffect );
 static void	effectSetUpSatLaser		( EFFECT *psEffect );
 static void effectSetUpFirework		( EFFECT *psEffect );
-#ifdef DEBUG
-static BOOL	validatePie( EFFECT_GROUP group, iIMDShape *pie );
-#endif
 static void effectStructureUpdates(void);
 static void effectDroidUpdates(void);
+static UDWORD effectGetNumFrames(EFFECT *psEffect);
+
 
 static void positionEffect(EFFECT *psEffect)
 {
@@ -260,11 +259,22 @@ static void positionEffect(EFFECT *psEffect)
 
 static void killEffect(EFFECT *e)
 {
+	if (e->group == EFFECT_FIRE)
+	{
+		const int posX = map_coord(e->position.x);
+		const int posY = map_coord(e->position.z);
+		MAPTILE *psTile = mapTile(posX, posY);
+
+		ASSERT(psTile, "Fire effect on non-existing tile (%d, %d)", posX, posY);
+		if (psTile)
+		{
+			psTile->tileInfoBits &= ~BITS_ON_FIRE;	// clear fire bit
+		}
+	}
 	effectStatus[e-asEffectsList] = ES_INACTIVE;
 	e->control = (UBYTE) 0;
 }
 
-// ----------------------------------------------------------------------------------------
 static BOOL	essentialEffect(EFFECT_GROUP group, EFFECT_TYPE type)
 {
 	switch(group)
@@ -281,13 +291,8 @@ static BOOL	essentialEffect(EFFECT_GROUP group, EFFECT_TYPE type)
 		{
 			return(true);
 		}
-		else
-		{
-			return(false);
-		}
 	default:
 		return(false);
-		break;
 	}
 }
 
@@ -301,12 +306,10 @@ static BOOL utterlyReject( EFFECT_GROUP group )
 		return(true);
 	default:
 		return(false);
-		break;
 	}
 }
 
-// ----------------------------------------------------------------------------------------
-/*	Simply sets the free pointer to the start - actually this isn't necessary
+/**	Simply sets the free pointer to the start - actually this isn't necessary
 	as it will work just fine anyway. This WOULD be necessary were we to change
 	the system so that it seeks FREE slots rather than the oldest one. This is because
 	different effects last for different times and the oldest effect may have
@@ -314,9 +317,8 @@ static BOOL utterlyReject( EFFECT_GROUP group )
 */
 void	initEffectsSystem( void )
 {
-UDWORD	i;
-EFFECT	*psEffect;
-
+	UDWORD	i;
+	EFFECT	*psEffect;
 
 	/* Set position to first */
 	freeEffect = 0;
@@ -334,30 +336,30 @@ EFFECT	*psEffect;
 	{
 		/* Get a pointer - just cos our macro requires it, speeds not an issue here */
 		psEffect = &asEffectsList[i];
-		/* Clear all the control bits */
-		psEffect->control = (UBYTE)0;
+		// clear the fire bit
+		killEffect(psEffect);
+		// Clear out the whole array
+		memset(psEffect, 0x0, sizeof(EFFECT));
 		/* All effects are initially inactive */
 		effectStatus[i] = ES_INACTIVE;
 	}
 }
 
-// ----------------------------------------------------------------------------------------
 void	effectSetLandLightSpec(LAND_LIGHT_SPEC spec)
 {
 	ellSpec = spec;
 }
-// ----------------------------------------------------------------------------------------
+
 void	effectSetSize(UDWORD size)
 {
 	specifiedSize = size;
 }
-// ----------------------------------------------------------------------------------------
+
 void	addMultiEffect(Vector3i *basePos, Vector3i *scatter, EFFECT_GROUP group,
 					   EFFECT_TYPE type,BOOL specified, iIMDShape *imd, UDWORD number, BOOL lit, UDWORD size)
 {
-UDWORD	i;
-Vector3i scatPos;
-
+	UDWORD	i;
+	Vector3i scatPos;
 
 	if(number==0)
 	{
@@ -392,12 +394,11 @@ Vector3i scatPos;
 	}
 }
 
-// ----------------------------------------------------------------------------------------
 UDWORD	getNumActiveEffects( void )
 {
 	return(activeEffects);
 }
-// ----------------------------------------------------------------------------------------
+
 UDWORD	getMissCount( void )
 {
 	return(missCount);
@@ -412,15 +413,14 @@ UDWORD	getNumEvenEffects(void)
 {
 	return(letThrough);
 }
-// ----------------------------------------------------------------------------------------
 
-UDWORD Reject1;
 
-void	addEffect(Vector3i *pos, EFFECT_GROUP group, EFFECT_TYPE type,BOOL specified, iIMDShape *imd, BOOL lit)
+void	addEffect(Vector3i *pos, EFFECT_GROUP group, EFFECT_TYPE type,BOOL specified, iIMDShape *imd, int lit)
 {
+	static unsigned int aeCalls = 0;
 	UDWORD	essentialCount;
 	UDWORD	i;
-	BOOL	bSmoke;
+	EFFECT	*psEffect = NULL;
 
 	aeCalls++;
 
@@ -435,6 +435,8 @@ void	addEffect(Vector3i *pos, EFFECT_GROUP group, EFFECT_TYPE type,BOOL specifie
 		/* 	If effect is essentail - then let it through */
 	  	if(!essentialEffect(group,type) )
 		{
+			bool bSmoke = false;
+
 			/* Some we can get rid of right away */
 			if ( utterlyReject( group ) )
 			{
@@ -446,10 +448,7 @@ void	addEffect(Vector3i *pos, EFFECT_GROUP group, EFFECT_TYPE type,BOOL specifie
 			{
 				bSmoke = true;
 			}
-			else
-			{
-				bSmoke = false;
-			}
+
 			/* Others intermittently (50/50 for most and 25/100 for smoke */
 			if(bSmoke ? (aeCalls & 0x03) : (aeCalls & 0x01) )
 			{
@@ -462,11 +461,11 @@ void	addEffect(Vector3i *pos, EFFECT_GROUP group, EFFECT_TYPE type,BOOL specifie
 	}
 
 
-	for(i=freeEffect,essentialCount=0; (asEffectsList[i].control & EFFECT_ESSENTIAL)
-		&& essentialCount<MAX_EFFECTS; i++)
+	for (i = freeEffect, essentialCount = 0; (asEffectsList[i].control & EFFECT_ESSENTIAL)
+		&& essentialCount < MAX_EFFECTS; i++)
 	{
 		/* Check for wrap around */
-		if(i>= (MAX_EFFECTS-1))
+		if (i >= (MAX_EFFECTS-1))
 		{
 			/* Go back to the first one */
 			i = 0;
@@ -476,98 +475,91 @@ void	addEffect(Vector3i *pos, EFFECT_GROUP group, EFFECT_TYPE type,BOOL specifie
 	}
 
 	/* Check the list isn't just full of essential effects */
-	if(essentialCount>=MAX_EFFECTS)
+	if (essentialCount >= MAX_EFFECTS)
 	{
 		/* All of the effects are essential!?!? */
 		return;
 	}
-	else
-	{
-		freeEffect = i;
-	}
+
+	freeEffect = i;
+
+	psEffect = &asEffectsList[freeEffect];
 
 	/* Store away it's position - into FRACTS */
-	asEffectsList[freeEffect].position.x = pos->x;
-	asEffectsList[freeEffect].position.y = pos->y;
-	asEffectsList[freeEffect].position.z = pos->z;
-
+	psEffect->position.x = pos->x;
+	psEffect->position.y = pos->y;
+	psEffect->position.z = pos->z;
 
 	/* Now, note group and type */
-	asEffectsList[freeEffect].group = group;
-	asEffectsList[freeEffect].type = type;
+	psEffect->group = group;
+	psEffect->type = type;
 
 	/* Set when it entered the world */
-	asEffectsList[freeEffect].birthTime = asEffectsList[freeEffect].lastFrame = gameTime;
+	psEffect->birthTime = psEffect->lastFrame = gameTime;
 
 	if(group == EFFECT_GRAVITON && (type == GRAVITON_TYPE_GIBLET || type == GRAVITON_TYPE_EMITTING_DR))
 	{
-		asEffectsList[freeEffect].frameNumber = lit;
+		psEffect->frameNumber = lit;
 	}
 
 	else
 	{
 		/* Starts off on frame zero */
-		asEffectsList[freeEffect].frameNumber = 0;
+		psEffect->frameNumber = 0;
 	}
 
 	/*
 		See what kind of effect it is - the add fucnction is different for each,
 		although some things are shared
 	*/
-	asEffectsList[freeEffect].imd = NULL;
+	psEffect->imd = NULL;
 	if(lit)
 	{
-		SET_LITABS(asEffectsList[freeEffect]);
+		SET_LIT(psEffect);
 	}
 
 	if(specified)
 	{
 		/* We're specifying what the imd is - override */
-		asEffectsList[freeEffect].imd = imd;
-//		if(type == EXPLOSION_TYPE_SPECIFIED_FIXME)
-//		{
-//			asEffectsList[freeEffect].size = EXPLOSION_SIZE;
-//		}
-//		else
-//		{
-		asEffectsList[freeEffect].size =(UWORD) specifiedSize;
-//		}
+		psEffect->imd = imd;
+		psEffect->size = specifiedSize;
 	}
 
 	/* Do all the effect type specific stuff */
 	switch(group)
 	{
 		case EFFECT_SMOKE:
-			effectSetupSmoke(&asEffectsList[freeEffect]);
+			effectSetupSmoke(psEffect);
 			break;
 		case EFFECT_GRAVITON:
-			effectSetupGraviton(&asEffectsList[freeEffect]);
+			effectSetupGraviton(psEffect);
 			break;
 		case EFFECT_EXPLOSION:
-			effectSetupExplosion(&asEffectsList[freeEffect]);
+			effectSetupExplosion(psEffect);
 			break;
 		case EFFECT_CONSTRUCTION:
-			effectSetupConstruction(&asEffectsList[freeEffect]);
+			effectSetupConstruction(psEffect);
 			break;
 		case EFFECT_WAYPOINT:
-			effectSetupWayPoint(&asEffectsList[freeEffect]);
+			effectSetupWayPoint(psEffect);
 			break;
 		case EFFECT_BLOOD:
-			effectSetupBlood(&asEffectsList[freeEffect]);
+			effectSetupBlood(psEffect);
 			break;
 		case EFFECT_DESTRUCTION:
-			effectSetupDestruction(&asEffectsList[freeEffect]);
+			effectSetupDestruction(psEffect);
 			break;
 		case EFFECT_FIRE:
-			effectSetupFire(&asEffectsList[freeEffect]);
+			effectSetupFire(psEffect);
 			break;
 		case EFFECT_SAT_LASER:
-			effectSetUpSatLaser(&asEffectsList[freeEffect]);
+			effectSetUpSatLaser(psEffect);
 			break;
 		case EFFECT_FIREWORK:
-			effectSetUpFirework(&asEffectsList[freeEffect]);
+			effectSetUpFirework(psEffect);
 			break;
-		default:
+		case EFFECT_STRUCTURE:
+		case EFFECT_DUST_BALL:
 			ASSERT( false,"Weirdy group type for an effect" );
 			break;
 	}
@@ -578,21 +570,10 @@ void	addEffect(Vector3i *pos, EFFECT_GROUP group, EFFECT_TYPE type,BOOL specifie
 	/* As of yet, it hasn't bounced (or whatever)... */
 	if(type!=EXPLOSION_TYPE_LAND_LIGHT)
 	{
-		asEffectsList[freeEffect].specific = 0;
+		psEffect->specific = 0;
 	}
 
-	/* Looks like we didn't establish an imd for the effect */
-	/*
-	ASSERT( asEffectsList[freeEffect].imd != NULL || group == EFFECT_DESTRUCTION || group == EFFECT_FIRE || group == EFFECT_SAT_LASER,
-		"null effect imd" );
-	*/
-
-#ifdef DEBUG
-	if ( validatePie( group, asEffectsList[freeEffect].imd ) == false )
-	{
-		ASSERT( false,"No PIE found or specified for an effect" );
-	}
-#endif
+	ASSERT(psEffect->imd != NULL || group == EFFECT_DESTRUCTION || group == EFFECT_FIRE || group == EFFECT_SAT_LASER, "null effect imd");
 
 	/* No more slots available? */
 	if(freeEffect++ >= (MAX_EFFECTS-1))
@@ -602,30 +583,6 @@ void	addEffect(Vector3i *pos, EFFECT_GROUP group, EFFECT_TYPE type,BOOL specifie
 	}
 }
 
-#ifdef DEBUG
-// ----------------------------------------------------------------------------------------
-static BOOL validatePie( EFFECT_GROUP group, iIMDShape *pie )
-{
-
-	/* If we haven't got a pie */
-	if(pie == NULL)
-	{
-		if(group == EFFECT_DESTRUCTION || group == EFFECT_FIRE || group == EFFECT_SAT_LASER)
-		{
-			/* Ok in these cases */
-			return(true);
-		}
-
-		return(false);
-	}
-	else
-	{
-		return(true);
-	}
-}
-// ----------------------------------------------------------------------------------------
-#endif
-
 /* Calls all the update functions for each different currently active effect */
 void	processEffects(void)
 {
@@ -652,7 +609,6 @@ void	processEffects(void)
 				/* Add it to the bucket */
 				bucketAddTypeToList(RENDER_EFFECT,&asEffectsList[i]);
 			}
-
 		}
 	}
 
@@ -666,7 +622,6 @@ void	processEffects(void)
 	skippedEffects = skipped;
 }
 
-// ----------------------------------------------------------------------------------------
 /* The general update function for all effects - calls a specific one for each */
 static void updateEffect(EFFECT *psEffect)
 {
@@ -675,56 +630,55 @@ static void updateEffect(EFFECT *psEffect)
 	{
 	case EFFECT_EXPLOSION:
 		updateExplosion(psEffect);
-		break;
+		return;
 
 	case EFFECT_WAYPOINT:
 		if(!gamePaused()) updateWaypoint(psEffect);
-		break;
+		return;
 
 	case EFFECT_CONSTRUCTION:
 		if(!gamePaused()) updateConstruction(psEffect);
-		break;
+		return;
 
 	case EFFECT_SMOKE:
 		if(!gamePaused()) updatePolySmoke(psEffect);
-		break;
+		return;
 
 	case EFFECT_STRUCTURE:
-		break;
+		return;
 
 	case EFFECT_GRAVITON:
 		if(!gamePaused()) updateGraviton(psEffect);
-		break;
+		return;
 
 	case EFFECT_BLOOD:
 		if(!gamePaused()) updateBlood(psEffect);
-		break;
+		return;
 
 	case EFFECT_DESTRUCTION:
 		if(!gamePaused()) updateDestruction(psEffect);
-		break;
+		return;
 
 	case EFFECT_FIRE:
 		if(!gamePaused()) updateFire(psEffect);
-		break;
+		return;
 
 	case EFFECT_SAT_LASER:
 		if(!gamePaused()) updateSatLaser(psEffect);
-		break;
+		return;
+
 	case EFFECT_FIREWORK:
 		if(!gamePaused()) updateFirework(psEffect);
-		break;
-	default:
-		debug( LOG_ERROR, "Weirdy class of effect passed to updateEffect" );
-		abort();
-		break;
+		return;
 	}
+
+	debug( LOG_ERROR, "Weirdy class of effect passed to updateEffect" );
 }
 
 // ----------------------------------------------------------------------------------------
 // ALL THE UPDATE FUNCTIONS
 // ----------------------------------------------------------------------------------------
-/* Update the waypoint effects.*/
+/** Update the waypoint effects.*/
 static void updateWaypoint(EFFECT *psEffect)
 {
 	if(!(keyDown(KEY_LCTRL) || keyDown(KEY_RCTRL) ||
@@ -734,8 +688,6 @@ static void updateWaypoint(EFFECT *psEffect)
 	}
 }
 
-
-// ----------------------------------------------------------------------------------------
 static void updateFirework(EFFECT *psEffect)
 {
 	UDWORD	height;
@@ -822,7 +774,7 @@ static void updateFirework(EFFECT *psEffect)
 			psEffect->lastFrame = gameTime;
 
 			/* Are we on the last frame? */
-			if(++psEffect->frameNumber >= EffectGetNumFrames(psEffect))
+			if(++psEffect->frameNumber >= effectGetNumFrames(psEffect))
 			{
 				/* Does the anim wrap around? */
 				if(TEST_CYCLIC(psEffect))
@@ -851,7 +803,6 @@ static void updateFirework(EFFECT *psEffect)
 	}
 }
 
-// ----------------------------------------------------------------------------------------
 static void updateSatLaser(EFFECT *psEffect)
 {
 	Vector3i dv;
@@ -945,8 +896,7 @@ static void updateSatLaser(EFFECT *psEffect)
 	}
 }
 
-// ----------------------------------------------------------------------------------------
-/* The update function for the explosions */
+/** The update function for the explosions */
 static void updateExplosion(EFFECT *psEffect)
 {
 	LIGHT light;
@@ -1002,7 +952,7 @@ static void updateExplosion(EFFECT *psEffect)
 	{
 		psEffect->size += timeAdjustedIncrement(SHOCKWAVE_SPEED, true);
 		scaling = (float)psEffect->size / (float)MAX_SHOCKWAVE_SIZE;
-		psEffect->frameNumber = scaling * EffectGetNumFrames(psEffect);
+		psEffect->frameNumber = scaling * effectGetNumFrames(psEffect);
 
 		light.position.x = psEffect->position.x;
 		light.position.y = psEffect->position.y;
@@ -1027,7 +977,7 @@ static void updateExplosion(EFFECT *psEffect)
 			psEffect->lastFrame = gameTime;
 			/* Are we on the last frame? */
 
-			if(++psEffect->frameNumber >= EffectGetNumFrames(psEffect))
+			if(++psEffect->frameNumber >= effectGetNumFrames(psEffect))
 			{
 		 		if(psEffect->type!=EXPLOSION_TYPE_LAND_LIGHT)
 				{
@@ -1054,8 +1004,9 @@ static void updateExplosion(EFFECT *psEffect)
 	}
 }
 
-// ----------------------------------------------------------------------------------------
-/* The update function for blood */
+
+
+/** The update function for blood */
 static void updateBlood(EFFECT *psEffect)
 {
 	/* Time to update the frame number on the blood */
@@ -1063,7 +1014,7 @@ static void updateBlood(EFFECT *psEffect)
 	{
 		psEffect->lastFrame = gameTime;
 		/* Are we on the last frame? */
-		if(++psEffect->frameNumber >= EffectGetNumFrames(psEffect))
+		if(++psEffect->frameNumber >= effectGetNumFrames(psEffect))
 		{
 			/* Kill it off */
 			killEffect(psEffect);
@@ -1076,8 +1027,7 @@ static void updateBlood(EFFECT *psEffect)
 	psEffect->position.z += timeAdjustedIncrement(psEffect->velocity.z, true);
 }
 
-// ----------------------------------------------------------------------------------------
-/* Processes all the drifting smoke
+/** Processes all the drifting smoke
 	Handles the smoke puffing out the factory as well */
 static void updatePolySmoke(EFFECT *psEffect)
 {
@@ -1089,7 +1039,7 @@ static void updatePolySmoke(EFFECT *psEffect)
 		psEffect->lastFrame = gameTime;
 
 		/* Are we on the last frame? */
-		if(++psEffect->frameNumber >= EffectGetNumFrames(psEffect))
+		if(++psEffect->frameNumber >= effectGetNumFrames(psEffect))
 		{
 			/* Does the anim wrap around? */
 			if(TEST_CYCLIC(psEffect))
@@ -1131,8 +1081,7 @@ static void updatePolySmoke(EFFECT *psEffect)
 	}
 }
 
-// ----------------------------------------------------------------------------------------
-/*
+/**
 	Gravitons just fly up for a bit and then drop down and are
 	killed off when they hit the ground
 */
@@ -1281,10 +1230,7 @@ static void updateGraviton(EFFECT *psEffect)
 }
 
 
-// ----------------------------------------------------------------------------------------
-/* updateDestruction
-This isn't really an on-screen effect itself - it just spawns other ones....
-  */
+/** This isn't really an on-screen effect itself - it just spawns other ones.... */
 static void updateDestruction(EFFECT *psEffect)
 {
 	Vector3i pos;
@@ -1461,11 +1407,7 @@ static void updateDestruction(EFFECT *psEffect)
 	}
 }
 
-// ----------------------------------------------------------------------------------------
-/*
-updateConstruction:-
-Moves the construction graphic about - dust cloud or whatever....
-*/
+/** Moves the construction graphic about - dust cloud or whatever.... */
 static void updateConstruction(EFFECT *psEffect)
 {
 
@@ -1474,7 +1416,7 @@ static void updateConstruction(EFFECT *psEffect)
 	{
 		psEffect->lastFrame = gameTime;
 		/* Are we on the last frame? */
-		if(++psEffect->frameNumber >= EffectGetNumFrames(psEffect))
+		if(++psEffect->frameNumber >= effectGetNumFrames(psEffect))
 		{
 			/* Is it a cyclic sprite? */
 			if(TEST_CYCLIC(psEffect))
@@ -1515,8 +1457,7 @@ static void updateConstruction(EFFECT *psEffect)
 	}
 }
 
-// ----------------------------------------------------------------------------------------
-/* Update fire sequences */
+/** Update fire sequences */
 static void updateFire(EFFECT *psEffect)
 {
 	Vector3i pos;
@@ -1542,14 +1483,14 @@ static void updateFire(EFFECT *psEffect)
 		psEffect->lastFrame = gameTime;
 		pos.x = (psEffect->position.x + ((rand() % psEffect->radius) - (rand() % (2 * psEffect->radius))));
 		pos.z = (psEffect->position.z + ((rand() % psEffect->radius) - (rand() % (2 * psEffect->radius))));
-		
+
 		// Effect is off map, no need to update it anymore
 		if(!worldOnMap(pos.x, pos.z))
 		{
 			killEffect(psEffect);
 			return;
 		}
-		
+
 		pos.y = map_Height(pos.x,pos.z);
 
 		if(psEffect->type == FIRE_TYPE_SMOKY_BLUE)
@@ -1605,10 +1546,7 @@ static void updateFire(EFFECT *psEffect)
 // ----------------------------------------------------------------------------------------
 // ALL THE RENDER FUNCTIONS
 // ----------------------------------------------------------------------------------------
-/*
-renderEffect:-
-Calls the appropriate render routine for each type of effect
-*/
+/** Calls the appropriate render routine for each type of effect */
 void	renderEffect(EFFECT *psEffect)
 {
 		/* What type of effect are we dealing with? */
@@ -1616,55 +1554,55 @@ void	renderEffect(EFFECT *psEffect)
 	{
 	case EFFECT_WAYPOINT:
 		renderWaypointEffect(psEffect);
-		break;
+		return;
 
 	case EFFECT_EXPLOSION:
 		renderExplosionEffect(psEffect);
-		break;
+		return;
 
 	case EFFECT_CONSTRUCTION:
 		renderConstructionEffect(psEffect);
-		break;
+		return;
 
 	case EFFECT_SMOKE:
 		renderSmokeEffect(psEffect);
-		break;
+		return;
 
 	case EFFECT_GRAVITON:
 		renderGravitonEffect(psEffect);
-		break;
+		return;
 
 	case EFFECT_BLOOD:
 		renderBloodEffect(psEffect);
-		break;
+		return;
 
 	case EFFECT_STRUCTURE:
-		break;
+		return;
 
 	case EFFECT_DESTRUCTION:
 		/*	There is no display func for a destruction effect -
 			it merely spawn other effects over time */
 		renderDestructionEffect(psEffect);
-		break;
+		return;
+
 	case EFFECT_FIRE:
 		/* Likewise */
-		break;
+		return;
+
 	case EFFECT_SAT_LASER:
 		/* Likewise */
-		break;
+		return;
+
 	case EFFECT_FIREWORK:
 		renderFirework(psEffect);
-		break;
-	default:
-		debug( LOG_ERROR, "Weirdy class of effect passed to renderEffect" );
-		abort();
-		break;
+		return;
 	}
+
+	debug( LOG_ERROR, "Weirdy class of effect passed to renderEffect" );
 }
 
-// ----------------------------------------------------------------------------------------
-/* drawing func for wapypoints . AJL. */
-void	renderWaypointEffect(EFFECT *psEffect)
+/** drawing func for wapypoints */
+static void renderWaypointEffect(EFFECT *psEffect)
 {
 	positionEffect(psEffect);
 
@@ -1672,8 +1610,7 @@ void	renderWaypointEffect(EFFECT *psEffect)
 	iV_MatrixEnd();
 }
 
-// ----------------------------------------------------------------------------------------
-void	renderFirework(EFFECT *psEffect)
+static void renderFirework(EFFECT *psEffect)
 {
 	/* these don't get rendered */
 	if(psEffect->type == FIREWORK_TYPE_LAUNCHER)
@@ -1691,9 +1628,8 @@ void	renderFirework(EFFECT *psEffect)
  	iV_MatrixEnd();
 }
 
-// ----------------------------------------------------------------------------------------
-/* drawing func for blood. */
-void	renderBloodEffect(EFFECT *psEffect)
+/** drawing func for blood. */
+static void renderBloodEffect(EFFECT *psEffect)
 {
 	positionEffect(psEffect);
 
@@ -1705,8 +1641,7 @@ void	renderBloodEffect(EFFECT *psEffect)
 	iV_MatrixEnd();
 }
 
-// ----------------------------------------------------------------------------------------
-void	renderDestructionEffect(EFFECT *psEffect)
+static void renderDestructionEffect(EFFECT *psEffect)
 {
 	float	div;
 	SDWORD	percent;
@@ -1736,37 +1671,34 @@ void	renderDestructionEffect(EFFECT *psEffect)
 	iV_MatrixEnd();
 }
 
-// ----------------------------------------------------------------------------------------
-static BOOL	rejectLandLight(LAND_LIGHT_SPEC type)
+static bool rejectLandLight(LAND_LIGHT_SPEC type)
 {
-UDWORD	timeSlice;
+	unsigned int timeSlice = gameTime%2000;
 
-	timeSlice = gameTime%2000;
-	if(timeSlice<400)
+	if (timeSlice < 400)
 	{
-		if(type == LL_MIDDLE) return(false); else return(true);	// reject all expect middle
+		return (type != LL_MIDDLE); // reject all expect middle
 	}
-	else if(timeSlice<800)
+	else if (timeSlice < 800)
 	{
-		if(type == LL_OUTER) return(true); else return(false);	// reject only outer
+		return (type == LL_OUTER); // reject only outer
 	}
-	else if(timeSlice<1200)
+	else if (timeSlice < 1200)
 	{
-		return(false);	//reject none
+		return(false); //reject none
 	}
-	else if(timeSlice<1600)
+	else if (timeSlice < 1600)
 	{
-		if(type == LL_OUTER) return(true); else return(false);	// reject only outer
+		return (type == LL_OUTER); // reject only outer
 	}
 	else
 	{
-		if(type == LL_MIDDLE) return(false); else return(true);	// reject all expect middle
+		return (type != LL_MIDDLE); // reject all expect middle
 	}
 }
 
-// ----------------------------------------------------------------------------------------
-/* Renders the standard explosion effect */
-void	renderExplosionEffect(EFFECT *psEffect)
+/** Renders the standard explosion effect */
+static void renderExplosionEffect(EFFECT *psEffect)
 {
 	SDWORD	percent;
 	const PIELIGHT brightness = WZCOL_WHITE;
@@ -1823,8 +1755,7 @@ void	renderExplosionEffect(EFFECT *psEffect)
 	iV_MatrixEnd();
 }
 
-// ----------------------------------------------------------------------------------------
-void	renderGravitonEffect(EFFECT *psEffect)
+static void renderGravitonEffect(EFFECT *psEffect)
 {
 
 	positionEffect(psEffect);
@@ -1850,11 +1781,8 @@ void	renderGravitonEffect(EFFECT *psEffect)
 	iV_MatrixEnd();
 }
 
-// ----------------------------------------------------------------------------------------
-/*
-renderConstructionEffect:-
-Renders the standard construction effect */
-void	renderConstructionEffect(EFFECT *psEffect)
+/** Renders the standard construction effect */
+static void renderConstructionEffect(EFFECT *psEffect)
 {
 	Vector3i null;
 	SDWORD	percent;
@@ -1898,12 +1826,8 @@ void	renderConstructionEffect(EFFECT *psEffect)
 	iV_MatrixEnd();
 }
 
-// ----------------------------------------------------------------------------------------
-/*
-renderSmokeEffect:-
-Renders the standard smoke effect - it is now scaled in real-time as well
-*/
-void	renderSmokeEffect(EFFECT *psEffect)
+/** Renders the standard smoke effect - it is now scaled in real-time as well */
+static void renderSmokeEffect(EFFECT *psEffect)
 {
 	UDWORD	transparency = 0;
 	const PIELIGHT brightness = WZCOL_WHITE;
@@ -2026,7 +1950,6 @@ void	effectSetUpFirework(EFFECT *psEffect)
 
 }
 
-// ----------------------------------------------------------------------------------------
 void	effectSetupSmoke(EFFECT *psEffect)
 {
 	/* everything except steam drifts about */
@@ -2111,7 +2034,6 @@ void	effectSetupSmoke(EFFECT *psEffect)
 	}
 }
 
-// ----------------------------------------------------------------------------------------
 void effectSetUpSatLaser(EFFECT *psEffect)
 {
 	/* Does nothing at all..... Runs only for one frame! */
@@ -2119,7 +2041,6 @@ void effectSetUpSatLaser(EFFECT *psEffect)
 	return;
 }
 
-// ----------------------------------------------------------------------------------------
 void	effectSetupGraviton(EFFECT *psEffect)
 {
 	switch(psEffect->type)
@@ -2167,7 +2088,6 @@ void	effectSetupGraviton(EFFECT *psEffect)
 	}
 }
 
-// ----------------------------------------------------------------------------------------
 void effectSetupExplosion(EFFECT *psEffect)
 {
 	/* Get an imd if it's not established */
@@ -2295,7 +2215,6 @@ void effectSetupExplosion(EFFECT *psEffect)
 	}
 }
 
-// ----------------------------------------------------------------------------------------
 void	effectSetupConstruction(EFFECT *psEffect)
 {
 	psEffect->velocity.x = 0.f;//(1-rand()%3);
@@ -2322,9 +2241,17 @@ void	effectSetupConstruction(EFFECT *psEffect)
 	}
 }
 
-// ----------------------------------------------------------------------------------------
 void	effectSetupFire(EFFECT *psEffect)
 {
+	const int posX = map_coord(psEffect->position.x);
+	const int posY = map_coord(psEffect->position.z);
+	MAPTILE *psTile = mapTile(posX, posY);
+
+	ASSERT(psTile, "Cannot place a fire effect %d, %d - outside map!", posX, posY);
+	if (psTile)
+	{
+		psTile->tileInfoBits |= BITS_ON_FIRE;
+	}
 	psEffect->frameDelay = 300;	   // needs to be investigated...
 	psEffect->radius = auxVar;	// needs to be investigated
 	psEffect->lifeSpan = (UWORD)auxVarSec;
@@ -2333,7 +2260,6 @@ void	effectSetupFire(EFFECT *psEffect)
 
 }
 
-// ----------------------------------------------------------------------------------------
 void	effectSetupWayPoint(EFFECT *psEffect)
 {
 	psEffect->imd = pProximityMsgIMD;
@@ -2342,8 +2268,8 @@ void	effectSetupWayPoint(EFFECT *psEffect)
 	SET_ESSENTIAL(psEffect);
 }
 
-// ----------------------------------------------------------------------------------------
-void	effectSetupBlood(EFFECT *psEffect)
+
+static void effectSetupBlood(EFFECT *psEffect)
 {
 	psEffect->frameDelay = BLOOD_FRAME_DELAY;
 	psEffect->velocity.y = (float)BLOOD_FALL_SPEED;
@@ -2351,8 +2277,7 @@ void	effectSetupBlood(EFFECT *psEffect)
 	psEffect->size = (UBYTE)BLOOD_SIZE;
 }
 
-// ----------------------------------------------------------------------------------------
-void    effectSetupDestruction(EFFECT *psEffect)
+static void effectSetupDestruction(EFFECT *psEffect)
 {
 
 	if(psEffect->type == DESTRUCTION_TYPE_SKYSCRAPER)
@@ -2387,7 +2312,6 @@ void    effectSetupDestruction(EFFECT *psEffect)
 
 #define FX_PER_EDGE 6
 #define	SMOKE_SHIFT	(16 - (rand()%32))
-// ----------------------------------------------------------------------------------------
 void	initPerimeterSmoke(iIMDShape *pImd, UDWORD x, UDWORD y, UDWORD z)
 {
 	SDWORD	i;
@@ -2475,47 +2399,25 @@ void	initPerimeterSmoke(iIMDShape *pImd, UDWORD x, UDWORD y, UDWORD z)
 	}
 }
 
-// ----------------------------------------------------------------------------------------
-UDWORD	getNumEffects( void )
+
+static UDWORD effectGetNumFrames(EFFECT *psEffect)
 {
-	return(numEffects);
-}
-
-
-// ----------------------------------------------------------------------------------------
-UDWORD EffectGetNumFrames(EFFECT *psEffect)
-{
-
 	return psEffect->imd->numFrames;
-
 }
 
-UDWORD IMDGetNumFrames(iIMDShape *Shape)
-{
-
-	return Shape->numFrames;
-
-}
-
-UDWORD IMDGetAnimInterval(iIMDShape *Shape)
-{
-
-	return Shape->animInterval;
-
-}
 
 void	effectGiveAuxVar( UDWORD var)
 {
 	auxVar = var;
 }
 
+
 void	effectGiveAuxVarSec( UDWORD var)
 {
 	auxVarSec = var;
 }
 
-// ----------------------------------------------------------------------------------------
-/* Runs all the spot effect stuff for the droids - adding of dust and the like... */
+/** Runs all the spot effect stuff for the droids - adding of dust and the like... */
 static void effectDroidUpdates(void)
 {
 	unsigned int i;
@@ -2562,8 +2464,7 @@ static void effectDroidUpdates(void)
 	}
 }
 
-// ----------------------------------------------------------------------------------------
-/* Runs all the structure effect stuff - steam puffing out etc */
+/** Runs all the structure effect stuff - steam puffing out etc */
 static void effectStructureUpdates(void)
 {
 	UDWORD		i;
@@ -2685,16 +2586,10 @@ static void effectStructureUpdates(void)
 	}
 }
 
-UDWORD	getFreeEffect( void )
-{
-	return(freeEffect);
-}
 
-
-// ----------------------------------------------------------------------------------------
 void	effectResetUpdates( void )
 {
-UDWORD	i;
+	UDWORD	i;
 
 	for(i=0; i<EFFECT_DROID_DIVISION; i++)
 	{
@@ -2707,35 +2602,10 @@ UDWORD	i;
 }
 
 
-// -----------------------------------------------------------------------------------
-bool fireOnLocation(unsigned int x, unsigned int y)
-{
-	unsigned int i;
-
-	for(i = 0; i < MAX_EFFECTS; ++i)
-	{
-	 	if (effectStatus[i] == ES_ACTIVE
-	 	 && asEffectsList[i].group == EFFECT_FIRE)
-		{
-			const unsigned int posX = asEffectsList[i].position.x;
-			const unsigned int posY = asEffectsList[i].position.z;
-
-			if (posX == x
-			 && posY == y)
-			{
-				return true;
-			}
-		}
-	}
-
-	return false;
-}
-
 static const char FXData_tag_definition[] = "tagdefinitions/savegame/effects.def";
 static const char FXData_file_identifier[] = "FXData";
 
-// -----------------------------------------------------------------------------------
-/* This will save out the effects data */
+/** This will save out the effects data */
 bool writeFXData(const char* fileName)
 {
 	unsigned int count, i;
@@ -2801,8 +2671,7 @@ bool writeFXData(const char* fileName)
 	return true;
 }
 
-// -----------------------------------------------------------------------------------
-/* This will read in the effects data */
+/** This will read in the effects data */
 bool readFXData(const char* fileName)
 {
 	unsigned int count, i;
