@@ -1,7 +1,7 @@
 /*
 	This file is part of Warzone 2100.
 	Copyright (C) 1999-2004  Eidos Interactive
-	Copyright (C) 2005-2007  Warzone Resurrection Project
+	Copyright (C) 2005-2009  Warzone Resurrection Project
 
 	Warzone 2100 is free software; you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -93,6 +93,7 @@ BOOL		startGameOptionsMenu	(void);
 BOOL		startGameOptions2Menu	(void);
 BOOL		startGameOptions3Menu	(void);
 BOOL		startGameOptions4Menu	(void);
+BOOL		startGameOptions5Menu	(void);
 
 void		removeTopForm			(void);
 void		removeBottomForm		(void);
@@ -159,6 +160,10 @@ void changeTitleMode(tMode mode)
 		startGameOptions4Menu();
 		break;
 
+	case GAME5:
+		startGameOptions5Menu();
+		break;
+
 	case TUTORIAL:
 		startTutorialMenu();
 		break;
@@ -242,8 +247,9 @@ BOOL startTitleMenu(void)
 	addTextButton(FRONTEND_MULTIPLAYER, FRONTEND_POS3X, FRONTEND_POS3Y, _("Multi Player"), false, false);
 	addTextButton(FRONTEND_TUTORIAL, FRONTEND_POS4X, FRONTEND_POS4Y, _("Tutorial") ,false,false);
 	addTextButton(FRONTEND_OPTIONS, FRONTEND_POS5X, FRONTEND_POS5Y, _("Options") ,false,false);
+	addTextButton(FRONTEND_PLAYINTRO, FRONTEND_POS6X, FRONTEND_POS6Y, _("View Intro"), false, false);
 
-	addTextButton(FRONTEND_QUIT, FRONTEND_POS6X, FRONTEND_POS6Y, _("Quit Game"), false, false);
+	addTextButton(FRONTEND_QUIT, FRONTEND_POS7X, FRONTEND_POS7Y, _("Quit Game"), false, false);
 
 	addSideText(FRONTEND_SIDETEXT, FRONTEND_SIDEX, FRONTEND_SIDEY, _("MAIN MENU"));
 
@@ -274,6 +280,9 @@ BOOL runTitleMenu(void)
 		case FRONTEND_TUTORIAL:
 			changeTitleMode(TUTORIAL);
 			break;
+		case FRONTEND_PLAYINTRO:
+			changeTitleMode(SHOWINTRO);
+			break;
 		default:
 			break;
 	}
@@ -350,9 +359,9 @@ void startSinglePlayerMenu(void)
 	addTopForm();
 	addBottomForm();
 
-	addTextButton(FRONTEND_NEWGAME,  FRONTEND_POS3X,FRONTEND_POS3Y,_("New Campaign") ,false,false);
-	addTextButton(FRONTEND_SKIRMISH, FRONTEND_POS4X,FRONTEND_POS4Y, _("Start Skirmish Game"),false,false);
-	addTextButton(FRONTEND_LOADGAME, FRONTEND_POS5X,FRONTEND_POS5Y, _("Load Game"),false,false);
+	addTextButton(FRONTEND_NEWGAME,  FRONTEND_POS2X,FRONTEND_POS2Y,_("New Campaign") ,false,false);
+	addTextButton(FRONTEND_SKIRMISH, FRONTEND_POS3X,FRONTEND_POS3Y, _("Start Skirmish Game"),false,false);
+	addTextButton(FRONTEND_LOADGAME, FRONTEND_POS4X,FRONTEND_POS4Y, _("Load Game"),false,false);
 
 	addSideText	 (FRONTEND_SIDETEXT ,FRONTEND_SIDEX,FRONTEND_SIDEY,_("SINGLE PLAYER"));
 	addMultiBut(psWScreen, FRONTEND_BOTFORM, FRONTEND_QUIT, 10, 10, 30, 29, P_("menu", "Return"), IMAGE_RETURN, IMAGE_RETURN_HI, IMAGE_RETURN_HI);
@@ -365,7 +374,7 @@ static void frontEndNewGame( void )
 			sstrcpy(aLevelName, DEFAULT_LEVEL);
 			seq_ClearSeqList();
 
-			seq_AddSeqToList("cam1/c001.rpl",NULL,"cam1/c001.txa",false);
+			seq_AddSeqToList("cam1/c001.ogg",NULL,"cam1/c001.txa",false);
 
 			seq_StartNextFullScreenVideo();
             break;
@@ -404,14 +413,26 @@ BOOL runSinglePlayerMenu(void)
 	}
 	else
 	{
+		uint8_t playercolor = 0;
 
-	id = widgRunScreen(psWScreen);						// Run the current set of widgets
+		// FIXME: We should do a SPinit() to make sure all the variables are reset correctly.
+		// game.type is switched to SKIRMISH in startMultiOptions()
+		NetPlay.bComms = false;
+		bMultiPlayer = false;
+		game.type = CAMPAIGN;
+		// make sure we have a valid color choice for our SP game. Valid values are 0, 4-7
+		playercolor = getPlayerColour(0);
+		if (playercolor >= 1 && playercolor <= 3)
+		{
+			setPlayerColour(0,0);		// default is green
+		}
+
+		id = widgRunScreen(psWScreen);						// Run the current set of widgets
 
 		switch(id)
 		{
 			case FRONTEND_NEWGAME:
 				frontEndNewGame();
-				NetPlay.bComms = false;
 				break;
 
 			case FRONTEND_LOADCAM2:
@@ -438,7 +459,6 @@ BOOL runSinglePlayerMenu(void)
 				break;
 
 			case FRONTEND_SKIRMISH:
-				NetPlay.bComms = false; // use network = false
 				ingame.bHostSetup = true;
 				changeTitleMode(MULTIOPTION);
 				break;
@@ -500,6 +520,9 @@ BOOL runMultiPlayerMenu(void)
 	{
 	case FRONTEND_HOST:
 		ingame.bHostSetup = true;
+		bMultiPlayer = true;
+		game.type = SKIRMISH;		// needed?
+		NetPlay.bComms = true;
 		changeTitleMode(MULTIOPTION);
 		break;
 	case FRONTEND_JOIN:
@@ -535,7 +558,8 @@ BOOL startOptionsMenu(void)
 	addTextButton(FRONTEND_GAMEOPTIONS2,FRONTEND_POS3X,FRONTEND_POS3Y, _("Graphics Options"),false,false);
 	addTextButton(FRONTEND_GAMEOPTIONS4, FRONTEND_POS4X,FRONTEND_POS4Y, _("Video Options"), false, false);
 	addTextButton(FRONTEND_GAMEOPTIONS3,	FRONTEND_POS5X,FRONTEND_POS5Y, _("Audio Options"),false,false);
-	addTextButton(FRONTEND_KEYMAP,		FRONTEND_POS6X,FRONTEND_POS6Y, _("Key Mappings"),false,false);
+	addTextButton(FRONTEND_GAMEOPTIONS5,	FRONTEND_POS6X,FRONTEND_POS6Y, _("Mouse Options"),false,false);
+	addTextButton(FRONTEND_KEYMAP,		FRONTEND_POS7X,FRONTEND_POS7Y, _("Key Mappings"),false,false);
 	addMultiBut(psWScreen, FRONTEND_BOTFORM, FRONTEND_QUIT, 10, 10, 30, 29, P_("menu", "Return"), IMAGE_RETURN, IMAGE_RETURN_HI, IMAGE_RETURN_HI);
 
 	return true;
@@ -562,6 +586,9 @@ BOOL runOptionsMenu(void)
 	case FRONTEND_GAMEOPTIONS4:
 		changeTitleMode(GAME4);
 		break;
+	case FRONTEND_GAMEOPTIONS5:
+		changeTitleMode(GAME5);
+		break;
 //	case FRONTEND_VIDEO:
 //		changeTitleMode(VIDEO);
 //		break;
@@ -597,24 +624,33 @@ BOOL runOptionsMenu(void)
 }
 
 // ////////////////////////////////////////////////////////////////////////////
-// Game Options Menu 2!
+// Game options Menu
 BOOL startGameOptions2Menu(void)
 {
 	addBackdrop();
 	addTopForm();
 	addBottomForm();
 
+//	////////////
+//	//FMV mode.
+	addTextButton(FRONTEND_FMVMODE,	FRONTEND_POS2X - 35, FRONTEND_POS2Y, _("Video Playback"), true, false);
+	switch (war_GetFMVmode())
+	{
+		case FMV_1X:
+			addTextButton(FRONTEND_FMVMODE_R, FRONTEND_POS2M - 55,FRONTEND_POS2Y, _("1X"), true, false);
+			break;
 
-	////////////
-	// mouseflip
-	addTextButton(FRONTEND_MFLIP,	 FRONTEND_POS2X-35,   FRONTEND_POS2Y, _("Reverse Mouse"),true,false);
-	if( getInvertMouseStatus() )
-	{// flipped
-		addTextButton(FRONTEND_MFLIP_R, FRONTEND_POS2M-55,  FRONTEND_POS2Y, _("On"),true,false);
-	}
-	else
-	{	// not flipped
-		addTextButton(FRONTEND_MFLIP_R, FRONTEND_POS2M-55,  FRONTEND_POS2Y, _("Off"),true,false);
+		case FMV_2X:
+			addTextButton(FRONTEND_FMVMODE_R, FRONTEND_POS2M - 55,FRONTEND_POS2Y, _("2X"), true, false);
+			break;
+
+		case FMV_FULLSCREEN:
+			addTextButton(FRONTEND_FMVMODE_R, FRONTEND_POS2M - 55,FRONTEND_POS2Y, _("Fullscreen"), true, false);
+			break;
+
+		default:
+			ASSERT(!"invalid FMV mode", "Invalid FMV mode: %u", (unsigned int)war_GetFMVmode());
+			break;
 	}
 
 	////////////
@@ -641,21 +677,6 @@ BOOL startGameOptions2Menu(void)
 		addTextButton(FRONTEND_FOGTYPE_R,FRONTEND_POS4M-55,FRONTEND_POS4Y, _("Fog Of War"),true,false);
 	}
 
-//	////////////
-//	//sequence mode.
-	addTextButton(FRONTEND_SEQUENCE,	FRONTEND_POS6X-35,FRONTEND_POS6Y, _("Video Playback"),true,false);
-	if (war_GetSeqMode() == SEQ_FULL)
-	{
-		addTextButton(FRONTEND_SEQUENCE_R,	FRONTEND_POS6M-55,FRONTEND_POS6Y, _("Full"),true,false);
-	}
-	else if (war_GetSeqMode() == SEQ_SMALL)
-	{
-		addTextButton(FRONTEND_SEQUENCE_R,	FRONTEND_POS6M-55,FRONTEND_POS6Y, _("Windowed"),true,false);	}
-	else
-	{
-		addTextButton(FRONTEND_SEQUENCE_R,	FRONTEND_POS6M-55,FRONTEND_POS6Y, _("Minimal"),true,false);
-	}
-
 	////////////
 	//subtitle mode.
 	if(war_GetAllowSubtitles())
@@ -685,14 +706,14 @@ BOOL startGameOptions2Menu(void)
 
 	////////////
 	//shadows
-	addTextButton(FRONTEND_SHADOWS, FRONTEND_POS7X - 35, FRONTEND_POS7Y, _("Shadows"), true, false);
+	addTextButton(FRONTEND_SHADOWS, FRONTEND_POS6X - 35, FRONTEND_POS6Y, _("Shadows"), true, false);
 	if (getDrawShadows())
 	{
-		addTextButton(FRONTEND_SHADOWS_R, FRONTEND_POS7M - 55,  FRONTEND_POS7Y, _("On"), true, false);
+		addTextButton(FRONTEND_SHADOWS_R, FRONTEND_POS6M - 55,  FRONTEND_POS6Y, _("On"), true, false);
 	}
 	else
 	{	// not flipped
-		addTextButton(FRONTEND_SHADOWS_R, FRONTEND_POS7M - 55,  FRONTEND_POS7Y, _("Off"), true, false);
+		addTextButton(FRONTEND_SHADOWS_R, FRONTEND_POS6M - 55,  FRONTEND_POS6Y, _("Off"), true, false);
 	}
 
 	////////////
@@ -702,10 +723,11 @@ BOOL startGameOptions2Menu(void)
 	return true;
 }
 
-
+// Game options Menu
 BOOL runGameOptions2Menu(void)
 {
 	UDWORD id;
+	int mode = 0;
 
 	id = widgRunScreen(psWScreen);						// Run the current set of widgets
 	switch(id)
@@ -723,20 +745,6 @@ BOOL runGameOptions2Menu(void)
 			widgSetString(psWScreen,FRONTEND_SSHAKE_R, _("On"));
 		}
 		break;
-		break;
-	case FRONTEND_MFLIP:
-	case FRONTEND_MFLIP_R:
-		if( getInvertMouseStatus() )
-		{//	 flipped
-			setInvertMouseStatus(false);
-			widgSetString(psWScreen,FRONTEND_MFLIP_R, _("Off"));
-		}
-		else
-		{	// not flipped
-			setInvertMouseStatus(true);
-			widgSetString(psWScreen,FRONTEND_MFLIP_R, _("On"));
-		}
-		break;
 
 	case FRONTEND_FOGTYPE:
 	case FRONTEND_FOGTYPE_R:
@@ -787,22 +795,28 @@ BOOL runGameOptions2Menu(void)
 		}
 		break;
 
-	case FRONTEND_SEQUENCE:
-	case FRONTEND_SEQUENCE_R:
-		if( war_GetSeqMode() == SEQ_FULL )
+	case FRONTEND_FMVMODE:
+	case FRONTEND_FMVMODE_R:
+		switch (mode = war_GetFMVmode())
 		{
-			war_SetSeqMode(SEQ_SMALL);
-			widgSetString(psWScreen,FRONTEND_SEQUENCE_R, _("Windowed"));
-		}
-		else if( war_GetSeqMode() == SEQ_SMALL )
-		{
-			war_SetSeqMode(SEQ_SKIP);
-			widgSetString(psWScreen,FRONTEND_SEQUENCE_R, _("Minimal"));
-		}
-		else
-		{
-			war_SetSeqMode(SEQ_FULL);
-			widgSetString(psWScreen,FRONTEND_SEQUENCE_R, _("Full"));
+			case FMV_1X:
+				war_SetFMVmode(mode + 1);
+				widgSetString(psWScreen, FRONTEND_FMVMODE_R, _("2X"));
+				break;
+
+			case FMV_2X:
+				war_SetFMVmode(mode + 1);
+				widgSetString(psWScreen, FRONTEND_FMVMODE_R, _("Fullscreen"));
+				break;
+
+			case FMV_FULLSCREEN:
+				war_SetFMVmode(mode + 1);
+				widgSetString(psWScreen, FRONTEND_FMVMODE_R, _("1X"));
+				break;
+
+			default:
+				ASSERT(!"invalid FMV mode", "Invalid FMV mode: %u", (unsigned int)mode);
+				break;
 		}
 		break;
 
@@ -823,7 +837,7 @@ BOOL runGameOptions2Menu(void)
 
 
 // ////////////////////////////////////////////////////////////////////////////
-// Game Options Menu
+// Audio Options Menu
 BOOL startGameOptions3Menu(void)
 {
 	addBackdrop();
@@ -832,15 +846,15 @@ BOOL startGameOptions3Menu(void)
 
 	// 2d audio
 	addTextButton(FRONTEND_FX, FRONTEND_POS2X-25,FRONTEND_POS2Y, _("Voice Volume"),true,false);
-	addFESlider(FRONTEND_FX_SL, FRONTEND_BOTFORM, FRONTEND_POS2M, FRONTEND_POS2Y+5, AUDIO_VOL_MAX, (int)(sound_GetUIVolume() * 100.0), FRONTEND_FX );
+	addFESlider(FRONTEND_FX_SL, FRONTEND_BOTFORM, FRONTEND_POS2M, FRONTEND_POS2Y+5, AUDIO_VOL_MAX, sound_GetUIVolume() * 100.0);
 
 	// 3d audio
 	addTextButton(FRONTEND_3D_FX, FRONTEND_POS3X-25,FRONTEND_POS3Y, _("FX Volume"),true,false);
-	addFESlider(FRONTEND_3D_FX_SL, FRONTEND_BOTFORM, FRONTEND_POS3M, FRONTEND_POS3Y+5, AUDIO_VOL_MAX, (int)(sound_GetEffectsVolume() * 100.0), FRONTEND_3D_FX );
+	addFESlider(FRONTEND_3D_FX_SL, FRONTEND_BOTFORM, FRONTEND_POS3M, FRONTEND_POS3Y+5, AUDIO_VOL_MAX, sound_GetEffectsVolume() * 100.0);
 
 	// cd audio
 	addTextButton(FRONTEND_MUSIC, FRONTEND_POS4X-25,FRONTEND_POS4Y, _("Music Volume"),true,false);
-	addFESlider(FRONTEND_MUSIC_SL, FRONTEND_BOTFORM, FRONTEND_POS4M, FRONTEND_POS4Y+5, AUDIO_VOL_MAX, (int)(sound_GetMusicVolume() * 100.0), FRONTEND_MUSIC );
+	addFESlider(FRONTEND_MUSIC_SL, FRONTEND_BOTFORM, FRONTEND_POS4M, FRONTEND_POS4Y+5, AUDIO_VOL_MAX, sound_GetMusicVolume() * 100.0);
 
 	// quit.
 	addMultiBut(psWScreen, FRONTEND_BOTFORM, FRONTEND_QUIT, 10, 10, 30, 29, P_("menu", "Return"), IMAGE_RETURN, IMAGE_RETURN_HI, IMAGE_RETURN_HI);
@@ -851,7 +865,7 @@ BOOL startGameOptions3Menu(void)
 
 	return true;
 }
-
+// Audio Options
 BOOL runGameOptions3Menu(void)
 {
 	UDWORD id;
@@ -896,7 +910,7 @@ BOOL runGameOptions3Menu(void)
 	return true;
 }
 
-// Additional graphics game options menu
+// Graphics Options Menu
 BOOL startGameOptions4Menu(void)
 {
 	// Generate the resolution string
@@ -926,22 +940,22 @@ BOOL startGameOptions4Menu(void)
 	addTextButton(FRONTEND_RESOLUTION_R, FRONTEND_POS3M-55, FRONTEND_POS3Y, resolution, true, false);
 	widgSetString(psWScreen, FRONTEND_RESOLUTION_R, resolution);
 
-	// Cursor trapping
-	addTextButton(FRONTEND_TRAP, FRONTEND_POS4X-35, FRONTEND_POS4Y, _("Trap Cursor"), true, false);
+	// Texture size
+	addTextButton(FRONTEND_TEXTURESZ, FRONTEND_POS4X-35, FRONTEND_POS4Y, _("Texture size"), true, false);
+	addTextButton(FRONTEND_TEXTURESZ_R, FRONTEND_POS4M-55, FRONTEND_POS4Y, textureSize, true, false);
 
-	if (war_GetTrapCursor())
+	// Vsync
+	addTextButton(FRONTEND_VSYNC, FRONTEND_POS5X-35, FRONTEND_POS5Y, _("Vertical sync*"), true, false);
+
+	if (war_GetVsync())
 	{
-		addTextButton(FRONTEND_TRAP_R, FRONTEND_POS4M-55, FRONTEND_POS4Y, _("On"), true, false);
+		addTextButton(FRONTEND_VSYNC_R, FRONTEND_POS5M-55, FRONTEND_POS5Y, _("On"), true, false);
 	}
 	else
 	{
-		addTextButton(FRONTEND_TRAP_R, FRONTEND_POS4M-55, FRONTEND_POS4Y, _("Off"), true, false);
+		addTextButton(FRONTEND_VSYNC_R, FRONTEND_POS5M-55, FRONTEND_POS5Y, _("Off"), true, false);
 	}
 
-	// Texture size
-	addTextButton(FRONTEND_TEXTURESZ, FRONTEND_POS5X-35, FRONTEND_POS5Y, _("Texture size"), true, false);
-	addTextButton(FRONTEND_TEXTURESZ_R, FRONTEND_POS5M-55, FRONTEND_POS5Y, textureSize, true, false);
-
 	// Add a note about changes taking effect on restart for certain options
 	addTextButton(FRONTEND_TAKESEFFECT, FRONTEND_POS6X-35, FRONTEND_POS6Y, _("* Takes effect on game restart"), true, true);
 
@@ -950,7 +964,7 @@ BOOL startGameOptions4Menu(void)
 
 	return true;
 }
-
+// Graphics Options 
 BOOL runGameOptions4Menu(void)
 {
 	SDL_Rect **modes = SDL_ListModes(NULL, SDL_FULLSCREEN | SDL_HWSURFACE);
@@ -975,7 +989,7 @@ BOOL runGameOptions4Menu(void)
 		case FRONTEND_RESOLUTION:
 		case FRONTEND_RESOLUTION_R:
 		{
-			int current, count;
+			int current, count, oldcurrent;
 
 			// Get the current mode offset
 			for (count = 0, current = 0; modes[count]; count++)
@@ -988,8 +1002,13 @@ BOOL runGameOptions4Menu(void)
 			}
 
 			// Increment and clip if required
-			if (++current == count)
-				current = 0;
+			// Hide resolutions lower than Warzone can support
+			oldcurrent = current;
+			do
+			{
+				if (++current == count)
+					current = 0;
+			} while ((modes[current]->w < 640 || modes[current]->h < 480) && current != oldcurrent);
 
 			// Set the new width and height (takes effect on restart)
 			war_SetWidth(modes[current]->w);
@@ -1024,8 +1043,8 @@ BOOL runGameOptions4Menu(void)
 		{
 			int newTexSize = getTextureSize() * 2;
 
-			// Clip such that 32 <= size <= 128
-			if (newTexSize > 128)
+			// Clip such that 32 <= size <= 512
+			if (newTexSize > 512)
 			{
 				newTexSize = 32;
 			}
@@ -1042,6 +1061,22 @@ BOOL runGameOptions4Menu(void)
 			break;
 		}
 
+		case FRONTEND_VSYNC:
+		case FRONTEND_VSYNC_R:
+		{
+			if (war_GetVsync())
+			{
+				war_SetVsync(false);
+				widgSetString(psWScreen, FRONTEND_VSYNC_R, _("Off"));
+			}
+			else
+			{
+				war_SetVsync(true);
+				widgSetString(psWScreen, FRONTEND_VSYNC_R, _("On"));
+			}
+			break;
+		}
+
 		case FRONTEND_QUIT:
 			changeTitleMode(OPTIONS);
 			break;
@@ -1059,13 +1094,154 @@ BOOL runGameOptions4Menu(void)
 
 	return true;
 }
+// Mouse Menu options
+BOOL startGameOptions5Menu(void)
+{
+	addBackdrop();
+	addTopForm();
+	addBottomForm();
 
+	////////////
+	// mouseflip
+	addTextButton(FRONTEND_MFLIP,	 FRONTEND_POS2X-35,   FRONTEND_POS2Y, _("Reverse Rotation"),true,false);
+	if( getInvertMouseStatus() )
+	{	// flipped
+		addTextButton(FRONTEND_MFLIP_R, FRONTEND_POS2M-25,  FRONTEND_POS2Y, _("On"),true,false);
+	}
+	else
+	{	// not flipped
+		addTextButton(FRONTEND_MFLIP_R, FRONTEND_POS2M-25,  FRONTEND_POS2Y, _("Off"),true,false);
+	}
 
+	// Cursor trapping
+	addTextButton(FRONTEND_TRAP, FRONTEND_POS3X-35, FRONTEND_POS3Y, _("Trap Cursor"), true, false);
+
+	if (war_GetTrapCursor())
+	{
+		addTextButton(FRONTEND_TRAP_R, FRONTEND_POS3M-25, FRONTEND_POS3Y, _("On"), true, false);
+	}
+	else
+	{
+		addTextButton(FRONTEND_TRAP_R, FRONTEND_POS3M-25, FRONTEND_POS3Y, _("Off"), true, false);
+	}
+	
+	// Hardware / software cursor toggle
+	addTextButton(FRONTEND_CURSORMODE, FRONTEND_POS4X-35, FRONTEND_POS4Y, _("Colored Cursors *"), true, false);
+
+	if (war_GetColouredCursor())
+	{
+		addTextButton(FRONTEND_CURSORMODE_R, FRONTEND_POS4M-25, FRONTEND_POS4Y, _("On"), true, false);
+	}
+	else
+	{
+		addTextButton(FRONTEND_CURSORMODE_R, FRONTEND_POS4M-25, FRONTEND_POS4Y, _("Off"), true, false);
+	}
+
+	////////////
+	// left-click orders
+	addTextButton(FRONTEND_MBUTTONS,	 FRONTEND_POS2X-35,   FRONTEND_POS5Y, _("Switch Mouse Buttons"),true,false);
+	if( getRightClickOrders() )
+	{	// right-click orders
+		addTextButton(FRONTEND_MBUTTONS_R, FRONTEND_POS2M-25,  FRONTEND_POS5Y, _("On"),true,false);
+	}
+	else
+	{	// left-click orders
+		addTextButton(FRONTEND_MBUTTONS_R, FRONTEND_POS2M-25,  FRONTEND_POS5Y, _("Off"),true,false);
+	}
+
+	addTextButton(FRONTEND_TAKESEFFECT, FRONTEND_POS6X-35, FRONTEND_POS6Y, _("* May negatively affect performance"), true, true);
+
+	// Quit/return
+	addMultiBut(psWScreen, FRONTEND_BOTFORM, FRONTEND_QUIT, 10, 10, 30, 29, P_("menu", "Return"), IMAGE_RETURN, IMAGE_RETURN_HI, IMAGE_RETURN_HI);
+
+	return true;
+}
+// Mouse Menu 
+BOOL runGameOptions5Menu(void)
+{
+	UDWORD id = widgRunScreen(psWScreen);
+
+	switch (id)
+	{
+
+		case FRONTEND_MFLIP:
+		case FRONTEND_MFLIP_R:
+			if( getInvertMouseStatus() )
+			{//	 flipped
+				setInvertMouseStatus(false);
+				widgSetString(psWScreen,FRONTEND_MFLIP_R, _("Off"));
+			}
+			else
+			{	// not flipped
+				setInvertMouseStatus(true);
+				widgSetString(psWScreen,FRONTEND_MFLIP_R, _("On"));
+			}
+			break;
+		case FRONTEND_TRAP:
+		case FRONTEND_TRAP_R:
+			if (war_GetTrapCursor())
+			{
+				war_SetTrapCursor(false);
+				widgSetString(psWScreen, FRONTEND_TRAP_R, _("Off"));
+			}
+			else
+			{
+				war_SetTrapCursor(true);
+				widgSetString(psWScreen, FRONTEND_TRAP_R, _("On"));
+			}
+			break;
+
+		case FRONTEND_CURSORMODE:
+		case FRONTEND_CURSORMODE_R:
+			if (war_GetColouredCursor())
+			{
+				war_SetColouredCursor(false);
+				widgSetString(psWScreen, FRONTEND_CURSORMODE_R, _("Off"));
+			}
+			else
+			{
+				war_SetColouredCursor(true);
+				widgSetString(psWScreen, FRONTEND_CURSORMODE_R, _("On"));
+			}
+			break;
+
+		case FRONTEND_MBUTTONS:
+		case FRONTEND_MBUTTONS_R:
+			if( getRightClickOrders() )
+			{
+				setRightClickOrders(false);
+				widgSetString(psWScreen,FRONTEND_MBUTTONS_R, _("Off"));
+			}
+			else
+			{
+				setRightClickOrders(true);
+				widgSetString(psWScreen,FRONTEND_MBUTTONS_R, _("On"));
+			}
+			break;
+
+		case FRONTEND_QUIT:
+			changeTitleMode(OPTIONS);
+			break;
+
+		default:
+			break;
+	}
+
+	if (CancelPressed())
+	{
+		changeTitleMode(OPTIONS);
+	}
+
+	widgDisplayScreen(psWScreen);
+
+	return true;
+}
 // ////////////////////////////////////////////////////////////////////////////
 // Game Options Menu
 BOOL startGameOptionsMenu(void)
 {
 	UDWORD	w, h;
+	int playercolor;
 
 	addBackdrop();
 	addTopForm();
@@ -1089,7 +1265,7 @@ BOOL startGameOptionsMenu(void)
 
 	// Scroll speed
 	addTextButton(FRONTEND_SCROLLSPEED, FRONTEND_POS3X-25, FRONTEND_POS3Y, _("Scroll Speed"), true, false);
-	addFESlider(FRONTEND_SCROLLSPEED_SL, FRONTEND_BOTFORM, FRONTEND_POS3M, FRONTEND_POS3Y+5, 16, scroll_speed_accel / 100, FRONTEND_SCROLLSPEED);
+	addFESlider(FRONTEND_SCROLLSPEED_SL, FRONTEND_BOTFORM, FRONTEND_POS3M, FRONTEND_POS3Y+5, 16, scroll_speed_accel / 100);
 
 	// Colour stuff
 	w = iV_GetImageWidth(FrontImages, IMAGE_PLAYER0);
@@ -1105,7 +1281,15 @@ BOOL startGameOptionsMenu(void)
 	addTextButton(FRONTEND_LANGUAGE,  FRONTEND_POS2X - 25, FRONTEND_POS5Y, _("Language"), true, false);
 	addTextButton(FRONTEND_LANGUAGE_R,  FRONTEND_POS2M - 25, FRONTEND_POS5Y, getLanguageName(), true, false);
 
-	widgSetButtonState(psWScreen, FE_P0 + getPlayerColour(0), WBUT_LOCK);
+	// FIXME: if playercolor = 1-3, then we Assert in widgSetButtonState() since we don't define FE_P1 - FE_P3
+	// I assume the reason is that in SP games, those are reserved for the AI?  Valid values are 0, 4-7.
+	// This is a workaround, until we find what is setting that to 1-3.  See configuration.c:701
+	playercolor = getPlayerColour(0);
+	if (playercolor >= 1 && playercolor <= 3)
+	{
+		playercolor = 0;
+	}
+	widgSetButtonState(psWScreen, FE_P0 + playercolor, WBUT_LOCK);
 	addTextButton(FRONTEND_COLOUR, FRONTEND_POS4X-25, FRONTEND_POS4Y, _("Unit Colour"), true, false);
 
 	// Quit
@@ -1388,7 +1572,7 @@ void addTextButton(UDWORD id,  UDWORD PosX, UDWORD PosY, const char *txt,BOOL bA
 }
 
 // ////////////////////////////////////////////////////////////////////////////
-void addFESlider(UDWORD id, UDWORD parent, UDWORD x,UDWORD y,UDWORD stops,UDWORD pos,UDWORD attachID )
+void addFESlider(UDWORD id, UDWORD parent, UDWORD x, UDWORD y, UDWORD stops, UDWORD pos)
 {
 	W_SLDINIT		sSldInit;
 
