@@ -1,7 +1,7 @@
 /*
 	This file is part of Warzone 2100.
 	Copyright (C) 1999-2004  Eidos Interactive
-	Copyright (C) 2005-2007  Warzone Resurrection Project
+	Copyright (C) 2005-2009  Warzone Resurrection Project
 
 	Warzone 2100 is free software; you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -80,7 +80,6 @@
 #include "multiplay.h"
 #include "projectile.h"
 #include "radar.h"
-#include "raycast.h"
 #include "lib/framework/cursors.h"
 #include "scriptextern.h"
 #include "scripttabs.h"
@@ -184,10 +183,10 @@ void registerSearchPath( const char path[], unsigned int priority )
 {
 	wzSearchPath * curSearchPath = searchPathRegistry, * tmpSearchPath = NULL;
 
-	tmpSearchPath = (wzSearchPath*)malloc(sizeof(wzSearchPath));
-	strcpy( tmpSearchPath->path, path );
-	if( path[strlen(path)-1] != *PHYSFS_getDirSeparator() )
-		strcat( tmpSearchPath->path, PHYSFS_getDirSeparator() );
+	tmpSearchPath = (wzSearchPath*)malloc(sizeof(*tmpSearchPath));
+	sstrcpy(tmpSearchPath->path, path);
+	if (path[strlen(path)-1] != *PHYSFS_getDirSeparator())
+		sstrcat(tmpSearchPath->path, PHYSFS_getDirSeparator());
 	tmpSearchPath->priority = priority;
 
 	debug( LOG_WZ, "registerSearchPath: Registering %s at priority %i", path, priority );
@@ -226,7 +225,7 @@ void registerSearchPath( const char path[], unsigned int priority )
  * \brief Rebuilds the PHYSFS searchPath with mode specific subdirs
  *
  * Priority:
- * maps > mods > plain_dir > warzone.wz
+ * maps > mods > base > base.wz
  */
 BOOL rebuildSearchPath( searchPathMode mode, BOOL force )
 {
@@ -247,41 +246,43 @@ BOOL rebuildSearchPath( searchPathMode mode, BOOL force )
 		switch ( mode )
 		{
 			case mod_clean:
-				debug( LOG_WZ, "rebuildSearchPath: Cleaning up" );
+				debug(LOG_WZ, "Cleaning up");
 
 				while( curSearchPath )
 				{
 #ifdef DEBUG
-					debug( LOG_WZ, "rebuildSearchPath: Removing [%s] from search path", curSearchPath->path );
+					debug(LOG_WZ, "Removing [%s] from search path", curSearchPath->path);
 #endif // DEBUG
 					// Remove maps and mods
 					removeSubdirs( curSearchPath->path, "maps", NULL );
+					removeSubdirs( curSearchPath->path, "mods/music", NULL );
 					removeSubdirs( curSearchPath->path, "mods/global", global_mods );
-					removeSubdirs( curSearchPath->path, "mods/global/autoload", NULL );
 					removeSubdirs( curSearchPath->path, "mods/campaign", campaign_mods );
-					removeSubdirs( curSearchPath->path, "mods/campaign/autoload", NULL );
 					removeSubdirs( curSearchPath->path, "mods/multiplay", multiplay_mods );
-					removeSubdirs( curSearchPath->path, "mods/multiplay/autoload", NULL );
 
 					// Remove multiplay patches
-					strlcpy(tmpstr, curSearchPath->path, sizeof(tmpstr));
-					strlcat(tmpstr, "mp", sizeof(tmpstr));
+					sstrcpy(tmpstr, curSearchPath->path);
+					sstrcat(tmpstr, "mp");
 					PHYSFS_removeFromSearchPath( tmpstr );
-					strlcpy(tmpstr, curSearchPath->path, sizeof(tmpstr));
-					strlcat(tmpstr, "mp.wz", sizeof(tmpstr));
+					sstrcpy(tmpstr, curSearchPath->path);
+					sstrcat(tmpstr, "mp.wz");
 					PHYSFS_removeFromSearchPath( tmpstr );
 
 					// Remove plain dir
-					if(!PHYSFS_removeFromSearchPath( curSearchPath->path ))
-					{
-						debug(LOG_WARNING, "*Could not remove %s, because %s", curSearchPath->path,  PHYSFS_getLastError());
-					}
+					PHYSFS_removeFromSearchPath( curSearchPath->path );
 
-					// Remove warzone.wz
-					strlcpy(tmpstr, curSearchPath->path, sizeof(tmpstr));
-					strlcat(tmpstr, "warzone.wz", sizeof(tmpstr));
+					// Remove base files
+					sstrcpy(tmpstr, curSearchPath->path);
+					sstrcat(tmpstr, "base");
+					PHYSFS_removeFromSearchPath( tmpstr );
+					sstrcpy(tmpstr, curSearchPath->path);
+					sstrcat(tmpstr, "base.wz");
 					PHYSFS_removeFromSearchPath( tmpstr );
 
+					// remove video search path as well
+					sstrcpy(tmpstr, curSearchPath->path);
+					sstrcat(tmpstr, "sequences.wz");
+					PHYSFS_removeFromSearchPath( tmpstr );
 					curSearchPath = curSearchPath->higherPriority;
 				}
 				break;
@@ -291,26 +292,33 @@ BOOL rebuildSearchPath( searchPathMode mode, BOOL force )
 				while( curSearchPath )
 				{
 #ifdef DEBUG
-					debug( LOG_WZ, "rebuildSearchPath: Adding [%s] to search path", curSearchPath->path );
+					debug(LOG_WZ, "Adding [%s] to search path", curSearchPath->path);
 #endif // DEBUG
 					// Add global and campaign mods
 					PHYSFS_addToSearchPath( curSearchPath->path, PHYSFS_APPEND );
 
+					addSubdirs( curSearchPath->path, "mods/music", PHYSFS_APPEND, NULL );
 					addSubdirs( curSearchPath->path, "mods/global", PHYSFS_APPEND, global_mods );
-					addSubdirs( curSearchPath->path, "mods/global/autoload", PHYSFS_APPEND, NULL );
 					addSubdirs( curSearchPath->path, "mods/campaign", PHYSFS_APPEND, campaign_mods );
-					addSubdirs( curSearchPath->path, "mods/campaign/autoload", PHYSFS_APPEND, NULL );
 					if (!PHYSFS_removeFromSearchPath( curSearchPath->path ))
 					{
-						debug(LOG_ERROR, "Failed to remove path %s again, because of %s", curSearchPath->path, PHYSFS_getLastError());
+						info("* Failed to remove path %s again", curSearchPath->path);
 					}
 
 					// Add plain dir
 					PHYSFS_addToSearchPath( curSearchPath->path, PHYSFS_APPEND );
 
-					// Add warzone.wz
-					strlcpy(tmpstr, curSearchPath->path, sizeof(tmpstr));
-					strlcat(tmpstr, "warzone.wz", sizeof(tmpstr));
+					// Add base files
+					sstrcpy(tmpstr, curSearchPath->path);
+					sstrcat(tmpstr, "base");
+					PHYSFS_addToSearchPath( tmpstr, PHYSFS_APPEND );
+					sstrcpy(tmpstr, curSearchPath->path);
+					sstrcat(tmpstr, "base.wz");
+					PHYSFS_addToSearchPath( tmpstr, PHYSFS_APPEND );
+
+					// add the video file
+					sstrcpy(tmpstr, curSearchPath->path);
+					sstrcat(tmpstr, "sequences.wz");
 					PHYSFS_addToSearchPath( tmpstr, PHYSFS_APPEND );
 
 					curSearchPath = curSearchPath->higherPriority;
@@ -322,43 +330,50 @@ BOOL rebuildSearchPath( searchPathMode mode, BOOL force )
 				while( curSearchPath )
 				{
 #ifdef DEBUG
-					debug( LOG_WZ, "rebuildSearchPath: Adding [%s] to search path", curSearchPath->path );
+					debug(LOG_WZ, "Adding [%s] to search path", curSearchPath->path);
 #endif // DEBUG
 					// Add maps and global and multiplay mods
 					PHYSFS_addToSearchPath( curSearchPath->path, PHYSFS_APPEND );
 					addSubdirs( curSearchPath->path, "maps", PHYSFS_APPEND, NULL );
+					addSubdirs( curSearchPath->path, "mods/music", PHYSFS_APPEND, NULL );
 					addSubdirs( curSearchPath->path, "mods/global", PHYSFS_APPEND, global_mods );
-					addSubdirs( curSearchPath->path, "mods/global/autoload", PHYSFS_APPEND, NULL );
 					addSubdirs( curSearchPath->path, "mods/multiplay", PHYSFS_APPEND, multiplay_mods );
-					addSubdirs( curSearchPath->path, "mods/multiplay/autoload", PHYSFS_APPEND, NULL );
 					PHYSFS_removeFromSearchPath( curSearchPath->path );
 
 					// Add multiplay patches
-					strlcpy(tmpstr, curSearchPath->path, sizeof(tmpstr));
-					strlcat(tmpstr, "mp", sizeof(tmpstr));
+					sstrcpy(tmpstr, curSearchPath->path);
+					sstrcat(tmpstr, "mp");
 					PHYSFS_addToSearchPath( tmpstr, PHYSFS_APPEND );
-					strlcpy( tmpstr, curSearchPath->path, sizeof(tmpstr));
-					strlcat( tmpstr, "mp.wz", sizeof(tmpstr));
+					sstrcpy(tmpstr, curSearchPath->path);
+					sstrcat(tmpstr, "mp.wz");
 					PHYSFS_addToSearchPath( tmpstr, PHYSFS_APPEND );
 
 					// Add plain dir
 					PHYSFS_addToSearchPath( curSearchPath->path, PHYSFS_APPEND );
 
-					// Add warzone.wz
-					strlcpy(tmpstr, curSearchPath->path, sizeof(tmpstr));
-					strlcat(tmpstr, "warzone.wz", sizeof(tmpstr));
+					// Add base files
+					sstrcpy(tmpstr, curSearchPath->path);
+					sstrcat(tmpstr, "base");
+					PHYSFS_addToSearchPath( tmpstr, PHYSFS_APPEND );
+					sstrcpy(tmpstr, curSearchPath->path);
+					sstrcat(tmpstr, "base.wz");
+					PHYSFS_addToSearchPath( tmpstr, PHYSFS_APPEND );
+
+					// add the video file
+					sstrcpy(tmpstr, curSearchPath->path);
+					sstrcat(tmpstr, "sequences.wz");
 					PHYSFS_addToSearchPath( tmpstr, PHYSFS_APPEND );
 
 					curSearchPath = curSearchPath->higherPriority;
 				}
 				break;
 			default:
-				debug( LOG_ERROR, "rebuildSearchPath: Can't switch to unknown mods %i", mode );
+				debug(LOG_ERROR, "Can't switch to unknown mods %i", mode);
 				return false;
 		}
 
 		// User's home dir must be first so we allways see what we write
-		PHYSFS_removeFromSearchPath( PHYSFS_getWriteDir() );
+		PHYSFS_removeFromSearchPath(PHYSFS_getWriteDir());
 		PHYSFS_addToSearchPath( PHYSFS_getWriteDir(), PHYSFS_PREPEND );
 
 #ifdef DEBUG
@@ -405,26 +420,26 @@ BOOL systemInitialise(void)
 	buildMapList();
 
 	// Initialize render engine
-	war_SetFog(false);
+	war_SetFog(war_GetFog());		// Set Fog mode based on user preferences
 	if (!pie_Initialise()) {
 		debug(LOG_ERROR, "Unable to initialise renderer");
 		return false;
 	}
-
-	pie_SetGammaValue((float)gammaValue / 20.0f);
-
+	
 	if ( war_getSoundEnabled() )
 	{
-		if( !audio_Init(droidAudioTrackStopped) )
-			debug( LOG_SOUND, "Couldn't initialise audio system: continuing without audio\n" );
+		if (!audio_Init(droidAudioTrackStopped))
+		{
+			debug(LOG_SOUND, "Could not initialise audio system: Continuing without audio");
+		}
+		if (war_GetMusicEnabled())
+		{
+			cdAudio_Open(UserMusicPath);
+		}
 	}
 	else
 	{
-		debug( LOG_SOUND, "Sound disabled!" );
-	}
-
-	if (war_GetPlayAudioCDs()) {
-		cdAudio_Open(UserMusicPath);
+		debug(LOG_SOUND, "Sound disabled");
 	}
 
 	if (!dataInitLoadFuncs()) // Pass all the data loading functions to the framework library
@@ -432,20 +447,11 @@ BOOL systemInitialise(void)
 		return false;
 	}
 
-	if (!rayInitialise()) /* Initialise the ray tables */
-	{
-		return false;
-	}
-
 	if (!fpathInitialise())
 	{
 		return false;
 	}
 
-#ifdef ARROWS
-	arrowInit();
-#endif
-
 	// Initialize the iVis text rendering module
 	iV_TextInit();
 
@@ -462,9 +468,6 @@ BOOL systemInitialise(void)
 void systemShutdown(void)
 {
 //	unsigned int i;
-#ifdef ARROWS
-	arrowShutDown();
-#endif
 
 	keyClearMappings();
 
@@ -478,11 +481,8 @@ void systemShutdown(void)
 
 	debug(LOG_MAIN, "shutting down audio subsystems");
 
-	if (war_GetPlayAudioCDs()) {
-		debug(LOG_MAIN, "shutting down CD audio");
-		cdAudio_Stop();
-		cdAudio_Close();
-	}
+	debug(LOG_MAIN, "shutting down CD audio");
+	cdAudio_Close();
 
 	if ( audio_Disabled() == false && !audio_Shutdown() )
 	{
@@ -495,6 +495,8 @@ void systemShutdown(void)
 	widgShutDown();
 	pie_TerrainCleanup();
 
+	fpathShutdown();
+
 	return;
 }
 
@@ -525,7 +527,7 @@ init_ObjectDead( void * psObj )
 
 			default:
 				debug( LOG_ERROR, "init_ObjectAnimRemoved: unrecognised object type" );
-				abort();
+				return false;
 		}
 	}
 
@@ -620,10 +622,7 @@ BOOL frontendInitialise(const char *ResourceFile)
 	gameTimeInit();
 
 	// hit me with some funky beats....
-	if (war_GetPlayAudioCDs())
-	{
-		cdAudio_PlayTrack(playlist_frontend); // frontend music
-	}
+	cdAudio_PlayTrack(SONG_FRONTEND);
 
 	return true;
 }
@@ -635,14 +634,14 @@ BOOL frontendShutdown(void)
 
 	saveConfig();// save settings to registry.
 
-	if (!mechShutdown())
+	if (!mechanicsShutdown())
 	{
 		return false;
 	}
 
 	releasePlayerPower();
 
-	intShutDown();
+	interfaceShutDown();
 	scrShutDown();
 
 	//do this before shutting down the iV library
@@ -848,7 +847,7 @@ BOOL stageOneShutDown(void)
 	debug(LOG_TEXTURE, "=== stageOneShutDown ===");
 	pie_TexShutDown();
 	// Use mod_multiplay as the default (campaign might have set it to mod_singleplayer)
-	rebuildSearchPath( mod_multiplay, true );	
+	rebuildSearchPath( mod_multiplay, true );
 	pie_TexInit(); // restart it
 
 	initMiscVars();
@@ -886,19 +885,15 @@ BOOL stageTwoInitialise(void)
 		return false;
 	}
 
-	LOADBARCALLBACK();	//	loadingScreenCallback();
-
 	if(!InitRadar()) 	// After resLoad cause it needs the game palette initialised.
 	{
 		return false;
 	}
 
-	LOADBARCALLBACK();	//	loadingScreenCallback();
-
 	if(!initMiscImds())			/* Set up the explosions */
 	{
 		iV_ShutDown();
-		debug( LOG_ERROR, "Can't find all the explosions PCX's" );
+		debug( LOG_ERROR, "Can't find all the explosions graphics?" );
 		abort();
 		return false;
 	}
@@ -908,8 +903,6 @@ BOOL stageTwoInitialise(void)
 		return false;
 	}
 
-	LOADBARCALLBACK();	//	loadingScreenCallback();
-
 #ifdef BUCKET
 	if ( !bucketSetupList() )	/* reset object list */
 	{
@@ -920,15 +913,11 @@ BOOL stageTwoInitialise(void)
    	/* Shift the interface initialisation here temporarily so that it
    		can pick up the stats after they have been loaded */
 
-	LOADBARCALLBACK();	//	loadingScreenCallback();
-
 	if (!intInitialise())
 	{
 		return false;
 	}
 
-	LOADBARCALLBACK();	//	loadingScreenCallback();
-
 	if (!initMessage())			/* Initialise the message heaps */
 	{
 		return false;
@@ -940,10 +929,8 @@ BOOL stageTwoInitialise(void)
 	}
 
 	// keymappings
-	LOADBARCALLBACK();	//	loadingScreenCallback();
 	keyClearMappings();
 	keyInitMappings(false);
-	LOADBARCALLBACK();	//	loadingScreenCallback();
 
 	// Set the default uncoloured cursor here, since it looks slightly
 	// better for menus and such.
@@ -964,9 +951,7 @@ BOOL stageTwoShutDown(void)
 {
 	debug(LOG_WZ, "== stageTwoShutDown ==");
 
-	if (war_GetPlayAudioCDs()) {
-		cdAudio_Stop();
-	}
+	cdAudio_Stop();
 
 	freeAllStructs();
 	freeAllDroids();
@@ -978,7 +963,7 @@ BOOL stageTwoShutDown(void)
 		return false;
 	}
 
-	if (!mechShutdown())
+	if (!mechanicsShutdown())
 	{
 		return false;
 	}
@@ -988,7 +973,7 @@ BOOL stageTwoShutDown(void)
 		return false;
 	}
 
-	intShutDown();
+	interfaceShutDown();
 
 	cmdDroidShutDown();
 
@@ -1070,10 +1055,11 @@ BOOL stageThreeInitialise(void)
 
 	driveInitVars(false);
 	displayInitVars();
+	resizeRadar();
 
 	setAllPauseStates(false);
 
-	/* decide if we have to create teams ONLY in multiplayer mode!*/
+	/* decide if we have to create teams, ONLY in multiplayer mode!*/
 	if( bMultiPlayer && game.alliance == ALLIANCES_TEAMS && game.type == SKIRMISH)
 	{
 		createTeamAlliances();
@@ -1087,14 +1073,14 @@ BOOL stageThreeInitialise(void)
 				for(psStr=apsStructLists[i]; psStr; psStr=psStr->psNext)
 				{
 					if(aiCheckAlliances(psStr->player,selectedPlayer))
-					visTilesUpdate((BASE_OBJECT *)psStr);
+					visTilesUpdate((BASE_OBJECT *)psStr, rayTerrainCallback);
 				}
 
 				/* Droids */
 				for(psDroid=apsDroidLists[i]; psDroid; psDroid=psDroid->psNext)
 				{
 					if(aiCheckAlliances(psDroid->player,selectedPlayer))
-					visTilesUpdate((BASE_OBJECT *)psDroid);
+					visTilesUpdate((BASE_OBJECT *)psDroid, rayTerrainCallback);
 				}
 			}
 		}
@@ -1186,9 +1172,7 @@ BOOL saveGameReset(void)
 {
 	debug(LOG_MAIN, "saveGameReset");
 
-	if (war_GetPlayAudioCDs()) {
-		cdAudio_Stop();
-	}
+	cdAudio_Stop();
 
 	freeAllStructs();
 	freeAllDroids();
