@@ -1,7 +1,7 @@
 /*
 	This file is part of Warzone 2100.
 	Copyright (C) 1999-2004  Eidos Interactive
-	Copyright (C) 2005-2007  Warzone Resurrection Project
+	Copyright (C) 2005-2009  Warzone Resurrection Project
 
 	Warzone 2100 is free software; you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -59,10 +59,11 @@
 
 #include "multiplay.h"
 #include "lib/sound/cdaudio.h"
+#include "lib/sequence/sequence.h"
 
 #include "scriptextern.h"
 
-#define NO_VIDEO
+//#define NO_VIDEO
 
 /* Intelligence Map screen IDs */
 #define IDINTMAP_MSGFORM		6001	//The intelligence map tabbed form
@@ -1082,9 +1083,7 @@ void intRemoveMessageView(BOOL animated)
 
 		//stop the video
 		psViewResearch = (VIEW_RESEARCH *)Form->pUserData;
-		seq_RenderVideoToBuffer(NULL, psViewResearch->sequenceName,
-			gameTime2, SEQUENCE_KILL);
-
+		seq_RenderVideoToBuffer(psViewResearch->sequenceName, SEQUENCE_KILL);
 
 		if (animated)
 		{
@@ -1292,7 +1291,7 @@ void intDisplayPIEView(WIDGET *psWidget, UDWORD xOffset, UDWORD yOffset, WZ_DECL
 
 #ifndef NO_VIDEO
 /* displays the FLIC view for the current message */
-void intDisplayFLICView(WIDGET *psWidget, UDWORD xOffset, UDWORD yOffset, PIELIGHT *pColours)
+void intDisplayFLICView(WIDGET *psWidget, UDWORD xOffset, UDWORD yOffset, WZ_DECL_UNUSED PIELIGHT *pColours)
 {
 
 	W_TABFORM		*Form = (W_TABFORM*)psWidget;
@@ -1316,19 +1315,18 @@ void intDisplayFLICView(WIDGET *psWidget, UDWORD xOffset, UDWORD yOffset, PIELIG
 		x1 = x0 + Form->width;
 		y1 = y0 + Form->height;
 
-
 		if (((VIEWDATA *)psMessage->pViewData)->type != VIEW_RES)
 		{
 			ASSERT( false, "intDisplayFLICView: Invalid message type" );
 			return;
 		}
-		//render a frame of the current movie
+
+		RenderWindowFrame(FRAME_NORMAL, x0, y0, x1 - x0, y1 - y0);
 		psViewResearch = (VIEW_RESEARCH *)((VIEWDATA *)psCurrentMsg->pViewData)->pData;
-		seq_RenderVideoToBuffer(NULL, psViewResearch->sequenceName,
-			gameTime2, SEQUENCE_HOLD);
-		//download to screen now
-		seq_BlitBufferToScreen((SBYTE *)rendSurface.buffer, rendSurface.scantable[1],
-			x0, y0);
+		// set the dimensions to window size & position
+		seq_SetDisplaySize(192, 168, x0, y0);
+		//render a frame of the current movie *must* force above resolution!
+		seq_RenderVideoToBuffer(psViewResearch->sequenceName, SEQUENCE_HOLD);
 		CloseButtonRender();
 	}
 
@@ -1361,7 +1359,7 @@ void intDisplayTEXTView(WIDGET *psWidget, UDWORD xOffset, UDWORD yOffset, WZ_DEC
 		/* Get the travel to the next line */
 		linePitch = iV_GetTextLineSize();
 		/* Fix for spacing.... */
-		linePitch+=6;
+		linePitch+=3;
 		ty+=3;
 		/* Fix for spacing.... */
 
@@ -1377,7 +1375,7 @@ void intDisplayTEXTView(WIDGET *psWidget, UDWORD xOffset, UDWORD yOffset, WZ_DEC
 				return;
 			}
 			//need to check the string will fit!
-			iV_DrawText(((VIEWDATA *)psMessage->pViewData)->ppTextMsg[i], x0 + TEXT_XINDENT,
+			iV_DrawText(_(((VIEWDATA *)psMessage->pViewData)->ppTextMsg[i]), x0 + TEXT_XINDENT,
 				(ty + TEXT_YINDENT*3) + (i * linePitch));
 		}
 	}
@@ -1393,19 +1391,21 @@ void addVideoText(SEQ_DISPLAY *psSeqDisplay, UDWORD sequence)
 	{
 		debug( LOG_NEVER, "avt seq=%d [%s]\n", sequence, psSeqDisplay->ppTextMsg[0] );
 		//add each message, first at the top
+		// FIXME We should perhaps get font size, and use that to calculate offset(s) ?
 		x = VIDEO_TEXT_TOP_X;
 		y = VIDEO_TEXT_TOP_Y;
 
 
-		seq_AddTextForVideo(psSeqDisplay->ppTextMsg[0], x, y, TEXT_START_FRAME, TEXT_END_FRAME, false); //startframe endFrame
+		seq_AddTextForVideo(psSeqDisplay->ppTextMsg[0], x, y, TEXT_START_FRAME, TEXT_END_FRAME, SEQ_TEXT_POSITION); //startframe endFrame
 
 		//add each message, the rest at the bottom
 		x = VIDEO_TEXT_BOTTOM_X;
-		y = VIDEO_TEXT_BOTTOM_Y;
+		// calculate the real bottom... NOTE, game assumes all videos to be 640x480
+		y = (double)pie_GetVideoBufferHeight() / 480. * (double)VIDEO_TEXT_BOTTOM_Y;
 		i = 1;
 		while (i < psSeqDisplay->numText)
 		{
-			seq_AddTextForVideo(psSeqDisplay->ppTextMsg[i], x, y, TEXT_START_FRAME, TEXT_END_FRAME, false); //startframe endFrame
+			seq_AddTextForVideo(psSeqDisplay->ppTextMsg[i], x, y, TEXT_START_FRAME, TEXT_END_FRAME, SEQ_TEXT_POSITION); //startframe endFrame
 			//initialise after the first setting
 			x = y = 0;
 			i++;
@@ -1466,29 +1466,21 @@ void resetIntelligencePauseState(void)
 	}
 }
 
-
-// play this message immediately, but definitely donot tell the intelligence screen to start
-
+/** Play an intelligence message.
+ * This function is used from scripts to give updates to the mission.
+ */
 void displayImmediateMessage(MESSAGE *psMessage)
 {
 	/*
 		This has to be changed to support a script calling a message in the intellegence screen
 	*/
 
-#ifdef NO_VIDEO
 	psCurrentMsg = psMessage;
-	/* so we lied about definately not starting the intelligence screen */
-	addIntelScreen();
-
-	// Using software cursors (when on) for these menus due to a bug in SDL's SDL_ShowCursor()
-	pie_SetMouse(CURSOR_DEFAULT, war_GetColouredCursor());
-
-	/* addIntelScreen() (via addIntelMap()) actually starts
-	 * playing psCurrentMsg automatically */
-	return;
-#endif
-
 	StartMessageSequences(psMessage,true);
+	// remind the player that the message can be seen again from
+	// the intelligence screen, and flash the button
+	addConsoleMessage(_("New Intelligence Report"), CENTRE_JUSTIFY, SYSTEM_MESSAGE);
+	flashReticuleButton(IDRET_INTEL_MAP);
 }
 
 
