@@ -1,7 +1,7 @@
 /*
 	This file is part of Warzone 2100.
 	Copyright (C) 1999-2004  Eidos Interactive
-	Copyright (C) 2005-2007  Warzone Resurrection Project
+	Copyright (C) 2005-2009  Warzone Resurrection Project
 
 	Warzone 2100 is free software; you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -27,11 +27,12 @@
 
 #include <ctype.h>
 #include <physfs.h>
+#include <time.h>
 
 #include "lib/framework/frame.h"
 #include "lib/framework/strres.h"
 #include "lib/framework/input.h"
-#include "lib/framework/input.h"
+#include "lib/framework/stdio_ext.h"
 #include "lib/widget/button.h"
 #include "lib/widget/editbox.h"
 #include "lib/widget/widget.h"
@@ -59,25 +60,26 @@
 #include "lib/gamelib/gtime.h"
 //======================================================================================
 //--------------------------------
-#define totalslots 20			//saves slots.. was 10 , now 20   *Away with hard coding values!* -Q
-#define totalslotspace 64		//guessing 64 max chars for filename.
+#define totalslots 36			// saves slots
+#define slotsInColumn 12		// # of slots in a column
+#define totalslotspace 64		// guessing 64 max chars for filename.
 //--------------------------------
 // ////////////////////////////////////////////////////////////////////////////
-#define LOADSAVE_X				130	+ D_W
-#define LOADSAVE_Y				30	+ D_H		//was 170 -Q
-#define LOADSAVE_W				380
-#define LOADSAVE_H				240	//was 200 -Q
+#define LOADSAVE_X				D_W + 16
+#define LOADSAVE_Y				D_H + 5
+#define LOADSAVE_W				610
+#define LOADSAVE_H				220
 
 #define MAX_SAVE_NAME			60
 
 
-#define LOADSAVE_HGAP			9		//from 5 to 9 -Q
-#define LOADSAVE_VGAP			9		//from 5 to 9 -Q
-#define LOADSAVE_BANNER_DEPTH	40 		//was 25 top banner which displays either load or save
+#define LOADSAVE_HGAP			9
+#define LOADSAVE_VGAP			9
+#define LOADSAVE_BANNER_DEPTH	40 		//top banner which displays either load or save
 
 
-#define LOADENTRY_W				(LOADSAVE_W -(3 * LOADSAVE_HGAP)) /2
-#define LOADENTRY_H				(LOADSAVE_H -(6 * LOADSAVE_VGAP )- (LOADSAVE_BANNER_DEPTH+LOADSAVE_VGAP) ) /5
+#define LOADENTRY_W				((LOADSAVE_W / 3 )-(3 * LOADSAVE_HGAP)) 
+#define LOADENTRY_H				(LOADSAVE_H -(5 * LOADSAVE_VGAP )- (LOADSAVE_BANNER_DEPTH+LOADSAVE_VGAP) ) /5
 
 #define ID_LOADSAVE				21000
 #define LOADSAVE_FORM			ID_LOADSAVE+1		// back form.
@@ -88,7 +90,7 @@
 #define LOADENTRY_START			ID_LOADSAVE+10		// each of the buttons.
 #define LOADENTRY_END			ID_LOADSAVE+10 +totalslots  // must have unique ID hmm -Q
 
-#define SAVEENTRY_EDIT			ID_LOADSAVE+50		// save edit box. must be highest value possible I guess. -Q
+#define SAVEENTRY_EDIT			ID_LOADSAVE + totalslots + totalslots		// save edit box. must be highest value possible I guess. -Q
 
 // ////////////////////////////////////////////////////////////////////////////
 void drawBlueBox				(UDWORD x,UDWORD y, UDWORD w, UDWORD h);
@@ -164,12 +166,13 @@ static BOOL _addLoadSave(BOOL bLoad, const char *sSearchPath, const char *sExten
 	W_LABINIT		sLabInit;
 	UDWORD			slotCount;
 // removed hardcoded values!  change with the defines above! -Q
-	static char	sSlots[totalslots][totalslotspace];
+	static char	sSlotCaps[totalslots][totalslotspace];
+	static char	sSlotTips[totalslots][totalslotspace];
 	char **i, **files;
 	const char* checkExtension;
 
 	mode = bLoad;
-	debug(LOG_WZ, "_addLoadSave(%d, %s, %s, %s)", bLoad, sSearchPath, sExtension, title);
+	debug(LOG_SAVE, "called (%d, %s, %s, %s)", bLoad, sSearchPath, sExtension, title);
 
 	if ((bLoadSaveMode == LOAD_INGAME) || (bLoadSaveMode == SAVE_INGAME))
 	{
@@ -213,10 +216,12 @@ static BOOL _addLoadSave(BOOL bLoad, const char *sSearchPath, const char *sExten
 	sFormInit.formID = 0;				//this adds the blue background, and the "box" behind the buttons -Q
 	sFormInit.id = LOADSAVE_FORM;
 	sFormInit.style = WFORM_PLAIN;
-	sFormInit.x = (SWORD)(LOADSAVE_X);
-	sFormInit.y = (SWORD)(LOADSAVE_Y);
+	sFormInit.x = (SWORD) LOADSAVE_X;
+	sFormInit.y = (SWORD) LOADSAVE_Y;
 	sFormInit.width = LOADSAVE_W;
-	sFormInit.height = (LOADSAVE_H*2)-46;		// hmm..the bottom of the box.... -Q
+	// we need the form to be long enough for all resolutions, so we take the total number of items * height
+	// and * the gaps, add the banner, and finally, the fudge factor ;)
+	sFormInit.height = (slotsInColumn * LOADENTRY_H + LOADSAVE_HGAP* slotsInColumn)+ LOADSAVE_BANNER_DEPTH+20;
 	sFormInit.disableChildren = true;
 	sFormInit.pDisplay = intOpenPlainForm;
 	widgAddForm(psRequestScreen, &sFormInit);
@@ -277,17 +282,23 @@ static BOOL _addLoadSave(BOOL bLoad, const char *sSearchPath, const char *sExten
 	{
 		sButInit.id		= slotCount+LOADENTRY_START;
 
-		if(slotCount<(totalslots/2))
+		if(slotCount < slotsInColumn)
 		{
-			sButInit.x	= LOADSAVE_HGAP;
+			sButInit.x	= 22 + LOADSAVE_HGAP;
 			sButInit.y	= (SWORD)((LOADSAVE_BANNER_DEPTH +(2*LOADSAVE_VGAP)) + (
-                slotCount*(LOADSAVE_VGAP+LOADENTRY_H)));
+				slotCount*(LOADSAVE_VGAP+LOADENTRY_H)));
+		}
+		else if (slotCount >= slotsInColumn && (slotCount < (slotsInColumn *2)))
+		{
+			sButInit.x	= 22 + (2*LOADSAVE_HGAP + LOADENTRY_W);
+			sButInit.y	= (SWORD)((LOADSAVE_BANNER_DEPTH +(2* LOADSAVE_VGAP)) + (
+				(slotCount % slotsInColumn)*(LOADSAVE_VGAP+LOADENTRY_H)));
 		}
 		else
 		{
-			sButInit.x	= (2*LOADSAVE_HGAP)+LOADENTRY_W;
+			sButInit.x	= 22 + (3*LOADSAVE_HGAP + (2*LOADENTRY_W));
 			sButInit.y	= (SWORD)((LOADSAVE_BANNER_DEPTH +(2* LOADSAVE_VGAP)) + (
-                (slotCount-(totalslots/2)) *(LOADSAVE_VGAP+LOADENTRY_H)));
+				(slotCount % slotsInColumn)*(LOADSAVE_VGAP+LOADENTRY_H)));
 		}
 		widgAddButton(psRequestScreen, &sButInit);
 	}
@@ -298,7 +309,7 @@ static BOOL _addLoadSave(BOOL bLoad, const char *sSearchPath, const char *sExten
 	sstrcpy(sPath, sSearchPath);  // setup locals.
 	sstrcpy(sExt, sExtension);
 
-	debug(LOG_WZ, "_addLoadSave: Searching \"%s\" for savegames", sSearchPath);
+	debug(LOG_SAVE, "Searching \"%s\" for savegames", sSearchPath);
 
 	// Check for an extension like ".ext", not "ext"
 	sasprintf((char**)&checkExtension, ".%s", sExtension);
@@ -308,6 +319,8 @@ static BOOL _addLoadSave(BOOL bLoad, const char *sSearchPath, const char *sExten
 	for (i = files; *i != NULL; ++i)
 	{
 		W_BUTTON *button;
+		char savefile[256];
+		time_t savetime;
 
 		// See if this filename contains the extension we're looking for
 		if (!strstr(*i, checkExtension))
@@ -318,12 +331,20 @@ static BOOL _addLoadSave(BOOL bLoad, const char *sSearchPath, const char *sExten
 
 		button = (W_BUTTON*)widgGetFromID(psRequestScreen, LOADENTRY_START + slotCount);
 
-		debug(LOG_WZ, "_addLoadSave: We found [%s]", *i);
-		/* Set the tip and add the button */
+		debug(LOG_SAVE, "We found [%s]", *i);
+
+		/* Figure save-time */
+		snprintf(savefile, sizeof(savefile), "%s/%s", sSearchPath, *i);
+		savetime = PHYSFS_getLastModTime(savefile);
+		sstrcpy(sSlotTips[slotCount], ctime(&savetime));
+
+		/* Set the button-text */
 		(*i)[strlen(*i) - 4] = '\0'; // remove .gam extension
-		sstrcpy(sSlots[slotCount], *i);  //store it!
-		button->pTip = sSlots[slotCount];
-		button->pText = sSlots[slotCount];
+		sstrcpy(sSlotCaps[slotCount], *i);  //store it!
+		
+		/* Add button */
+		button->pTip = sSlotTips[slotCount];
+		button->pText = sSlotCaps[slotCount];
 		slotCount++;		// goto next but...
 		if (slotCount == totalslots)
 		{
@@ -400,7 +421,7 @@ void deleteSaveGame(char* saveGameName)
 		// filename to the directory it is in.
 		snprintf(del_file, sizeof(del_file), "%s/%s", saveGameName, *i);
 
-		debug(LOG_WZ, "Deleting [%s].", del_file);
+		debug(LOG_SAVE, "Deleting [%s].", del_file);
 
 		// Delete the file
 		if (!PHYSFS_delete(del_file))
@@ -428,7 +449,7 @@ static BOOL _runLoadSave(BOOL bResetMissionWidgets)
 	UDWORD		id=0;
 	W_EDBINIT	sEdInit;
 	static char     sDelete[PATH_MAX];
-	UDWORD		i;
+	UDWORD		i, campaign;
 	W_CONTEXT		context;
 
 	id = widgRunScreen(psRequestScreen);
@@ -517,7 +538,7 @@ static BOOL _runLoadSave(BOOL bResetMissionWidgets)
 	{
 		char sTemp[MAX_STR_LENGTH];
 
-		if(!keyPressed(KEY_RETURN))						// enter was not pushed, so not a vaild entry.
+		if(!keyPressed(KEY_RETURN) && !keyPressed(KEY_KPENTER))						// enter was not pushed, so not a vaild entry.
 		{
 			widgDelete(psRequestScreen,SAVEENTRY_EDIT);	//unselect this box, and go back ..
 			widgReveal(psRequestScreen,chosenSlotId);
@@ -588,7 +609,9 @@ failure:
 
 // success on load.
 success:
-	setCampaignNumber( getCampaign(sRequestResult) );
+	campaign = getCampaign(sRequestResult);
+	setCampaignNumber(campaign);
+	debug(LOG_WZ, "Set campaign for %s to %u", sRequestResult, campaign);
 successforce:
 	closeLoadSave();
 	bRequestLoad = true;
@@ -648,7 +671,7 @@ void removeWildcards(char *pStr)
 // ////////////////////////////////////////////////////////////////////////////
 // DISPLAY FUNCTIONS
 
-static void displayLoadBanner(WIDGET *psWidget, UDWORD xOffset, UDWORD yOffset, PIELIGHT *pColours)
+static void displayLoadBanner(WIDGET *psWidget, UDWORD xOffset, UDWORD yOffset, WZ_DECL_UNUSED PIELIGHT *pColours)
 {
 	PIELIGHT col;
 	UDWORD	x = xOffset+psWidget->x;
@@ -668,7 +691,7 @@ static void displayLoadBanner(WIDGET *psWidget, UDWORD xOffset, UDWORD yOffset,
 }
 
 // ////////////////////////////////////////////////////////////////////////////
-static void displayLoadSlot(WIDGET *psWidget, UDWORD xOffset, UDWORD yOffset, PIELIGHT *pColours)
+static void displayLoadSlot(WIDGET *psWidget, UDWORD xOffset, UDWORD yOffset, WZ_DECL_UNUSED PIELIGHT *pColours)
 {
 
 	UDWORD	x = xOffset+psWidget->x;
@@ -677,9 +700,9 @@ static void displayLoadSlot(WIDGET *psWidget, UDWORD xOffset, UDWORD yOffset, PI
 
 	drawBlueBox(x,y,psWidget->width,psWidget->height);	//draw box
 
-	if(((W_BUTTON *)psWidget)->pTip )
+	if(((W_BUTTON *)psWidget)->pText )
 	{
-		sstrcpy(butString, ((W_BUTTON *)psWidget)->pTip);
+		sstrcpy(butString, ((W_BUTTON *)psWidget)->pText);
 
 		iV_SetFont(font_regular);									// font
 		iV_SetTextColour(WZCOL_TEXT_BRIGHT);
@@ -695,7 +718,7 @@ static void displayLoadSlot(WIDGET *psWidget, UDWORD xOffset, UDWORD yOffset, PI
 }
 
 // ////////////////////////////////////////////////////////////////////////////
-static void displayLoadSaveEdit(WIDGET *psWidget, UDWORD xOffset, UDWORD yOffset, PIELIGHT *pColours)
+static void displayLoadSaveEdit(WIDGET *psWidget, UDWORD xOffset, UDWORD yOffset, WZ_DECL_UNUSED PIELIGHT *pColours)
 {
 	UDWORD	x = xOffset+psWidget->x;
 	UDWORD	y = yOffset+psWidget->y;
