@@ -1,7 +1,7 @@
 /*
 	This file is part of Warzone 2100.
 	Copyright (C) 1999-2004  Eidos Interactive
-	Copyright (C) 2005-2007  Warzone Resurrection Project
+	Copyright (C) 2005-2009  Warzone Resurrection Project
 
 	Warzone 2100 is free software; you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -126,8 +126,8 @@ static	UDWORD	numMissionDroids[MAX_PLAYERS];
 static	UDWORD	numTransporterDroids[MAX_PLAYERS];
 static	UDWORD	numCommandDroids[MAX_PLAYERS];
 static	UDWORD	numConstructorDroids[MAX_PLAYERS];
-// flag to signal a quick exit from the game
-static SDWORD videoMode;
+
+static SDWORD videoMode = 0;
 
 LOOP_MISSION_STATE		loopMissionState = LMS_NORMAL;
 
@@ -136,7 +136,6 @@ SDWORD	nextMissionType = LDS_NONE;//MISSION_NONE;
 
  /* Force 3D display */
 UDWORD	mcTime;
-extern BOOL		godMode;
 
 /* The main game loop */
 GAMECODE gameLoop(void)
@@ -173,6 +172,8 @@ GAMECODE gameLoop(void)
 	HandleClosingWindows();	// Needs to be done outside the pause case.
 
 	audio_Update();
+	
+	pie_ShowMouse(true);
 
 	if (!paused)
 	{
@@ -227,7 +228,7 @@ GAMECODE gameLoop(void)
 				// Update abandoned structures
 				handleAbandonedStructures();
 			}
-			
+
 			//handles callbacks for positioning of DP's
 			process3DBuilding();
 
@@ -364,6 +365,7 @@ GAMECODE gameLoop(void)
 					}
 				}
 
+				// FIXME: These for-loops are code duplicationo
 				/*set this up AFTER droidUpdate so that if trying to building a
 				new one, we know whether one exists already*/
 				setLasSatExists(false, i);
@@ -504,15 +506,21 @@ GAMECODE gameLoop(void)
 			}
 			else
 			{
+				char msgbuffer[256]= {'\0'};
+
 				if (saveInMissionRes())
 				{
 					if (saveGame(sRequestResult, GTYPE_SAVE_START))
 					{
-						addConsoleMessage(_("GAME SAVED!"), LEFT_JUSTIFY, SYSTEM_MESSAGE);
+						sstrcpy(msgbuffer, _("GAME SAVED :"));
+						sstrcat(msgbuffer, sRequestResult);
+						addConsoleMessage( msgbuffer, LEFT_JUSTIFY, NOTIFY_MESSAGE);
 					}
 					else
 					{
 						ASSERT( false,"Mission Results: saveGame Failed" );
+						sstrcpy(msgbuffer, _("Could not save game!"));
+						addConsoleMessage( msgbuffer, LEFT_JUSTIFY, NOTIFY_MESSAGE);
 						deleteSaveGame(sRequestResult);
 					}
 				}
@@ -520,11 +528,15 @@ GAMECODE gameLoop(void)
 				{
 					if (saveGame(sRequestResult, GTYPE_SAVE_MIDMISSION))//mid mission from [esc] menu
 					{
-						addConsoleMessage(_("GAME SAVED!"), LEFT_JUSTIFY, SYSTEM_MESSAGE);
+						sstrcpy(msgbuffer, _("GAME SAVED :"));
+						sstrcat(msgbuffer, sRequestResult);
+						addConsoleMessage( msgbuffer, LEFT_JUSTIFY, NOTIFY_MESSAGE);
 					}
 					else
 					{
 						ASSERT(!"saveGame(sRequestResult, GTYPE_SAVE_MIDMISSION) failed", "Mid Mission: saveGame Failed" );
+						sstrcpy(msgbuffer, _("Could not save game!"));
+						addConsoleMessage( msgbuffer, LEFT_JUSTIFY, NOTIFY_MESSAGE);
 						deleteSaveGame(sRequestResult);
 					}
 				}
@@ -817,6 +829,7 @@ void loop_SetVideoPlaybackMode(void)
 	gameTimeStop();
 	pie_SetFogStatus(false);
 	audio_StopAll();
+	pie_ShowMouse(false);
 }
 
 
@@ -828,6 +841,7 @@ void loop_ClearVideoPlaybackMode(void)
 	gameTimeStart();
 	pie_SetFogStatus(true);
 	cdAudio_Resume();
+	pie_ShowMouse(true);
 	ASSERT( videoMode == 0, "loop_ClearVideoPlaybackMode: out of sync." );
 }
 
