@@ -1,7 +1,7 @@
 /*
 	This file is part of Warzone 2100.
 	Copyright (C) 1999-2004  Eidos Interactive
-	Copyright (C) 2005-2007  Warzone Resurrection Project
+	Copyright (C) 2005-2009  Warzone Resurrection Project
 
 	Warzone 2100 is free software; you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -40,11 +40,14 @@
 #include "lib/framework/physfs_ext.h"
 #include "lib/framework/tagfile.h"
 #include "lib/exceptionhandler/exceptionhandler.h"
+#include "lib/exceptionhandler/dumpinfo.h"
 
+#include "lib/sound/playlist.h"
 #include "lib/gamelib/gtime.h"
 #include "lib/ivis_common/piestate.h"
 #include "lib/ivis_common/rendmode.h"
 #include "lib/ivis_common/piepalette.h"
+#include "lib/ivis_common/piemode.h"
 #include "lib/ivis_opengl/screen.h"
 #include "lib/netplay/netplay.h"
 #include "lib/script/script.h"
@@ -71,6 +74,7 @@
 #include "version.h"
 #include "map.h"
 #include "parsetest.h"
+#include "keybind.h"
 
 /* Always use fallbacks on Windows */
 #if defined(WZ_OS_WIN)
@@ -83,13 +87,13 @@
 
 
 #if defined(WZ_OS_WIN)
-# define WZ_WRITEDIR "Warzone 2100 2.1"
+# define WZ_WRITEDIR "Warzone 2100 2.2"
 #elif defined(WZ_OS_MAC)
 # include <CoreServices/CoreServices.h>
 # include <unistd.h>
-# define WZ_WRITEDIR "Warzone 2100 2.1"
+# define WZ_WRITEDIR "Warzone 2100 2.2"
 #else
-# define WZ_WRITEDIR ".warzone2100-2.1"
+# define WZ_WRITEDIR ".warzone2100-2.2"
 #endif
 
 char datadir[PATH_MAX] = "\0"; // Global that src/clparse.c:ParseCommandLine can write to, so it can override the default datadir on runtime. Needs to be \0 on startup for ParseCommandLine to work!
@@ -270,15 +274,8 @@ static void getPlatformUserDir(char * const tmpstr, size_t const size)
 		strlcat(tmpstr, PHYSFS_getDirSeparator(), size);
 	else
 #elif defined(WZ_OS_MAC)
-	short vol_ref;
-	long dir_id;
-	FSSpec fsspec;
 	FSRef fsref;
-	OSErr error = FindFolder(kUserDomain, kApplicationSupportFolderType, false, &vol_ref, &dir_id);
-	if (!error)
-		error = FSMakeFSSpec(vol_ref, dir_id, (const unsigned char *) "", &fsspec);
-	if (!error)
-		error = FSpMakeFSRef(&fsspec, &fsref);
+	OSErr error = FSFindFolder(kUserDomain, kApplicationSupportFolderType, false, &fsref);
 	if (!error)
 		error = FSRefMakePath(&fsref, (UInt8 *) tmpstr, size);
 	if (!error)
@@ -596,7 +593,10 @@ static void stopGameLoop(void)
 		fogStatus = 0;
 		if (gameLoopStatus != GAMECODE_LOADGAME)
 		{
-			levReleaseAll();
+			if (!levReleaseAll())
+			{
+				debug(LOG_ERROR, "levReleaseAll failed!");
+			}
 		}
 	}
 
@@ -719,10 +719,8 @@ static void runTitleLoop(void)
 		case TITLECODE_SHOWINTRO:
 			debug(LOG_MAIN, "TITLECODE_SHOWINTRO");
 			seq_ClearSeqList();
-			seq_AddSeqToList("eidos-logo.rpl", NULL, NULL, false);
-			seq_AddSeqToList("pumpkin.rpl", NULL, NULL, false);
-			seq_AddSeqToList("titles.rpl", NULL, NULL, false);
-			seq_AddSeqToList("devastation.rpl", NULL, "devastation.txa", false);
+			seq_AddSeqToList("titles.ogg", NULL, NULL, false);
+			seq_AddSeqToList("devastation.ogg", NULL, "devastation.txa", false);
 			seq_StartNextFullScreenVideo();
 			break;
 		default:
@@ -744,7 +742,7 @@ static void handleActiveEvent(SDL_ActiveEvent * activeEvent)
 
 	if ( activeEvent->gain == 1 )
 	{
-		debug( LOG_NEVER, "WM_SETFOCUS\n");
+		debug( LOG_NEVER, "WM_SETFOCUS");
 		if (focusState != FOCUS_IN)
 		{
 			focusState = FOCUS_IN;
@@ -759,7 +757,7 @@ static void handleActiveEvent(SDL_ActiveEvent * activeEvent)
 	// Only loose focus when the config settings allow us to
 	else if (war_GetPauseOnFocusLoss())
 	{
-		debug( LOG_NEVER, "WM_KILLFOCUS\n");
+		debug( LOG_NEVER, "WM_KILLFOCUS");
 		if (focusState != FOCUS_OUT)
 		{
 			focusState = FOCUS_OUT;
@@ -819,6 +817,12 @@ static void mainLoop(void)
 					break;
 			}
 		}
+		// Screenshot key is now available globally
+		if(keyPressed(KEY_F10))
+		{
+			kf_ScreenDump();
+			inputLooseFocus();		// remove it from input stream
+		}
 
 		if (focusState == FOCUS_IN)
 		{
@@ -914,7 +918,89 @@ int main(int argc, char *argv[])
 	// Find out where to find the data
 	scanDataDirs();
 
-	if (!frameInitialise( "Warzone 2100", pie_GetVideoBufferWidth(), pie_GetVideoBufferHeight(), pie_GetVideoBufferDepth(), war_getFullscreen() ))
+	// Must be run before OpenGL driver is properly initialized due to
+	// strange conflicts - Per
+	if (selfTest)
+	{
+		memset(enabled_debug, 0, sizeof(*enabled_debug) * LOG_LAST);
+		fprintf(stdout, "Carrying out self-test:\n");
+		playListTest();
+		audioTest();
+		soundTest();
+	}
+
+	// Now we check the mods to see if they exsist or not (specified on the command line)
+	// They are all capped at 100 mods max(see clparse.c)
+	// FIX ME: I know this is a bit hackish, but better than nothing for now?
+	{
+		char *modname;
+		char modtocheck[256];
+		int i = 0;
+		int result = 0;
+
+		// check global mods
+		for(i=0; i < 100; i++)
+		{
+			modname = global_mods[i];
+			if (modname == NULL)
+			{
+				break;
+			}
+			ssprintf(modtocheck, "mods/global/%s", modname);
+			result = PHYSFS_exists(modtocheck);
+			result |= PHYSFS_isDirectory(modtocheck); 
+			if (!result)
+			{
+				debug(LOG_ERROR, "The (global) mod (%s) you have specified doesn't exist!", modname);
+			}
+			else
+			{
+				info("(global) mod (%s) is enabled", modname);
+			}
+		}
+		// check campaign mods
+		for(i=0; i < 100; i++)
+		{
+			modname = campaign_mods[i];
+			if (modname == NULL)
+			{
+				break;
+			}
+			ssprintf(modtocheck, "mods/campaign/%s", modname);
+			result = PHYSFS_exists(modtocheck);
+			result |= PHYSFS_isDirectory(modtocheck); 
+			if (!result)
+			{
+				debug(LOG_ERROR, "The mod_ca (%s) you have specified doesn't exist!", modname);
+			}
+			else
+			{
+				info("mod_ca (%s) is enabled", modname);
+			}
+		}
+		// check multiplay mods
+		for(i=0; i < 100; i++)
+		{
+			modname = multiplay_mods[i];
+			if (modname == NULL)
+			{
+				break;
+			}
+			ssprintf(modtocheck, "mods/multiplay/%s", modname);
+			result = PHYSFS_exists(modtocheck);
+			result |= PHYSFS_isDirectory(modtocheck); 
+			if (!result)
+			{
+				debug(LOG_ERROR, "The mod_mp (%s) you have specified doesn't exist!", modname);
+			}
+			else
+			{
+				info("mod_mp (%s) is enabled", modname);
+			}
+		}
+	}
+
+	if (!frameInitialise( "Warzone 2100", pie_GetVideoBufferWidth(), pie_GetVideoBufferHeight(), pie_GetVideoBufferDepth(), war_getFullscreen(), war_GetVsync()))
 	{
 		return -1;
 	}
@@ -942,8 +1028,6 @@ int main(int argc, char *argv[])
 	/* Runtime unit testing */
 	if (selfTest)
 	{
-		memset(enabled_debug, 0, sizeof(*enabled_debug) * LOG_LAST);
-		fprintf(stdout, "Carrying out self-test:\n");
 		NETtest();
 		tagTest();
 		parseTest();
@@ -953,6 +1037,13 @@ int main(int argc, char *argv[])
 		exit(0);
 	}
 
+	{
+		// Copy this info to be used by the crash handler for the dump file
+		char buf[256];
+
+		ssprintf(buf,"Using language: %s", getLanguageName());
+		addDumpInfo(buf);
+	}
 	// Do the game mode specific initialisation.
 	switch(GetGameMode())
 	{
