@@ -1,7 +1,7 @@
 /*
 	This file is part of Warzone 2100.
 	Copyright (C) 1999-2004  Eidos Interactive
-	Copyright (C) 2005-2007  Warzone Resurrection Project
+	Copyright (C) 2005-2009  Warzone Resurrection Project
 
 	Warzone 2100 is free software; you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -44,6 +44,7 @@
 #include "scripttabs.h"
 #include "scriptcb.h"
 #include "loop.h"
+#include "transporter.h"
 
 #include "lib/netplay/netplay.h"
 #include "multiplay.h"
@@ -132,7 +133,7 @@ BOOL sendGift(uint8_t type, uint8_t to)
 			break;
 		default:
 			debug( LOG_ERROR, "Unknown Gift sent" );
-			abort();
+
 			return false;
 			break;
 	}
@@ -190,7 +191,7 @@ static void recvGiftDroids(uint8_t from, uint8_t to, uint32_t droidID)
 	}
 	else
 	{
-		debug(LOG_ERROR, "recvGiftDroids: Bad droid id %d", (int)droidID);
+		debug(LOG_ERROR, "Bad droid id %u, from %u to %u", droidID, from, to);
 	}
 }
 
@@ -233,9 +234,10 @@ static void sendGiftDroids(uint8_t from, uint8_t to)
 		// Store the next droid in the list as the list may change
 		next = psD->psNext;
 
-		if (psD->droidType == DROID_TRANSPORTER)
+		if (psD->droidType == DROID_TRANSPORTER
+		 && !transporterIsEmpty(psD))
 		{
-			CONPRINTF(ConsoleString, (ConsoleString, _("Tried to give away a %s - but this is not allowed."), psD->aName));
+			CONPRINTF(ConsoleString, (ConsoleString, _("Tried to give away a non-empty %s - but this is not allowed."), psD->aName));
 			continue;
 		}
 		if (psD->selected)
@@ -319,6 +321,10 @@ void giftPower(uint8_t from, uint8_t to, BOOL send)
 
 	addPower(to, gifval);
 
+	if (from == ANYPLAYER && to == selectedPlayer)
+	{
+		CONPRINTF(ConsoleString,(ConsoleString,_("You found %u power in an oil drum"),gifval));
+	}
 	if (send)
 	{
 		uint8_t giftType = POWER_GIFT;
@@ -332,7 +338,7 @@ void giftPower(uint8_t from, uint8_t to, BOOL send)
 	}
 	else if (to == selectedPlayer)
 	{
-		CONPRINTF(ConsoleString,(ConsoleString,_("%s Gives You Power"),getPlayerName(from)));
+		CONPRINTF(ConsoleString,(ConsoleString,_("%s Gives You %u Power"),getPlayerName(from),gifval));
 	}
 }
 
@@ -616,7 +622,6 @@ static void addLoserGifts(void)
 				if (!pickATileGen(&x, &y, LOOK_FOR_EMPTY_TILE, zonedPAT))
 				{
 					ASSERT(false, "addlosergifts: Unable to find a free location");
-					break;
 				}
 
 				NETlogEntry("gift", 0, 0);
@@ -644,13 +649,14 @@ static void addLoserGifts(void)
  *  \param subType the type of feature to build
  *  \param x,y the coordinates to place the feature at
  */
-void sendMultiPlayerFeature(FEATURE_TYPE subType, uint32_t x, uint32_t y)
+void sendMultiPlayerFeature(FEATURE_TYPE subType, uint32_t x, uint32_t y, uint32_t id)
 {
 	NETbeginEncode(NET_FEATURES, NET_ALL_PLAYERS);
 	{
 		NETenum(&subType);
 		NETuint32_t(&x);
 		NETuint32_t(&y);
+		NETuint32_t(&id);
 	}
 	NETend();
 }
@@ -658,7 +664,7 @@ void sendMultiPlayerFeature(FEATURE_TYPE subType, uint32_t x, uint32_t y)
 void recvMultiPlayerFeature()
 {
 	FEATURE_TYPE subType;
-	uint32_t     x, y;
+	uint32_t     x, y, id;
 	unsigned int i;
 
 	NETbeginDecode(NET_FEATURES);
@@ -666,6 +672,7 @@ void recvMultiPlayerFeature()
 		NETenum(&subType);
 		NETuint32_t(&x);
 		NETuint32_t(&y);
+		NETuint32_t(&id);
 	}
 	NETend();
 
@@ -676,8 +683,8 @@ void recvMultiPlayerFeature()
 		if (asFeatureStats[i].subType == subType)
 		{
 			// Create a feature of the specified type at the given location
-			buildFeature(&asFeatureStats[i], x, y, false);
-
+			FEATURE *result = buildFeature(&asFeatureStats[i], x, y, false);
+			result->id = id;
 			break;
 		}
 	}
@@ -709,7 +716,6 @@ void  addMultiPlayerRandomArtifacts(uint8_t quantity, FEATURE_TYPE type)
 			if (!pickATileGen(&x, &y, LOOK_FOR_EMPTY_TILE, zonedPAT))
 			{
 				ASSERT(false, "addMultiPlayerRandomArtifacts: Unable to find a free location");
-				break;
 			}
 
 			pF = buildFeature(asFeatureStats + i, world_coord(x), world_coord(y), false);
@@ -787,23 +793,32 @@ void recvMultiPlayerRandomArtifacts()
 void giftArtifact(UDWORD owner, UDWORD x, UDWORD y)
 {
 	PLAYER_RESEARCH *pO,*pR;
-	UDWORD	topic=0;
+	UDWORD	topic = 0;
 	pR   = asPlayerResList[selectedPlayer];
 
 	if (owner < MAX_PLAYERS)
 	{
 		pO	 = asPlayerResList[owner];
 
-		for (topic = numResearch; topic > 0; topic--)
+		for (topic = numResearch - 1; topic > 0; topic--)
 		{
 			if (IsResearchCompleted(&pO[topic])
 			 && !IsResearchPossible(&pR[topic]))
 			{
-				MakeResearchPossible(&pR[topic]);
-				CONPRINTF(ConsoleString,(ConsoleString,_("You Discover Blueprints For %s"),
-					getName(asResearch[topic].pName)));
-				
-				break;
+				// Make sure the topic can be researched
+				if (asResearch[topic].researchPower 
+				 && asResearch[topic].researchPoints)
+				{
+					MakeResearchPossible(&pR[topic]);
+					CONPRINTF(ConsoleString,(ConsoleString,_("You Discover Blueprints For %s"),
+						getName(asResearch[topic].pName)));
+					break;
+				}
+				// Invalid topic
+				else
+				{
+					debug(LOG_WARNING, "%s is a invalid research topic?", getName(asResearch[topic].pName));
+				}
 			}
 		}
 	}
