@@ -1,7 +1,7 @@
 /*
 	This file is part of Warzone 2100.
 	Copyright (C) 1999-2004  Eidos Interactive
-	Copyright (C) 2005-2007  Warzone Resurrection Project
+	Copyright (C) 2005-2009  Warzone Resurrection Project
 
 	Warzone 2100 is free software; you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -69,8 +69,8 @@ BOOL	MultiMenuUp			= false;
 BOOL	ClosingMultiMenu	= false;
 BOOL	DebugMenuUp		= false;
 static UDWORD	context = 0;
-static UDWORD	current_tech = 0;
-static UDWORD	current_numplayers = 0;
+UDWORD	current_tech = 1;
+UDWORD	current_numplayers = 4;
 
 #define DEBUGMENU_FORM_W		200
 #define DEBUGMENU_FORM_H		300
@@ -147,14 +147,38 @@ char		debugMenuEntry[DEBUGMENU_MAX_ENTRIES][MAX_STR_LENGTH];
 // ////////////////////////////////////////////////////////////////////////////
 // Map / force / name load save stuff.
 
+/* Sets the player's text color depending on if alliance formed, or if dead.
+ * \param mode  the specified alliance
+ * \param player the specified player
+ */
+static void SetPlayerTextColor( int mode, UDWORD player )
+{
+	// the colors were chosen to match the FRIEND/FOE radar map colors.
+	if (mode == ALLIANCE_FORMED)
+	{
+		iV_SetTextColour(WZCOL_YELLOW);			// Human alliance text color
+	}
+	else if (isHumanPlayer(player))				// Human player, no alliance
+	{
+		iV_SetTextColour(WZCOL_TEXT_BRIGHT);	// Normal text color
+	}
+	else
+	{
+		iV_SetTextColour(WZCOL_RED);			// Enemy color
+	}
+	// override color if they are dead...
+	if (!apsDroidLists[player] && !apsStructLists[player])
+	{
+		iV_SetTextColour(WZCOL_GREY);			// dead text color
+	}
+}
 // ////////////////////////////////////////////////////////////////////////////
 // enumerates maps in the gamedesc file.
 // returns only maps that are valid the right 'type'
-
-static BOOL enumerateMultiMaps(char *found, UDWORD *players,BOOL first, UBYTE camToUse, UBYTE numPlayers)
+static char* enumerateMultiMaps(UDWORD* players, bool first, unsigned int camToUse, unsigned int numPlayers)
 {
 	static LEVEL_DATASET *lev;
-	UBYTE cam;
+	unsigned int cam;
 
 	if(first)
 	{
@@ -181,10 +205,18 @@ static BOOL enumerateMultiMaps(char *found, UDWORD *players,BOOL first, UBYTE ca
 				&& (numPlayers == 0 || numPlayers == lev->players)
 				&& cam == camToUse )
 			{
-				strcpy(found,lev->pName);
+				char* const found = strdup(lev->pName);
+				if (found == NULL)
+				{
+					debug(LOG_ERROR, "Out of memory");
+					// No way to indicate out-of-memory failure by return value
+					abort();
+					return NULL;
+				}
+
 				*players = lev->players;
 				lev = lev->psNext;
-				return true;
+				return found;
 			}
 		}
 		else	//  campaign
@@ -208,15 +240,24 @@ static BOOL enumerateMultiMaps(char *found, UDWORD *players,BOOL first, UBYTE ca
 			    && (numPlayers == 0 || numPlayers == lev->players)
 			    && cam == camToUse )
 			{
-				strcpy(found,lev->pName);
+				char* const found = strdup(lev->pName);
+				if (found == NULL)
+				{
+					debug(LOG_ERROR, "Out of memory");
+					// No way to indicate out-of-memory failure by return value
+					abort();
+					return NULL;
+				}
+
 				*players = lev->players;
 				lev = lev->psNext;
-				return true;
+				return found;
 			}
 		}
 		lev = lev->psNext;
 	}
-	return false;
+
+	return NULL;
 }
 
 // ////////////////////////////////////////////////////////////////////////////
@@ -352,11 +393,14 @@ void addMultiRequest(const char* searchDir, const char* fileExtension, UDWORD mo
 
 	if(mode == MULTIOP_MAP)									// if its a map, also look in the predone stuff.
 	{
-		if(enumerateMultiMaps(tips[0],&players, true,mapCam,numPlayers))
+		char* map;
+		if ((map = enumerateMultiMaps(&players, true, mapCam, numPlayers)))
 		{
+			free(map);
 			numButtons++;
-			while(enumerateMultiMaps(tips[0],&players, false,mapCam,numPlayers))
+			while ((map = enumerateMultiMaps(&players, false, mapCam, numPlayers)))
 			{
+				free(map);
 				numButtons++;
 			}
 		}
@@ -512,14 +556,16 @@ void addMultiRequest(const char* searchDir, const char* fileExtension, UDWORD mo
 
 	if(mode == MULTIOP_MAP)
 	{
-		char sTemp[64];
-		if(enumerateMultiMaps( sTemp,&players,true,mapCam,numPlayers))
+		char* mapName;
+		if ((mapName = enumerateMultiMaps(&players, true, mapCam, numPlayers)))
 		{
-			do{
+			do
+			{
 				unsigned int tip_index = check_tip_index(sButInit.id-M_REQUEST_BUT);
 
 				// add number of players to string.
-				sprintf(tips[tip_index],"%s",sTemp );
+				sstrcpy(tips[tip_index], mapName);
+				free(mapName);
 
 				sButInit.pTip = tips[tip_index];
 				sButInit.pText = tips[tip_index];
@@ -539,7 +585,7 @@ void addMultiRequest(const char* searchDir, const char* fileExtension, UDWORD mo
 					sButInit.y = 4;
 					sButInit.majorID += 1;
 				}
-			}while(enumerateMultiMaps(sTemp,&players,false,mapCam,numPlayers));
+			} while ((mapName = enumerateMultiMaps(&players, false, mapCam, numPlayers)));
 		}
 	}
 	multiRequestUp = true;
@@ -678,10 +724,6 @@ static void displayExtraGubbins(UDWORD height)
 {
 	char	str[128];
 
-	// draw timer
-	getAsciiTime(str,gameTime);
-	iV_DrawText(str, MULTIMENU_FORM_X+MULTIMENU_C2 ,MULTIMENU_FORM_Y+MULTIMENU_FONT_OSET) ;
-
 	//draw grid
 	iV_Line(MULTIMENU_FORM_X+MULTIMENU_C0 -6 , MULTIMENU_FORM_Y,
 			MULTIMENU_FORM_X+MULTIMENU_C0 -6 , MULTIMENU_FORM_Y+height, WZCOL_BLACK);
@@ -701,23 +743,36 @@ static void displayExtraGubbins(UDWORD height)
 	iV_Line(MULTIMENU_FORM_X				, MULTIMENU_FORM_Y+MULTIMENU_PLAYER_H,
 			MULTIMENU_FORM_X+MULTIMENU_FORM_W, MULTIMENU_FORM_Y+MULTIMENU_PLAYER_H, WZCOL_BLACK);
 
-	//draw titles.
-	iV_SetFont(font_regular);											// font
-	iV_SetTextColour(WZCOL_TEXT_BRIGHT);
+	iV_SetFont(font_regular);						// font
+	iV_SetTextColour(WZCOL_TEXT_BRIGHT);			// main wz text color
 
+	// draw timer
+	getAsciiTime(str, gameTime);
+	iV_DrawText(str, MULTIMENU_FORM_X+MULTIMENU_C2, MULTIMENU_FORM_Y+MULTIMENU_FONT_OSET) ;
+
+	// draw titles.
 	iV_DrawText(_("Alliances"), MULTIMENU_FORM_X+MULTIMENU_C0, MULTIMENU_FORM_Y+MULTIMENU_FONT_OSET);
 	iV_DrawText(_("Score"), MULTIMENU_FORM_X+MULTIMENU_C8, MULTIMENU_FORM_Y+MULTIMENU_FONT_OSET);
 	iV_DrawText(_("Kills"), MULTIMENU_FORM_X+MULTIMENU_C9, MULTIMENU_FORM_Y+MULTIMENU_FONT_OSET);
 
 	if(getDebugMappingStatus())
-	{
+	{	// shows # units for *all* players in debug mode ONLY!
 		iV_DrawText("Units", MULTIMENU_FORM_X+MULTIMENU_C10, MULTIMENU_FORM_Y+MULTIMENU_FONT_OSET);
 		iV_DrawText("Power", MULTIMENU_FORM_X+MULTIMENU_C11, MULTIMENU_FORM_Y+MULTIMENU_FONT_OSET);
 	}
 	else
-	{
-		iV_DrawText(_("Ping"), MULTIMENU_FORM_X+MULTIMENU_C10, MULTIMENU_FORM_Y+MULTIMENU_FONT_OSET);
-		iV_DrawText(_("Played"), MULTIMENU_FORM_X+MULTIMENU_C11, MULTIMENU_FORM_Y+MULTIMENU_FONT_OSET);
+	{	// shows # units for *yourself* (+ team member?) only.
+		iV_DrawText(_("Units"), MULTIMENU_FORM_X+MULTIMENU_C10, MULTIMENU_FORM_Y+MULTIMENU_FONT_OSET);
+
+		// ping is useless for non MP games, so display something useful depending on mode.
+		if (runningMultiplayer())
+		{
+			iV_DrawText(_("Ping"), MULTIMENU_FORM_X+MULTIMENU_C11, MULTIMENU_FORM_Y+MULTIMENU_FONT_OSET);
+		}
+		else
+		{
+			iV_DrawText(_("Structs"), MULTIMENU_FORM_X+MULTIMENU_C11, MULTIMENU_FORM_Y+MULTIMENU_FONT_OSET);
+		}
 	}
 
 #ifdef DEBUG
@@ -755,6 +810,15 @@ static void displayMultiPlayer(WIDGET *psWidget, UDWORD xOffset, UDWORD yOffset,
 	if(isHumanPlayer(player) || (game.type == SKIRMISH && player<game.maxPlayers) )
 	{
 		ssprintf(str, "%d:%s", player, getPlayerName(player));
+		if (isHumanPlayer(player))
+		{
+			SetPlayerTextColor(alliances[selectedPlayer][player], player);
+		}
+		else
+		{
+			SetPlayerTextColor(alliances[selectedPlayer][player], player);
+		}
+
 		while(iV_GetTextWidth(str) >= (MULTIMENU_C0-MULTIMENU_C2-10) )
 		{
 			str[strlen(str)-1]='\0';
@@ -799,6 +863,8 @@ static void displayMultiPlayer(WIDGET *psWidget, UDWORD xOffset, UDWORD yOffset,
 	}
 	if(isHumanPlayer(player))
 	{
+		SetPlayerTextColor(alliances[selectedPlayer][player], player);
+
 		//c8:score,
 		sprintf(str,"%d",getMultiStats(player,true).recentScore);
 		iV_DrawText(str, x+MULTIMENU_C8, y+MULTIMENU_FONT_OSET);
@@ -809,27 +875,47 @@ static void displayMultiPlayer(WIDGET *psWidget, UDWORD xOffset, UDWORD yOffset,
 
 		if(!getDebugMappingStatus())
 		{
-			//c10:ping
-			if(player != selectedPlayer)
+			//only show player's units, and nobody elses.
+			//c10:units
+			if (myResponsibility(player))
 			{
-				if(ingame.PingTimes[player] >2000)
-				{
-					sprintf(str,"***");
-				}
-				else
-				{
-					sprintf(str,"%d",ingame.PingTimes[player]);
-				}
+				SetPlayerTextColor(alliances[selectedPlayer][player], player);
+				sprintf(str, "%d", getNumDroids(player));
 				iV_DrawText(str, x+MULTIMENU_C10, y+MULTIMENU_FONT_OSET);
 			}
 
-			//c11:played
-			sprintf(str,"%d",getMultiStats(player,true).played);
-			iV_DrawText(str, x+MULTIMENU_C11, y+MULTIMENU_FONT_OSET);
+			if (runningMultiplayer())
+			{
+				//c11:ping
+				if (player != selectedPlayer)
+				{
+					if (ingame.PingTimes[player] >= 2000)
+					{
+						sprintf(str,"???");
+					}
+					else
+					{
+						sprintf(str, "%d", ingame.PingTimes[player]);
+					}
+					iV_DrawText(str, x+MULTIMENU_C11, y+MULTIMENU_FONT_OSET);
+				}
+			}
+			else
+			{
+				int num;
+				STRUCTURE *temp;
+				// NOTE, This tallys up *all* the structures you have. Test out via 'start with no base'.
+				for (num = 0, temp = apsStructLists[player]; temp != NULL;num++,temp = temp->psNext);
+				//c11: Structures
+				sprintf(str, "%d", num);
+				iV_DrawText(str, x+MULTIMENU_C11, y+MULTIMENU_FONT_OSET);
+			}
 		}
 	}
 	else
 	{
+		SetPlayerTextColor(alliances[selectedPlayer][player], player);
+
 		// estimate of score.
 		sprintf(str,"%d",ingame.skScores[player][0]);
 		iV_DrawText(str, x+MULTIMENU_C8, y+MULTIMENU_FONT_OSET);
