@@ -1,7 +1,7 @@
 /*
 	This file is part of Warzone 2100.
 	Copyright (C) 1999-2004  Eidos Interactive
-	Copyright (C) 2005-2007  Warzone Resurrection Project
+	Copyright (C) 2005-2009  Warzone Resurrection Project
 
 	Warzone 2100 is free software; you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -51,6 +51,7 @@
 
 #include "init.h"
 #include "warcam.h"	// these 4 for fireworks
+#include "mission.h"
 #include "effects.h"
 #include "lib/gamelib/gtime.h"
 #include "keybind.h"
@@ -88,9 +89,8 @@ BOOL						bPlayerReadyGUI[MAX_PLAYERS] = {false};
 /* multiplayer message stack stuff */
 /////////////////////////////////////
 #define MAX_MSG_STACK	100				// must be *at least* 64
-#define MAX_STR			MAX_STR_LENGTH
 
-static char msgStr[MAX_MSG_STACK][MAX_STR];
+static char msgStr[MAX_MSG_STACK][MAX_STR_LENGTH];
 static SDWORD msgPlFrom[MAX_MSG_STACK];
 static SDWORD msgPlTo[MAX_MSG_STACK];
 static SDWORD callbackType[MAX_MSG_STACK];
@@ -124,7 +124,8 @@ BOOL turnOffMultiMsg(BOOL bDoit)
 	{
 		if(bTemp == true)
 		{
-			debug(LOG_NET, "turnOffMultiMsg: multiple calls to turn off");
+			// Surely, this should be a Assert?
+			debug(LOG_ERROR, "multiple calls to turn off");
 		}
 		if(bMultiPlayer)
 		{
@@ -176,7 +177,7 @@ BOOL multiplayerWinSequence(BOOL firstCall)
 	}
 
 	// rotate world
-	if(!getWarCamStatus())
+	if(MissionResUp && !getWarCamStatus())
 	{
 		rotAmount = timeAdjustedIncrement(MAP_SPIN_RATE / 12, true);
 		player.r.y += rotAmount;
@@ -704,31 +705,6 @@ BOOL recvMessage(void)
 		case NET_RESEARCH:					// some research has been done.
 			recvResearch();
 			break;
-		case NET_LEAVING:					// player leaving nicely
-		{
-			uint32_t player_id;
-			BOOL host;
-
-			resetReadyStatus(false);
-
-			NETbeginDecode(NET_LEAVING);
-				NETuint32_t(&player_id);
-				NETbool(&host);                 // Added to check for host quit here -- Buggy
-			NETend();
-			if (player_id >= MAX_PLAYERS)
-			{
-				debug(LOG_ERROR, "Bad NET_LEAVING received, player ID is %d", (int)player_id);
-				break;
-			}
-			MultiPlayerLeave(player_id);
-			if (host)                               // host has quit, need to quit too.
-			{
-				//stopJoining();		//NOT defined here, checking if we need it or not.
-				debug(LOG_NET, "***Need to call stopJoining()");
-				addConsoleMessage(_("The host has left the game!"), LEFT_JUSTIFY, SYSTEM_MESSAGE);
-			}
-			break;
-		}
 		case NET_OPTIONS:
 			recvOptions();
 			break;
@@ -777,14 +753,18 @@ BOOL recvMessage(void)
 			break;
 		case NET_KICK:
 		{
+			// FIX ME: in game kick ?  Is this even possible with current code?
 			uint32_t player_id;
+			char reason[MAX_KICK_REASON];
 
 			NETbeginDecode(NET_KICK);
 				NETuint32_t(&player_id);
+				NETstring( reason, MAX_KICK_REASON);
 			NETend();
 
 			if (NetPlay.dpidPlayer == player_id)  // we've been told to leave.
 			{
+				debug(LOG_ERROR, "You were kicked because, %s", reason);
 				setPlayerHasLost(true);
 			}
 			break;
@@ -1055,6 +1035,11 @@ BOOL sendTextMessage(const char *pStr, BOOL all)
 
 	if (!ingame.localOptionsReceived)
 	{
+		if(!bMultiPlayer)
+		{
+			// apparently we are not in a mp game, so dump the message to the console.
+			addConsoleMessage(pStr,LEFT_JUSTIFY, SYSTEM_MESSAGE);
+		}
 		return true;
 	}
 
@@ -1126,12 +1111,19 @@ BOOL sendTextMessage(const char *pStr, BOOL all)
 	}
 
 	//This is for local display
-	for(i = 0; NetPlay.players[i].dpid != NetPlay.dpidPlayer; i++);	//findplayer
-	sstrcat(display, NetPlay.players[i].name);		// name
-	sstrcat(display, ": ");						// seperator
-	sstrcat(display, pStr);						// add message
+	for (i = 0; i < ARRAY_SIZE(NetPlay.players); ++i)
+	{
+		if (NetPlay.players[i].dpid == NetPlay.dpidPlayer)
+		{
+			sstrcat(display, NetPlay.players[i].name);		// name
+			sstrcat(display, ": ");						// seperator
+			sstrcat(display, pStr);						// add message
+
+			addConsoleMessage(display, DEFAULT_JUSTIFY, selectedPlayer);	// display
+			break;
+		}
+	}
 
-	addConsoleMessage(display, DEFAULT_JUSTIFY, selectedPlayer);	// display
 
 	return true;
 }
@@ -1260,7 +1252,15 @@ BOOL recvTextMessage()
 	NETend();
 
 
-	for (i = 0; NetPlay.players[i].dpid != dpid; i++);		//findplayer
+	for (i = 0; i < MAX_PLAYERS; i++)		//findplayer
+	{
+		if (NetPlay.players[i].dpid == dpid)
+		{
+			break;
+		}
+	}
+
+	ASSERT_OR_RETURN(false, i != MAX_PLAYERS, "Failed to find player's dpid %u", dpid);
 
 	//console callback - find real number of the player
 	for (j = 0; i < MAX_PLAYERS; j++)
@@ -1272,7 +1272,7 @@ BOOL recvTextMessage()
 		}
 	}
 
-	ASSERT(player != MAX_PLAYERS, "recvTextMessage: failed to find owner of dpid %d", dpid);
+	ASSERT_OR_RETURN(false, player != MAX_PLAYERS, "Failed to find 'real' owner of dpid %u", dpid);
 
 	sstrcpy(msg, NetPlay.players[i].name);
 	// Seperator
@@ -1353,7 +1353,7 @@ BOOL sendTemplate(DROID_TEMPLATE *pTempl)
 		NETstring(pTempl->aName, sizeof(pTempl->aName));
 		NETuint8_t(&pTempl->NameVersion);
 
-		for (i = 0; i < DROID_MAXCOMP; i++)
+		for (i = 0; i < ARRAY_SIZE(pTempl->asParts); ++i)
 		{
 			// signed, but sent as a bunch of bits...
 			NETint32_t(&pTempl->asParts[i]);
@@ -1391,7 +1391,7 @@ BOOL recvTemplate()
 		NETstring(pT->aName, sizeof(pT->aName));
 		NETuint8_t(&pT->NameVersion);
 
-		for (i = 0; i < DROID_MAXCOMP; i++)
+		for (i = 0; i < ARRAY_SIZE(pT->asParts); ++i)
 		{
 			// signed, but sent as a bunch of bits...
 			NETint32_t(&pT->asParts[i]);
@@ -1910,7 +1910,7 @@ const char* getPlayerColourName(unsigned int player)
 		N_("Cyan")
 	};
 
-	ASSERT(player < ARRAY_SIZE(playerColors), "player number (%d) exceeds maximum (%zu)", player, ARRAY_SIZE(playerColors));
+	ASSERT(player < ARRAY_SIZE(playerColors), "player number (%d) exceeds maximum (%lu)", player, (unsigned long) ARRAY_SIZE(playerColors));
 
 	if (player >= ARRAY_SIZE(playerColors))
 	{
