@@ -1,7 +1,7 @@
 /*
 	This file is part of Warzone 2100.
 	Copyright (C) 1999-2004  Eidos Interactive
-	Copyright (C) 2005-2007  Warzone Resurrection Project
+	Copyright (C) 2005-2009  Warzone Resurrection Project
 
 	Warzone 2100 is free software; you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -109,6 +109,7 @@ extern UBYTE				bDisplayMultiJoiningStatus;	// draw load progress?
 
 #define DIFF_SLIDER_STOPS		20			//max number of stops for the multiplayer difficulty slider
 
+#define MAX_KICK_REASON			80			// max array size for the reason your kicking someone
 // functions
 
 extern WZ_DECL_WARN_UNUSED_RESULT BASE_OBJECT		*IdToPointer(UDWORD id,UDWORD player);
@@ -191,7 +192,8 @@ extern BOOL addTemplate			(UDWORD	player,DROID_TEMPLATE *psNew);
 extern BOOL sendCheck			(void);							//send/recv  check info
 extern BOOL sendScoreCheck		(void);							//score check only(frontend)
 extern BOOL sendPing			(void);							// allow game to request pings.
-extern BOOL ForceDroidSync( DROID* droidToSend );				// force a sync on a droid to happen
+
+extern BOOL ForceDroidSync(const DROID* droidToSend);
 // multijoin
 extern void modifyResources		(POWER_GEN_FUNCTION* psFunction);
 
