@@ -1,7 +1,7 @@
 /*
 	This file is part of Warzone 2100.
 	Copyright (C) 1999-2004  Eidos Interactive
-	Copyright (C) 2005-2007  Warzone Resurrection Project
+	Copyright (C) 2005-2009  Warzone Resurrection Project
 
 	Warzone 2100 is free software; you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -399,7 +399,7 @@ BOOL scrOrderGroup(void)
 		return false;
 	}
 
-	debug( LOG_NEVER, "scrOrderGroup: group %p (%d) order %d\n", psGroup, grpNumMembers(psGroup), order);
+	debug(LOG_NEVER, "group %p (%u) order %d", psGroup, grpNumMembers(psGroup), order);
 	orderGroup(psGroup, order);
 
 	return true;
@@ -433,12 +433,11 @@ BOOL scrOrderGroupLoc(void)
 	 || y < 0
 	 || y > world_coord(mapHeight))
 	{
-		ASSERT( false,
-			"scrOrderGroupLoc: Invalid location" );
+		ASSERT(false, "Invalid map location (%d, %d), max is (%d, %d)", x, y, world_coord(mapWidth), world_coord(mapHeight));
 		return false;
 	}
 
-	debug( LOG_NEVER, "scrOrderGroupLoc: group %p (%d) order %d (%d,%d)\n",
+	debug(LOG_NEVER, "group %p (%u) order %d (%d,%d)",
 		psGroup, grpNumMembers(psGroup), order, x,y);
 	orderGroupLoc(psGroup, order, (UDWORD)x,(UDWORD)y);
 
@@ -477,7 +476,7 @@ BOOL scrOrderGroupObj(void)
 		return false;
 	}
 
-	debug( LOG_NEVER, "scrOrderGroupObj: group %p (%d) order %d,  obj type %d player %d id %d\n",
+	debug(LOG_NEVER, "group %p (%u) order %d,  obj type %d player %d id %d",
 		psGroup, grpNumMembers(psGroup), order, psObj->type, psObj->player, psObj->id);
 
 	orderGroupObj(psGroup, order, psObj);
@@ -508,7 +507,8 @@ BOOL scrOrderDroid(void)
 		order != DORDER_DESTRUCT &&
 		order != DORDER_RTR &&
 		order != DORDER_RTB &&
-		order != DORDER_RUN)
+		order != DORDER_RUN &&
+		order != DORDER_NONE )
 	{
 		ASSERT( false,
 			"scrOrderUnit: Invalid order" );
@@ -651,7 +651,17 @@ BOOL scrOrderDroidStatsLoc(void)
 
 	// Don't allow scripts to order structure builds if players structure
 	// limit has been reached.
-	if(IsPlayerStructureLimitReached(psDroid->player) == false) {
+	if (!IsPlayerStructureLimitReached(psDroid->player))
+	{
+		// HACK: FIXME: Looks like a script error in the player*.slo files
+		// buildOnExactLocation() which references previously destroyed buildings from
+		// _stat = rebuildStructStat[_count] causes this
+		if (strcmp(psStats->pName, "A0ADemolishStructure") == 0)
+		{
+			// I don't feel like spamming a ASSERT here, we *know* it is a issue. 
+			return true;
+		}
+
 		orderDroidStatsLoc(psDroid, order, psStats, (UDWORD)x,(UDWORD)y);
 	}
 
@@ -1098,27 +1108,29 @@ static UDWORD scrDroidTargetMask(DROID *psDroid)
 	psPStats = asPropulsionStats + psDroid->asBits[COMP_PROPULSION].nStat;
 	switch(psPStats->propulsionType)
 	{
-	case WHEELED:
+	case PROPULSION_TYPE_WHEELED:
 		mask |= SCR_DT_WHEEL;
 		break;
-	case TRACKED:
+	case PROPULSION_TYPE_TRACKED:
 		mask |= SCR_DT_TRACK;
 		break;
-	case LEGGED:
-	case JUMP:
+	case PROPULSION_TYPE_LEGGED:
+	case PROPULSION_TYPE_JUMP:
 		mask |= SCR_DT_LEGS;
 		break;
-	case HOVER:
+	case PROPULSION_TYPE_HOVER:
 		mask |= SCR_DT_HOVER;
 		break;
-	case LIFT:
+	case PROPULSION_TYPE_LIFT:
 		mask |= SCR_DT_VTOL;
 		break;
-	case HALF_TRACKED:
+	case PROPULSION_TYPE_HALF_TRACKED:
 		mask |= SCR_DT_HTRACK;
 		break;
-	case PROPELLOR:
-	case SKI:
+	case PROPULSION_TYPE_PROPELLOR:
+		mask |= SCR_DT_PROPELLOR;
+		break;
+	case PROPULSION_TYPE_SKI:
 	default:
 		ASSERT( false,
 			"scrUnitTargetMask: unknown or invalid target unit propulsion type" );
@@ -1705,7 +1717,7 @@ BOOL scrSkVtolEnableCheck(void)
 		// vtol propulsion
 		for(i=0;(i<numPropulsionStats);i++)
 		{
-			if((asPropulsionStats[i].propulsionType == LIFT)
+			if((asPropulsionStats[i].propulsionType == PROPULSION_TYPE_LIFT)
 			 && apCompLists[player][COMP_PROPULSION][i] == AVAILABLE)
 			{
 				scrFunctionResult.v.bval = true;
@@ -1917,7 +1929,7 @@ static BOOL defenseLocation(BOOL variantB)
 			gX = (psGate->x1 + psGate->x2)/2;
 			gY = (psGate->y1 + psGate->y2)/2;
 			/* Estimate the distance to it */
-			dist = dirtySqrt(x,y,gX,gY);
+			dist = dirtyHypot(x - gX, y - gY);
 			/* Is it best we've found? */
 			if(dist<nearestSoFar && dist<30)
 			{
