@@ -1,7 +1,7 @@
 /*
 	This file is part of Warzone 2100.
 	Copyright (C) 1999-2004  Eidos Interactive
-	Copyright (C) 2005-2007  Warzone Resurrection Project
+	Copyright (C) 2005-2009  Warzone Resurrection Project
 
 	Warzone 2100 is free software; you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -30,8 +30,12 @@
 // MAX_BLOCKING_TILES on some location
 #define MAX_BLOCKING_TILES		1
 
+/// Forward declarations to allow pointers to these types
+struct BASE_OBJECT;
+struct DROID;
+
 // not used in scripts, but used in code.
-extern  BOOL objectInRange(BASE_OBJECT *psList, SDWORD x, SDWORD y, SDWORD range);
+extern  BOOL objectInRange(struct BASE_OBJECT *psList, SDWORD x, SDWORD y, SDWORD range);
 
 // Check for any player object being within a certain range of a position
 extern BOOL scrObjectInRange(void);
@@ -602,12 +606,12 @@ extern BOOL scrRecallPlayerBaseLoc(void);
 extern BOOL scrRecallPlayerVisibility(void);
 extern BOOL scrCanRememberPlayerBaseLoc(void);
 extern BOOL scrStructInRangeVis(void);
+extern BOOL scrDroidInRangeVis(void);
 extern BOOL scrRecallBaseDefendLoc(void);
 extern BOOL scrRecallOilDefendLoc(void);
 extern BOOL scrGetBaseDefendLocCount(void);
 extern BOOL scrGetOilDefendLocCount(void);
 
-extern BOOL scrStructInRangeVis(void);
 extern BOOL scrGiftRadar(void);
 extern BOOL scrNumAllies(void);
 extern BOOL scrNumAAinRange(void);
@@ -643,6 +647,20 @@ extern BOOL scrDebugModeEnabled(void);
 extern BOOL scrCalcDroidPower(void);
 extern BOOL scrGetDroidLevel(void);
 extern BOOL scrMoveDroidStopped(void);
+extern BOOL scrUpdateVisibleTiles(void);
+extern BOOL scrCheckVisibleTile(void);
+extern BOOL scrAssembleWeaponTemplate(void);
+extern BOOL scrWeaponShortHitUpgrade(void);
+extern BOOL scrWeaponLongHitUpgrade(void);
+extern BOOL scrWeaponDamageUpgrade(void);
+extern BOOL scrWeaponFirePauseUpgrade(void);
+extern BOOL scrIsComponentAvailable(void);
+extern BOOL scrGetBodySize(void);
+extern BOOL scrGettext(void);
+extern BOOL scrGettext_noop(void);
+extern BOOL scrPgettext(void);
+extern BOOL scrPgettext_expr(void);
+extern BOOL scrPgettext_noop(void);
 
 
 extern BOOL beingResearchedByAlly(SDWORD resIndex, SDWORD player);
@@ -653,11 +671,11 @@ extern UDWORD numPlayerWeapStructsInRange(SDWORD player, SDWORD lookingPlayer, S
 extern UDWORD playerWeapDroidsCostInRange(SDWORD player, SDWORD lookingPlayer, SDWORD range, SDWORD rangeX, SDWORD rangeY, BOOL bVTOLs);
 extern UDWORD playerWeapStructsCostInRange(SDWORD player, SDWORD lookingPlayer, SDWORD range, SDWORD rangeX, SDWORD rangeY, BOOL bFinished);
 extern UDWORD numEnemyObjInRange(SDWORD player, SDWORD range, SDWORD rangeX, SDWORD rangeY, BOOL bVTOLs, BOOL bFinished);
-extern BOOL addHelpBlip(SDWORD x, SDWORD y, SDWORD forPlayer, SDWORD sender, char * textMsg);
+extern BOOL addBeaconBlip(SDWORD x, SDWORD y, SDWORD forPlayer, SDWORD sender, char * textMsg);
 extern BOOL sendBeaconToPlayer(SDWORD locX, SDWORD locY, SDWORD forPlayer, SDWORD sender, char * beaconMsg);
 extern MESSAGE * findBeaconMsg(UDWORD player, SDWORD sender);
-extern SDWORD getNumRepairedBy(DROID *psDroidToCheck, SDWORD player);
-extern BOOL objectInRangeVis(BASE_OBJECT *psList, SDWORD x, SDWORD y, SDWORD range, SDWORD lookingPlayer);
+extern SDWORD getNumRepairedBy(struct DROID *psDroidToCheck, SDWORD player);
+extern BOOL objectInRangeVis(struct BASE_OBJECT *psList, SDWORD x, SDWORD y, SDWORD range, SDWORD lookingPlayer);
 extern SDWORD getPlayerFromString(char *playerName);
 extern BOOL scrExp(void);
 extern BOOL scrSqrt(void);
