@@ -1,7 +1,7 @@
 /*
 	This file is part of Warzone 2100.
 	Copyright (C) 1999-2004  Eidos Interactive
-	Copyright (C) 2005-2007  Warzone Resurrection Project
+	Copyright (C) 2005-2009  Warzone Resurrection Project
 
 	Warzone 2100 is free software; you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -26,6 +26,7 @@
 #include <string.h>
 
 #include "lib/framework/frame.h"
+#include "lib/framework/endian_hack.h"
 #include "lib/framework/strres.h"
 #include "objects.h"
 
@@ -48,6 +49,9 @@
 #include "scriptvals.h"
 #include "research.h"
 
+// Marks a NULL pointer for the script value save/load routines
+static const int UNALLOCATED_OBJECT = -1;
+
 static INTERP_VAL	scrFunctionResult;	//function return value to be pushed to stack
 
 // Get values from a base object
@@ -58,7 +62,6 @@ BOOL scrBaseObjGet(UDWORD index)
 	DROID			*psDroid;
 	STRUCTURE		*psStruct;
 	FEATURE			*psFeature;
-	SDWORD			temp;
 
 	if (!stackPopParams(1, ST_BASEOBJECT, &psObj))
 	{
@@ -314,229 +317,7 @@ BOOL scrBaseObjGet(UDWORD index)
 		type = (INTERP_TYPE)ST_GROUP;
 		scrFunctionResult.v.oval = ((DROID *)psObj)->psGroup;
 		break;
-	case OBJID_WEAP_SHORT_RANGE:
 
-		type = VAL_INT;
-		temp = -1;		// in case object has no weapon return -1
-
-		if (psObj->type == OBJ_STRUCTURE)
-		{
-			if(((STRUCTURE *)psObj)->asWeaps[0].nStat > 0)
-			{
-				temp = asWeaponStats[((STRUCTURE *)psObj)->asWeaps[0].nStat].shortRange;
-			}
-		}
-		else if (psObj->type == OBJ_DROID)
-		{
-			if(((DROID *)psObj)->asWeaps[0].nStat > 0)
-			{
-				temp = asWeaponStats[((DROID *)psObj)->asWeaps[0].nStat].shortRange;
-			}
-		}
-		else
-		{
-			debug(LOG_ERROR, "scrBaseObjGet(): shortRange: features don't have weapons");
-			return false;
-		}
-
-		scrFunctionResult.v.ival = temp;
-
-		break;
-	case OBJID_WEAP_LONG_RANGE:
-
-		type = VAL_INT;
-		temp = -1;		// in case object has no weapon return -1
-
-		if (psObj->type == OBJ_STRUCTURE)
-		{
-			if(((STRUCTURE *)psObj)->asWeaps[0].nStat > 0)
-			{
-				temp = asWeaponStats[((STRUCTURE *)psObj)->asWeaps[0].nStat].longRange;
-			}
-		}
-		else if (psObj->type == OBJ_DROID)
-		{
-			if(((DROID *)psObj)->asWeaps[0].nStat > 0)
-			{
-				temp = asWeaponStats[((DROID *)psObj)->asWeaps[0].nStat].longRange;
-			}
-		}
-		else
-		{
-			debug(LOG_ERROR, "scrBaseObjGet(): shortRange: features don't have weapons");
-			return false;
-		}
-
-		scrFunctionResult.v.ival = temp;
-
-		break;
-	case OBJID_WEAP_SHORT_HIT:
-		type = VAL_INT;
-		temp = -1;		// in case object has no weapon return -1
-
-		if (psObj->type == OBJ_STRUCTURE)
-		{
-			if(((STRUCTURE *)psObj)->asWeaps[0].nStat > 0)
-			{
-				temp = weaponShortHit(&asWeaponStats[((STRUCTURE *)psObj)->asWeaps[0].nStat], psObj->player);
-			}
-		}
-		else if (psObj->type == OBJ_DROID)
-		{
-			if(((DROID *)psObj)->asWeaps[0].nStat > 0)
-			{
-				temp = weaponShortHit(&asWeaponStats[((DROID *)psObj)->asWeaps[0].nStat], psObj->player);
-			}
-		}
-		else
-		{
-			debug(LOG_ERROR, "scrBaseObjGet(): shortHit: features don't have weapons");
-			return false;
-		}
-
-		scrFunctionResult.v.ival = temp;
-
-		break;
-	case OBJID_WEAP_LONG_HIT:
-
-		type = VAL_INT;
-		temp = -1;		// in case object has no weapon return -1
-
-		if (psObj->type == OBJ_STRUCTURE)
-		{
-			if(((STRUCTURE *)psObj)->asWeaps[0].nStat > 0)
-			{
-				temp = weaponLongHit(&asWeaponStats[((STRUCTURE *)psObj)->asWeaps[0].nStat], psObj->player);
-			}
-		}
-		else if (psObj->type == OBJ_DROID)
-		{
-			if(((DROID *)psObj)->asWeaps[0].nStat > 0)
-			{
-				temp = weaponLongHit(&asWeaponStats[((DROID *)psObj)->asWeaps[0].nStat], psObj->player);
-			}
-		}
-		else
-		{
-			debug(LOG_ERROR, "scrBaseObjGet(): shortHit: features don't have weapons");
-			return false;
-		}
-
-		scrFunctionResult.v.ival = temp;
-
-		break;
-	case OBJID_WEAP_DAMAGE:
-
-		type = VAL_INT;
-		temp = -1;		// in case object has no weapon return -1
-
-		if (psObj->type == OBJ_STRUCTURE)
-		{
-			if(((STRUCTURE *)psObj)->asWeaps[0].nStat > 0)
-			{
-				temp = weaponDamage(&asWeaponStats[((STRUCTURE *)psObj)->asWeaps[0].nStat], psObj->player);
-			}
-		}
-		else if (psObj->type == OBJ_DROID)
-		{
-			if(((DROID *)psObj)->asWeaps[0].nStat > 0)
-			{
-				temp = weaponDamage(&asWeaponStats[((DROID *)psObj)->asWeaps[0].nStat], psObj->player);
-			}
-		}
-		else
-		{
-			debug(LOG_ERROR, "scrBaseObjGet(): shortHit: features don't have weapons");
-			return false;
-		}
-
-		scrFunctionResult.v.ival = temp;
-
-		break;
-	case OBJID_WEAP_FIRE_PAUSE:
-
-		type = VAL_INT;
-		temp = -1;		// in case object has no weapon return -1
-
-		if (psObj->type == OBJ_STRUCTURE)
-		{
-			if(((STRUCTURE *)psObj)->asWeaps[0].nStat > 0)
-			{
-				temp = weaponFirePause(&asWeaponStats[((STRUCTURE *)psObj)->asWeaps[0].nStat], psObj->player);
-			}
-		}
-		else if (psObj->type == OBJ_DROID)
-		{
-			if(((DROID *)psObj)->asWeaps[0].nStat > 0)
-			{
-				temp = weaponFirePause(&asWeaponStats[((DROID *)psObj)->asWeaps[0].nStat], psObj->player);
-			}
-		}
-		else
-		{
-			debug(LOG_ERROR, "scrBaseObjGet(): shortHit: features don't have weapons");
-			return false;
-		}
-
-		scrFunctionResult.v.ival = temp;
-
-		break;
-	case OBJID_WEAP_RELOAD_TIME:
-
-		type = VAL_INT;
-		temp = -1;		// in case object has no weapon return -1
-
-		if (psObj->type == OBJ_STRUCTURE)
-		{
-			if(((STRUCTURE *)psObj)->asWeaps[0].nStat > 0)
-			{
-				temp = asWeaponStats[((STRUCTURE *)psObj)->asWeaps[0].nStat].reloadTime;
-			}
-		}
-		else if (psObj->type == OBJ_DROID)
-		{
-			if(((DROID *)psObj)->asWeaps[0].nStat > 0)
-			{
-				temp = asWeaponStats[((DROID *)psObj)->asWeaps[0].nStat].reloadTime;
-			}
-		}
-		else
-		{
-			debug(LOG_ERROR, "scrBaseObjGet(): shortHit: features don't have weapons");
-			return false;
-		}
-
-		scrFunctionResult.v.ival = temp;
-
-		break;
-	case OBJID_WEAP_NUM_ROUNDS:
-
-		type = VAL_INT;
-		temp = -1;		// in case object has no weapon return -1
-
-		if (psObj->type == OBJ_STRUCTURE)
-		{
-			if(((STRUCTURE *)psObj)->asWeaps[0].nStat > 0)
-			{
-				temp = asWeaponStats[((STRUCTURE *)psObj)->asWeaps[0].nStat].numRounds;
-			}
-		}
-		else if (psObj->type == OBJ_DROID)
-		{
-			if(((DROID *)psObj)->asWeaps[0].nStat > 0)
-			{
-				temp = asWeaponStats[((DROID *)psObj)->asWeaps[0].nStat].numRounds;
-			}
-		}
-		else
-		{
-			debug(LOG_ERROR, "scrBaseObjGet(): shortHit: features don't have weapons");
-			return false;
-		}
-
-		scrFunctionResult.v.ival = temp;
-
-		break;
 	case OBJID_HITPOINTS:
 
 		type = VAL_INT;
@@ -686,6 +467,89 @@ BOOL scrObjToFeature(void)
 // to speed up access
 static SDWORD		lgX,lgY, lgMembers, lgHealth;
 
+// Get values from a weapon
+BOOL scrWeaponObjGet(UDWORD index)
+{
+	INTERP_TYPE		type;
+	SDWORD			weapIndex;
+
+	if (!stackPopParams(1, ST_WEAPON, &weapIndex))
+	{
+		return false;
+	}
+
+	switch (index)
+	{
+	case WEAPID_SHORT_RANGE:
+
+		type = VAL_INT;
+
+		scrFunctionResult.v.ival = asWeaponStats[weapIndex].shortRange;
+
+		break;
+	case WEAPID_LONG_RANGE:
+
+		type = VAL_INT;
+
+		scrFunctionResult.v.ival = asWeaponStats[weapIndex].longRange;
+
+		break;
+	case WEAPID_SHORT_HIT:
+		type = VAL_INT;
+
+		scrFunctionResult.v.ival = asWeaponStats[weapIndex].shortHit;
+
+		break;
+	case WEAPID_LONG_HIT:
+
+		type = VAL_INT;
+
+		scrFunctionResult.v.ival = asWeaponStats[weapIndex].longHit;
+
+		break;
+	case WEAPID_DAMAGE:
+
+		type = VAL_INT;
+
+		scrFunctionResult.v.ival = asWeaponStats[weapIndex].damage;
+
+		break;
+	case WEAPID_FIRE_PAUSE:
+
+		type = VAL_INT;
+
+		scrFunctionResult.v.ival = asWeaponStats[weapIndex].firePause;
+
+		break;
+	case WEAPID_RELOAD_TIME:
+
+		type = VAL_INT;
+
+		scrFunctionResult.v.ival = asWeaponStats[weapIndex].reloadTime;
+
+		break;
+	case WEAPID_NUM_ROUNDS:
+
+		type = VAL_INT;
+
+		scrFunctionResult.v.ival = asWeaponStats[weapIndex].numRounds;
+
+		break;
+	default:
+		ASSERT( false, "unknown variable index" );
+		return false;
+		break;
+	}
+
+	// Return the value
+	if (!stackPushResult(type, &scrFunctionResult))
+	{
+		return false;
+	}
+
+	return true;
+}
+
 // Get values from a group
 BOOL scrGroupObjGet(UDWORD index)
 {
@@ -901,9 +765,6 @@ BOOL scrValDefSave(INTERP_VAL *psVal, char *pBuffer, UDWORD *pSize)
 	RESEARCH	*psResearch;
 	char		*pPos;
 	DROID		*psCDroid;
-	SDWORD		members;
-	BOOL		bObjectDefined;
-	DROID_GROUP	*psGroup;
 #ifdef _DEBUG
 	BASE_OBJECT	*psObj;
 #endif
@@ -996,20 +857,20 @@ BOOL scrValDefSave(INTERP_VAL *psVal, char *pBuffer, UDWORD *pSize)
 		*pSize = sizeof(UDWORD);
 		break;
 	case ST_TEXTSTRING:
+	{
+		const char * const idStr = psVal->v.sval ? strresGetIDfromString(psStringRes, psVal->v.sval) : NULL;
+		uint16_t len = idStr ? strlen(idStr) + 1 : 0;
+
 		if (pBuffer)
 		{
-			if (psVal->v.sval == NULL)
-			{
-				*((UDWORD*)pBuffer) = UDWORD_MAX;
-			}
-			else
-			{
-				*((UDWORD*)pBuffer) = strresGetIDfromString(psStringRes, psVal->v.sval);
-			}
-			endian_udword((UDWORD*)pBuffer);
+			*((uint16_t*)pBuffer) = len;
+			endian_uword((uint16_t*)pBuffer);
+
+			memcpy(pBuffer + sizeof(len), idStr, len);
 		}
-		*pSize = sizeof(UDWORD);
+		*pSize = sizeof(len) + len;
 		break;
+	}
 	case ST_LEVEL:
 		if (psVal->v.sval != NULL)
 		{
@@ -1048,16 +909,9 @@ BOOL scrValDefSave(INTERP_VAL *psVal, char *pBuffer, UDWORD *pSize)
 		}
 		break;
 	case ST_GROUP:
-		bObjectDefined = (psVal->v.oval != NULL);
-
-		if (bObjectDefined)
-		{
-			members = grpNumMembers((DROID_GROUP *)psVal->v.oval);
-		}
-		else
-		{
-			members = UNALLOCATED_OBJECT;
-		}
+	{
+		DROID_GROUP* const psGroup = (DROID_GROUP *)psVal->v.oval;
+		const int members = psGroup ? grpNumMembers(psGroup) : UNALLOCATED_OBJECT;
 
 		if (pBuffer)
 		{
@@ -1067,10 +921,8 @@ BOOL scrValDefSave(INTERP_VAL *psVal, char *pBuffer, UDWORD *pSize)
 			endian_sdword((SDWORD*)pPos);
 			pPos += sizeof(SDWORD);
 
-			if(bObjectDefined)
+			if (psGroup)
 			{
-				psGroup = (DROID_GROUP *)psVal->v.oval;
-
 				// store the run data
 				*((SDWORD *)pPos) = psGroup->sRunData.sPos.x;
 				endian_sdword((SDWORD*)pPos);
@@ -1089,7 +941,7 @@ BOOL scrValDefSave(INTERP_VAL *psVal, char *pBuffer, UDWORD *pSize)
 				pPos += sizeof(SDWORD);
 
 				// now store the droids
-				for(psCDroid=((DROID_GROUP *)psVal->v.oval)->psList; psCDroid; psCDroid=psCDroid->psGrpNext)
+				for (psCDroid = psGroup->psList; psCDroid; psCDroid = psCDroid->psGrpNext)
 				{
 					checkValidId(psCDroid->id);
 
@@ -1101,7 +953,7 @@ BOOL scrValDefSave(INTERP_VAL *psVal, char *pBuffer, UDWORD *pSize)
 			}
 		}
 
-		if(!bObjectDefined)
+		if (!psGroup)
 		{
 			*pSize = sizeof(SDWORD);
 		}
@@ -1110,6 +962,7 @@ BOOL scrValDefSave(INTERP_VAL *psVal, char *pBuffer, UDWORD *pSize)
 			*pSize = sizeof(SDWORD) + sizeof(UDWORD) * members + sizeof(SDWORD) * 5;	// members + runData
 		}
 		break;
+	}
 	case ST_SOUND:
 		if(psVal->v.ival)
 		{
@@ -1321,15 +1174,110 @@ BOOL scrValDefLoad(SDWORD version, INTERP_VAL *psVal, char *pBuffer, UDWORD size
 		}
 		break;
 	case ST_TEXTSTRING:
-		id = *((UDWORD *)pBuffer);
-		endian_udword(&id);
-		if (id == UDWORD_MAX)
+		if (version < 4)
 		{
-			psVal->v.sval = '\0';
+			if (size != sizeof(UDWORD))
+			{
+				debug(LOG_ERROR, "Data size is too small, %u is expected, but %u is provided", (unsigned int)(sizeof(UDWORD)), (unsigned int)size);
+				return false;
+			}
+
+			id = *((UDWORD*)pBuffer);
+			endian_udword(&id);
+
+			if (id == UDWORD_MAX)
+			{
+				psVal->v.sval = NULL;
+			}
+			else
+			{
+				/* This code is commented out, because it depends on assigning the
+				 * id-th loaded string from the string resources. And from version
+				 * 4 of this file format onward, we do not count strings anymore.
+				 *
+				 * Thus loading of these strings is practically impossible.
+				 */
+#if 0
+				const char * const str = strresGetString(psStringRes, id);
+				if (!str)
+				{
+					debug(LOG_ERROR, "Couldn't find string with id %u", id);
+					abort();
+					return false;
+				}
+
+				psVal->v.sval = strdup(str);
+				if (!psVal->v.sval)
+				{
+					debug(LOG_ERROR, "Out of memory");
+					abort();
+					return false;
+				}
+#else
+				debug(LOG_ERROR, "Incompatible savegame format version %u, should be at least version 4", (unsigned int)version);
+				return false;
+#endif
+			}
 		}
 		else
 		{
-			psVal->v.sval = strresGetString(psStringRes, id);
+			const char* str;
+			char* idStr;
+			uint16_t len;
+
+			if (size < sizeof(len))
+			{
+				debug(LOG_ERROR, "Data size is too small, %u is expected, but %u is provided", (unsigned int)(sizeof(len)), (unsigned int)size);
+				return false;
+			}
+
+			len = *((uint16_t*)pBuffer);
+			endian_uword(&len);
+
+			if (size < sizeof(len) + len)
+			{
+				debug(LOG_ERROR, "Data size is too small, %u is expected, but %u is provided", (unsigned int)(sizeof(len) + len), (unsigned int)size);
+				return false;
+			}
+
+			if (len == 0)
+			{
+				psVal->v.sval = NULL;
+				return true;
+			}
+
+			idStr = malloc(len);
+			if (!idStr)
+			{
+				debug(LOG_ERROR, "Out of memory (tried to allocate %u bytes)", (unsigned int)len);
+				// Don't abort() here, as this might be the result from a bad "len" field in the data
+				return false;
+			}
+
+			memcpy(idStr, pBuffer + sizeof(len), len);
+
+			if (idStr[len - 1] != '\0')
+			{
+				debug(LOG_WARNING, "Non-NUL terminated string encountered!");
+			}
+			idStr[len - 1] = '\0';
+
+			str = strresGetString(psStringRes, idStr);
+			if (!str)
+			{
+				debug(LOG_ERROR, "Couldn't find string with id \"%s\"", idStr);
+				free(idStr);
+				return false;
+			}
+			free(idStr);
+
+			psVal->v.sval = strdup(str);
+			if (!psVal->v.sval)
+			{
+				debug(LOG_ERROR, "Out of memory");
+				abort();
+				return false;
+			}
 		}
 		break;
 	case ST_LEVEL:
@@ -1381,32 +1329,29 @@ BOOL scrValDefLoad(SDWORD version, INTERP_VAL *psVal, char *pBuffer, UDWORD size
 
 		pPos = pBuffer;
 
-		switch (version)
+		if (version < 2)
 		{
-			case 1:
-				members = size / sizeof(UDWORD);
-				break;
-			case 2:
-				members = (size - sizeof(SDWORD)*4) / sizeof(UDWORD);
-				break;
-			case 3:
-				members = (size - sizeof(SDWORD)*6) / sizeof(UDWORD);
+			members = size / sizeof(UDWORD);
+		}
+		else if (version < 3)
+		{
+			members = (size - sizeof(SDWORD)*4) / sizeof(UDWORD);
+		}
+		else
+		{
+			members = (size - sizeof(SDWORD)*6) / sizeof(UDWORD);
 
-				// get saved group member count/nullpointer flag
-				endian_sdword((SDWORD*)pPos);
-				bObjectDefined = ( *((SDWORD *)pPos) != UNALLOCATED_OBJECT );
+			// get saved group member count/nullpointer flag
+			endian_sdword((SDWORD*)pPos);
+			bObjectDefined = ( *((SDWORD *)pPos) != UNALLOCATED_OBJECT );
 
-				if(bObjectDefined)
-				{
-					savedMembers = *((SDWORD *)pPos);	// get number of saved group members
+			if(bObjectDefined)
+			{
+				savedMembers = *((SDWORD *)pPos);	// get number of saved group members
 
-					ASSERT(savedMembers == members, "scrValDefLoad: calculated and saved group member count did not match." );
-				}
-				pPos += sizeof(SDWORD);
-				break;
-			default:
-				members = 0;
-				debug( LOG_ERROR, "scrValDefLoad: unsupported version %i", version);
+				ASSERT(savedMembers == members, "scrValDefLoad: calculated and saved group member count did not match." );
+			}
+			pPos += sizeof(SDWORD);
 		}
 
 		// make sure group was allocated when it was saved (relevant starting from version 3)
