@@ -1,7 +1,7 @@
 /*
 	This file is part of Warzone 2100.
 	Copyright (C) 1999-2004  Eidos Interactive
-	Copyright (C) 2005-2007  Warzone Resurrection Project
+	Copyright (C) 2005-2009  Warzone Resurrection Project
 
 	Warzone 2100 is free software; you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -24,6 +24,8 @@
 #ifndef __INCLUDED_SRC_STRUCTURE_H__
 #define __INCLUDED_SRC_STRUCTURE_H__
 
+#include "lib/framework/string_ext.h"
+
 #include "objectdef.h"
 #include "structuredef.h"
 #include "visibility.h"
@@ -205,6 +207,8 @@ extern BOOL placeDroid(STRUCTURE *psStructure, UDWORD *droidX, UDWORD *droidY);
 /* is this a lassat structure? */
 static inline BOOL isLasSat(STRUCTURE_STATS *pStructureType)
 {
+	ASSERT_OR_RETURN(false, pStructureType != NULL, "LasSat is invalid?");
+
 	return (pStructureType->psWeapStat[0]
 	        && pStructureType->psWeapStat[0]->weaponSubClass == WSC_LAS_SAT);
 }
@@ -223,7 +227,7 @@ extern void setLasSatExists(BOOL state, UDWORD player);
 extern BOOL getLasSatExists(UDWORD player);
 
 /* added int weapon_slot to fix the alway slot 0 hack */
-extern BOOL calcStructureMuzzleLocation(STRUCTURE *psStructure, Vector3i *muzzle, int weapon_slot);
+extern BOOL calcStructureMuzzleLocation(STRUCTURE *psStructure, Vector3f *muzzle, int weapon_slot);
 
 /*this is called whenever a structure has finished building*/
 extern void buildingComplete(STRUCTURE *psBuilding);
@@ -233,13 +237,13 @@ extern void checkForPowerGen(STRUCTURE *psPowerGen);
 
 // Set the command droid that factory production should go to
 //struct _command_droid;
-extern void assignFactoryCommandDroid(STRUCTURE *psStruct, struct _droid *psCommander);
+extern void assignFactoryCommandDroid(STRUCTURE *psStruct, struct DROID *psCommander);
 
 // remove all factories from a command droid
 void clearCommandDroidFactory(DROID *psDroid);
 
 /*for a given structure, return a pointer to its module stat */
-extern STRUCTURE_STATS* getModuleStat(STRUCTURE *psStruct);
+extern STRUCTURE_STATS* getModuleStat(const STRUCTURE* psStruct);
 
 /*called when a Res extractor is destroyed or runs out of power or is disconnected
 adjusts the owning Power Gen so that it can link to a different Res Extractor if one
@@ -386,15 +390,13 @@ all StructureStats parts have been loaded*/
 extern BOOL checkStructureStats(void);
 
 /*returns the power cost to build this structure*/
-extern UDWORD  structPowerToBuild(STRUCTURE *psStruct);
+extern UDWORD structPowerToBuild(const STRUCTURE* psStruct);
 
 extern UDWORD getMaxDroids(UDWORD PlayerNumber);
 
 // check whether a factory of a certain number and type exists
 extern BOOL checkFactoryExists(UDWORD player, UDWORD factoryType, UDWORD inc);
 
-extern BOOL	ptInStructure(STRUCTURE *psStruct, UDWORD x, UDWORD y);
-
 /*checks the structure passed in is a Las Sat structure which is currently
 selected - returns true if valid*/
 extern BOOL lasSatStructSelected(STRUCTURE *psStruct);
@@ -435,23 +437,16 @@ static inline void _setStructureTarget(STRUCTURE *psBuilding, BASE_OBJECT *psNew
 #ifdef DEBUG
 	psBuilding->targetLine[idx] = line;
 	sstrcpy(psBuilding->targetFunc[idx], func);
+#else
+	// Prevent warnings about unused parameters
+	(void)line;
+	(void)func;
 #endif
 }
 
-#define CHECK_STRUCTURE(object) \
-do { \
-	unsigned int i; \
-\
-	ASSERT((object) != NULL, "CHECK_STRUCTURE: NULL pointer"); \
-	ASSERT((object)->type == OBJ_STRUCTURE, "CHECK_STRUCTURE: No structure (type num %u)", (unsigned int)(object)->type); \
-	ASSERT((object)->player < MAX_PLAYERS, "CHECK_STRUCTURE: Out of bound player num (%u)", (unsigned int)(object)->player); \
-	ASSERT((object)->pStructureType->type < NUM_DIFF_BUILDINGS, "CHECK_STRUCTURE: Out of bound structure type (%u)", (unsigned int)(object)->pStructureType->type); \
-	ASSERT((object)->numWeaps <= STRUCT_MAXWEAPS, "CHECK_STRUCTURE: Out of bound weapon count (%u)", (unsigned int)(object)->numWeaps); \
-	for (i = 0; i < STRUCT_MAXWEAPS; ++i) \
-	{ \
-		ASSERT((object)->turretRotation[i] <= 360, "Out of range turret rotation (turret %u; rotation: %u)", i, (unsigned int)(object)->turretRotation[i]); \
-	} \
-} while (0)
+void checkStructure(const STRUCTURE* psStructure, const char * const location_description, const char * function, const int recurse);
+
+#define CHECK_STRUCTURE(object) checkStructure((object), AT_MACRO, __FUNCTION__, max_check_object_recursion)
 
 extern void     structureInitVars(void);
 
